/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*	queue.c

	Implementation of a FIFO queue abstract data type.

	by: Steven Skiena
	begun: March 27, 2002
*/


/*
Copyright 2003 by Steven S. Skiena; all rights reserved. 

Permission is granted for use in non-commerical applications
provided this copyright notice remains intact and unchanged.

This program appears in my book:

"Programming Challenges: The Programming Contest Training Manual"
by Steven Skiena and Miguel Revilla, Springer-Verlag, New York 2003.

See our website www.programming-challenges.com for additional information.

This book can be ordered from Amazon.com at

http://www.amazon.com/exec/obidos/ASIN/0387001638/thealgorithmrepo/

*/

#include <stdio.h>
#include "queue.h"



void queueInit(QUEUE *q, char* name)
{
        q->first = 0;
        q->last = QUEUESIZE-1;
        q->count = 0;
	q->name = name;
}

void queueAdd(QUEUE *q, SEGMENT* sn)
{
        if (q->count >= QUEUESIZE)
		printf("Warning: queue overflow enqueue\n");
        else {
                q->last = (q->last+1) % QUEUESIZE;
                (q->s[q->last]).type = sn->type; 
                (q->s[q->last]).time = sn->time; 
		(q->s[q->last]).J = sn->J;
                (q->s[q->last]).A0 = sn->A0;
                (q->s[q->last]).V0 = sn->V0;
                (q->s[q->last]).X0 = sn->X0;
                (q->s[q->last]).dir = sn->dir;   
                q->count = q->count + 1;
        }
}

void queueAddDetails(QUEUE *q, int type, double time, double J, double A0, double V0, double X0, int Dir)
{
  if (time < 1.0) { // 1.0 tick
    return;
  } else {
    if (q->count >= QUEUESIZE)
      printf("Warning: queue overflow enqueue\n");
    else {
      q->last = (q->last+1) % QUEUESIZE;
      (q->s[q->last]).type = type; 
      (q->s[q->last]).time = time;
      (q->s[q->last]).J = J;
      (q->s[q->last]).A0 = A0;
      (q->s[q->last]).V0 = V0;
      (q->s[q->last]).X0 = X0;
      (q->s[q->last]).dir = Dir;    
      q->count = q->count + 1;
    }
  }
}

int queueGetAndRemove(QUEUE *q, SEGMENT* x )
{
   //     SEGMENT x;

  if (q->count <= 0) {
    printf("Warning: empty queue dequeue.\n");
    return 1;
  }
  else {
    x->type = q->s[ q->first ].type;
    x->time = q->s[ q->first ].time;
    x->J = q->s[ q->first ].J;
    x->A0 = q->s[ q->first ].A0;
    x->V0 = q->s[ q->first ].V0;
    x->X0 = q->s[ q->first ].X0;
    x->dir = q->s[ q->first ].dir;
    q->first = (q->first+1) % QUEUESIZE;
    q->count = q->count - 1;
  }
  
  return 0;
}

int queueVerifyIfEmpty(QUEUE *q)
{
        if (q->count <= 0) return 1;
        else return 0;
}

void queuePrint(QUEUE *q)
{
        int i;
	if (q->count  <= 0) return;
        i=q->first; 
	printf("Name of the queue: %s\n",q->name);       
        while (i <= q->last) {
	  printf("%d) Type: %d Time: %1.3f Jmax: %1.3f A0: %1.3f V0: %1.3f X0: %1.3f  Dir: %d\n",i, q->s[i].type, q->s[i].time, q->s[i].J, q->s[i].A0, q->s[i].V0, q->s[i].X0, q->s[i].dir);
          i = (i+1) % QUEUESIZE;
        }  
}
