/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*--------------------------------------------------------------------
                         -- C N R S --
         Laboratoire d'automatique et d'analyse des systemes
            Groupe Robotique et Intelligenxce Artificielle
                   7 Avenue du colonel Roche
                     31 077 Toulouse Cedex

  Fichier              : xarmFunctions.c
  Fonction             : Constants definition for xarm
  Date de creation     : Aout 2005
  Date de modification : Aout 2005
  Nb de lignes         : 

  Auteur               : Ignacio HERRERA AGUILAR
  Modification (Re-ecriture)        : Xavier BROQUERE (Ocobre 2007)

----------------------------------------------------------------------*/

#include "xarmFunctions.h"
#include "cFunctions.c"
#include "queue.h"
#include "tpFunctions.h"
#include "xarmJerkprofile.h"

static Gb_6rParameters pa10ParametersStruct;
static Gb_th xarmR6RtStruct;
Gb_6rParameters* pa10Parameters = 0;
Gb_th* xarmR6Rt = 0;  /* R6: pa10 wrist frame; Rt: tool frame */

STATUS InitParamsPa10Control(XARM_PI_CONTROL_GAIN_STR* piGain, TP_POSELIMITS *limitsGoto, XARM_POSE_CONTROL_STR *pa10Control)
{
  pa10Control->PX.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->PX.Kp     =  piGain->Linear_Kp;
  pa10Control->PX.Ti     =  piGain->Linear_Ti;
  pa10Control->PX.AccMax =  limitsGoto->linear.maxAcc;
  pa10Control->PX.VelMax =  limitsGoto->linear.maxVel;
                         
  pa10Control->PY.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->PY.Kp     =  piGain->Linear_Kp;
  pa10Control->PY.Ti     =  piGain->Linear_Ti;
  pa10Control->PY.AccMax =  limitsGoto->linear.maxAcc;
  pa10Control->PY.VelMax =  limitsGoto->linear.maxVel;
                         
  pa10Control->PZ.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->PZ.Kp     =  piGain->Linear_Kp;
  pa10Control->PZ.Ti     =  piGain->Linear_Ti;
  pa10Control->PZ.AccMax =  limitsGoto->linear.maxAcc;
  pa10Control->PZ.VelMax =  limitsGoto->linear.maxVel;
                         
  pa10Control->TX.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->TX.Kp     =  piGain->Angular_Kp;
  pa10Control->TX.Ti     =  piGain->Angular_Ti; 
  pa10Control->TX.AccMax =  limitsGoto->angular.maxAcc;
  pa10Control->TX.VelMax =  limitsGoto->angular.maxVel;
                         
  pa10Control->TY.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->TY.Kp     =  piGain->Angular_Kp;
  pa10Control->TY.Ti     =  piGain->Angular_Ti; 
  pa10Control->TY.AccMax =  limitsGoto->angular.maxAcc;
  pa10Control->TY.VelMax =  limitsGoto->angular.maxVel;
                         
  pa10Control->TZ.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->TZ.Kp     =  piGain->Angular_Kp;
  pa10Control->TZ.Ti     =  piGain->Angular_Ti; 
  pa10Control->TZ.AccMax =  limitsGoto->angular.maxAcc;
  pa10Control->TZ.VelMax =  limitsGoto->angular.maxVel;

  return OK;
}

STATUS InitParamsPa10ControlC1(XARM_PI_CONTROL_GAIN_STR* piGain, XARM_POSE_CONTROL_STR *pa10Control)
{
  pa10Control->PX.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->PX.Kp     =  piGain->Linear_Kp;
  pa10Control->PX.Ti     =  piGain->Linear_Ti;
  pa10Control->PX.AccMax =  C1_MAX_LINEAR_ACCELERATION;
  pa10Control->PX.VelMax =  C1_MAX_LINEAR_VELOCITY;
                         
  pa10Control->PY.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->PY.Kp     =  piGain->Linear_Kp;
  pa10Control->PY.Ti     =  piGain->Linear_Ti;
  pa10Control->PY.AccMax =  C1_MAX_LINEAR_ACCELERATION;
  pa10Control->PY.VelMax =  C1_MAX_LINEAR_VELOCITY;
                         
  pa10Control->PZ.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->PZ.Kp     =  piGain->Linear_Kp;
  pa10Control->PZ.Ti     =  piGain->Linear_Ti;
  pa10Control->PZ.AccMax =  C1_MAX_LINEAR_ACCELERATION;
  pa10Control->PZ.VelMax =  C1_MAX_LINEAR_VELOCITY;
                         
  pa10Control->TX.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->TX.Kp     =  piGain->Angular_Kp;
  pa10Control->TX.Ti     =  piGain->Angular_Ti; 
  pa10Control->TX.AccMax =  C1_MAX_ANGULAR_ACCELERATION;
  pa10Control->TX.VelMax =  C1_MAX_ANGULAR_VELOCITY;
                         
  pa10Control->TY.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->TY.Kp     =  piGain->Angular_Kp;
  pa10Control->TY.Ti     =  piGain->Angular_Ti; 
  pa10Control->TY.AccMax =  C1_MAX_ANGULAR_ACCELERATION;
  pa10Control->TY.VelMax =  C1_MAX_ANGULAR_VELOCITY;
                         
  pa10Control->TZ.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->TZ.Kp     =  piGain->Angular_Kp;
  pa10Control->TZ.Ti     =  piGain->Angular_Ti; 
  pa10Control->TZ.AccMax =  C1_MAX_ANGULAR_ACCELERATION;
  pa10Control->TZ.VelMax =  C1_MAX_ANGULAR_VELOCITY;

  return OK;
}


STATUS InitParamsPa10ControlC2( XARM_POSE_CONTROL_STR *pa10Control )
{
  pa10Control->PX.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->PX.Kp     =  C2_XARM_KP_LINEAR;
  pa10Control->PX.Ti     =  C2_XARM_TI_LINEAR;
  pa10Control->PX.AccMax =  C2_MAX_LINEAR_ACCELERATION;
  pa10Control->PX.VelMax =  C2_MAX_LINEAR_VELOCITY;

  pa10Control->PY.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->PY.Kp     =  C2_XARM_KP_LINEAR;
  pa10Control->PY.Ti     =  C2_XARM_TI_LINEAR;
  pa10Control->PY.AccMax =  C2_MAX_LINEAR_ACCELERATION;
  pa10Control->PY.VelMax =  C2_MAX_LINEAR_VELOCITY;

  pa10Control->PZ.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->PZ.Kp     =  C2_XARM_KP_LINEAR;
  pa10Control->PZ.Ti     =  C2_XARM_TI_LINEAR;
  pa10Control->PZ.AccMax =  C2_MAX_LINEAR_ACCELERATION;
  pa10Control->PZ.VelMax =  C2_MAX_LINEAR_VELOCITY;

  pa10Control->TX.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->TX.Kp     =  C2_XARM_KP_ANGULAR;
  pa10Control->TX.Ti     =  C2_XARM_TI_ANGULAR;
  pa10Control->TX.AccMax =  C2_MAX_ANGULAR_ACCELERATION;
  pa10Control->TX.VelMax =  C2_MAX_ANGULAR_VELOCITY;

  pa10Control->TY.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->TY.Kp     =  C2_XARM_KP_ANGULAR;
  pa10Control->TY.Ti     =  C2_XARM_TI_ANGULAR;
  pa10Control->TY.AccMax =  C2_MAX_ANGULAR_ACCELERATION;
  pa10Control->TY.VelMax =  C2_MAX_ANGULAR_VELOCITY;

  pa10Control->TZ.Ts     =  XARM_SAMPLING_TIME;
  pa10Control->TZ.Kp     =  C2_XARM_KP_LINEAR;
  pa10Control->TZ.Ti     =  C2_XARM_TI_LINEAR;
  pa10Control->TZ.AccMax =  C2_MAX_LINEAR_ACCELERATION;
  pa10Control->TZ.VelMax =  C2_MAX_LINEAR_VELOCITY;

  return OK;
}

STATUS InitControlsPa10(XARM_POSE_CONTROL_STR *pa10Control) 
{
  pa10Control->PX.Error_1 = 0;
  pa10Control->PX.Uk      = 0;

  pa10Control->PY.Error_1 = 0;
  pa10Control->PY.Uk      = 0;

  pa10Control->PZ.Error_1 = 0;
  pa10Control->PZ.Uk      = 0;

  pa10Control->TX.Error_1 = 0;
  pa10Control->TX.Uk      = 0;

  pa10Control->TY.Error_1 = 0;
  pa10Control->TY.Uk      = 0;

  pa10Control->TZ.Error_1 = 0;
  pa10Control->TZ.Uk      = 0;

  return OK;
} 
STATUS CalculLawOfControl(XARM_DIM_CONTROL_STR *dimControl, double ErrorDim)
{
  double Ts     = dimControl->Ts; 
  double Kp     = dimControl->Kp; 
  double Ti     = dimControl->Ti; 
  double AccMax = dimControl->AccMax * 1.1;
  double VelMax = dimControl->VelMax * 1.1;
  double E_1    = dimControl->Error_1; 
  double Uk     = 0;
  double Uk_1   = dimControl->Uk;
  double DU     = 0;
  double Ki     = 0;

  if (dimControl->Ts <= 0) { 
    dimControl->Uk = 0;
    dimControl->Error_1= 0;
    PRINT_ERROR(" SAMPLING TIME ERROR ");
    return ERROR;
  }

  if (dimControl->Ti == 0) { 
    Ti = 10000;
    PRINT_ERROR(" INTEGRAL TIME ERROR "); 
  }
  
  /* Gain Integral Calcul */ 
  Ki = Ts / Ti; 

  /* Delta_U calcul */ 
  DU = Kp * ( ErrorDim - E_1 + Ki*ErrorDim );
  if(isnan(DU)){
    PRINT_ERROR(" DU is NAN ");
    printf("ErrorDim %g  E_1 %g Ki %g  Kp  %g\n",  ErrorDim, E_1, Ki, Kp);
    return ERROR;
  }
  
  /* Delta_u limits -> AccelerationLimits */ 
  if (DU >  AccMax) DU =  AccMax*Ts; 

  if (DU < -AccMax) DU = -AccMax*Ts;

  /* Law of Control */
  Uk = Uk_1 + DU;

  /* Limitation of speed */
  if (Uk >=  VelMax) Uk =  VelMax;

  if (Uk <= -VelMax) Uk = -VelMax;

  if(isnan(Uk)){
    dimControl->Uk = Uk;
    PRINT_ERROR(" Uk is NAN ");
    return ERROR;
  }

  /* Copy to Structure */
  dimControl->Uk   = Uk;
  dimControl->Error_1  = ErrorDim;

  return OK;
}

void initPa10Parameters()
{
  Gb_dep dep1;
  Gb_dep dep2;
  Gb_th th1;
  Gb_th th2;

  pa10ParametersStruct.a2 = 0.45; 
  pa10ParametersStruct.r4 = 0.48;
  pa10ParametersStruct.epsilon = 0.01; /* to determine if the position
					  is singular */
  pa10ParametersStruct.of1 = M_PI;
  pa10ParametersStruct.of2 = -M_PI / 2.;
  pa10ParametersStruct.of3 = -M_PI / 2.;
  pa10ParametersStruct.of4 = 0;
  pa10ParametersStruct.of5 = 0;
  pa10ParametersStruct.of6 = 0;

  pa10Parameters = &pa10ParametersStruct;
  xarmR6Rt = &xarmR6RtStruct;

  dep1.x = 0;
  dep1.y = 0;
  dep1.z = XARM_6ARM_LENGTH + XARM_TOOL_LENGTH;
  dep1.rx = 0;
  dep1.ry = 1;
  dep1.rz = 0;
  dep1.a = -M_PI / 2;
  dep2.x = 0;
  dep2.y = 0;
  dep2.z = 0;
  dep2.rx = 1;
  dep2.ry = 0;
  dep2.rz = 0;
  dep2.a = -M_PI / 8;
  Gb_dep_th(&dep1, &th1);
  Gb_dep_th(&dep2, &th2);
  Gb_th_produit(&th1, &th2, xarmR6Rt);
}  

int xarmTh06_Tool(Gb_th* th06, Gb_th* thTool, TP_POSE* poseTool)
{
  TP_QUAT GBQuatL;

  Gb_th_produit(th06, xarmR6Rt, thTool);
  /* daniel: est-il utile de normer le quaternion ici ? */
  if (RotMat2Quat(thTool, &GBQuatL) != OK) {
    PRINT_ERROR("Pb - xarmTh06_Tool");
    return ERROR;
  }
  poseTool->p.x = thTool->vp.x;
  poseTool->p.y = thTool->vp.y;
  poseTool->p.z = thTool->vp.z;
  poseTool->q.n = GBQuatL.n;
  poseTool->q.i = GBQuatL.i;
  poseTool->q.j = GBQuatL.j;
  poseTool->q.k = GBQuatL.k;
  return OK;
}

int xarmMGDQuat(Gb_q6* q6, TP_POSE* pose)
{
  Gb_q6 gbq6;
  Gb_dataMGD data;
  Gb_th th;
  Gb_th thTool;
  int status;

  gbq6.q1 = q6->q1;
  gbq6.q2 = q6->q2;
  gbq6.q3 = q6->q3;
  gbq6.q4 = q6->q4;
  gbq6.q5 = q6->q5;
  gbq6.q6 = q6->q6;

  Gb_MGD6rTh(pa10Parameters, &gbq6, &data, &th);
  status = xarmTh06_Tool(&th, &thTool, pose);
  if (status != OK) {
    PRINT_ERROR("Pb xarmMGDQuat");
    return ERROR;
  }
  return OK;
}

XARM_STATUS xarmPrintMotion(TP_MOTION* motion)
{
  int i;
  for(i=0;i<=XARM_NB_DIM-1;i++) {
    printf("IC %f %f %f      FC  %f %f %f \n ",motion->IC[i].a, motion->IC[i].v, motion->IC[i].x, motion->FC[i].a, motion->FC[i].v, motion->FC[i].x);
    printf("motion->Pos[i].Tjpa %f\n", motion->Pos[i].Tjpa);
    printf("motion->Pos[i].Taca %f\n", motion->Pos[i].Taca);
    printf("motion->Pos[i].Tjna %f\n", motion->Pos[i].Tjna);
    printf("motion->Pos[i].Tvc %f\n",  motion->Pos[i].Tvc);
    printf("motion->Pos[i].Tjnb %f\n", motion->Pos[i].Tjnb);
    printf("motion->Pos[i].Tacb %f\n", motion->Pos[i].Tacb); 
    printf("motion->Pos[i].Tjpb %f\n", motion->Pos[i].Tjpb);  
   }
   printf("motion->TNE.Tjpa %f\n",motion->TNE.Tjpa);
   printf("motion->TNE.Taca %f\n",motion->TNE.Taca);
   printf("motion->TNE.Tjna %f\n",motion->TNE.Tjna);
   printf("motion->TNE.Tvc %f\n", motion->TNE.Tvc);
   printf("motion->TNE.Tjnb %f\n",motion->TNE.Tjnb);
   printf("motion->TNE.Tacb %f\n",motion->TNE.Tacb);
   printf("motion->TNE.Tjpb %f\n",motion->TNE.Tjpb);
   return XARM_OK;
}

XARM_STATUS InitxarmR6Rt(double* r0,double* r6,double* r7,XARM_THETA_STR* r8z) 
{
  double a8z;
  
  /* xarmR6Rt : R6= pa10 wrist frame; Rt tool frame */

  a8z = 0.3925; /* M_PI / 8 */
  r8z->nx =  0;
  r8z->ny =  0;
  r8z->nz =  1;
  r8z->ox =  sin(a8z);
  r8z->oy =  cos(a8z);
  r8z->oz =  0;
  r8z->ax =  -cos(a8z);
  r8z->ay =  sin(a8z);
  r8z->az =  0;
  r8z->px =  0;
  r8z->py =  0;
  r8z->pz =  0;  
  *r0 = XARM_BASE_HEIGHT;
  *r6 = XARM_6ARM_LENGTH;
  *r7 = XARM_TOOL_LENGTH;

  return XARM_OK;
}

XARM_STATUS CalculOfJacobianMatrix(dmat Jac, Gb_jac Jac06, double r6, double r7) 
{
 /* Jacobian Matrix Calcul */
  Jac.el[0][0] = Jac06.c1.x + (r6+r7)*Jac06.c1.ry;
  Jac.el[1][0] = Jac06.c1.y - (r6+r7)*Jac06.c1.rx;
  Jac.el[2][0] = Jac06.c1.z;
  Jac.el[3][0] = Jac06.c1.rx;
  Jac.el[4][0] = Jac06.c1.ry;
  Jac.el[5][0] = Jac06.c1.rz;
  
  Jac.el[0][1] = Jac06.c2.x + (r6+r7)*Jac06.c2.ry;
  Jac.el[1][1] = Jac06.c2.y - (r6+r7)*Jac06.c2.rx;
  Jac.el[2][1] = Jac06.c2.z;
  Jac.el[3][1] = Jac06.c2.rx;
  Jac.el[4][1] = Jac06.c2.ry;
  Jac.el[5][1] = Jac06.c2.rz;
  
  Jac.el[0][2] = Jac06.c3.x + (r6+r7)*Jac06.c3.ry;
  Jac.el[1][2] = Jac06.c3.y - (r6+r7)*Jac06.c3.rx;
  Jac.el[2][2] = Jac06.c3.z;
  Jac.el[3][2] = Jac06.c3.rx;
  Jac.el[4][2] = Jac06.c3.ry;
  Jac.el[5][2] = Jac06.c3.rz;

  Jac.el[0][3] = Jac06.c4.x + (r6+r7)*Jac06.c4.ry;
  Jac.el[1][3] = Jac06.c4.y - (r6+r7)*Jac06.c4.rx;
  Jac.el[2][3] = Jac06.c4.z;
  Jac.el[3][3] = Jac06.c4.rx;
  Jac.el[4][3] = Jac06.c4.ry;
  Jac.el[5][3] = Jac06.c4.rz;

  Jac.el[0][4] = Jac06.c5.x + (r6+r7)*Jac06.c5.ry;
  Jac.el[1][4] = Jac06.c5.y - (r6+r7)*Jac06.c5.rx;
  Jac.el[2][4] = Jac06.c5.z;
  Jac.el[3][4] = Jac06.c5.rx;
  Jac.el[4][4] = Jac06.c5.ry;
  Jac.el[5][4] = Jac06.c5.rz;

  Jac.el[0][5] = Jac06.c6.x + (r6+r7)*Jac06.c6.ry;
  Jac.el[1][5] = Jac06.c6.y - (r6+r7)*Jac06.c6.rx;
  Jac.el[2][5] = Jac06.c6.z;
  Jac.el[3][5] = Jac06.c6.rx;
  Jac.el[4][5] = Jac06.c6.ry;
  Jac.el[5][5] = Jac06.c6.rz;
  
  return XARM_OK;
}

//extern QARM_POSITION_STR qarmPosition;           /* Structure for poster Position Qarm reading */

XARM_STATUS xarmTestSingularity(QARM_POSITION_STR* qarmPosition, TP_POSE* pose)
{
  Gb_6rParameters pa10;
  int e1, e2, e3;
  Gb_th th;
  Gb_q6 qPose;
  Gb_dataMGD qData;
  Gb_quat Gb_pose;

  if (SDI_F->singularity == XARM_SINGULARITY_CROSS) return XARM_OK;

  if (pa10.a2 != QARM_ARM_A2) {
    pa10.a2 =  QARM_ARM_A2;
    pa10.r4 =  QARM_ARM_R4;
    pa10.of1 = QARM_ARM_OF1;
    pa10.of2 = QARM_ARM_OF2;
    pa10.of3 = QARM_ARM_OF3;
    pa10.of4 = QARM_ARM_OF4;
    pa10.of5 = QARM_ARM_OF5;
    pa10.of6 = QARM_ARM_OF6;
    pa10.epsilon = QARM_ARM_EPSILON;
  }
  Gb_MGD6r_gete1e2e3(&pa10, &(qarmPosition->q), &e1, &e2, &e3);
  printf("e1 %d e2 %d e3 %d\n",e1,e2,e3);
//  Gb_statusMGI ret = MGI_OK;
  /* Transform the quaternion to the Gb quaterniom type */
  Gb_pose.x =  pose->p.x;
  Gb_pose.y =  pose->p.y;
  Gb_pose.z =  pose->p.z;
  Gb_pose.w =  pose->q.n; 
  Gb_pose.vx = pose->q.i; 
  Gb_pose.vy = pose->q.j; 
  Gb_pose.vz = pose->q.k;
 
  /* Gb quaternion to matrix rotation */
  Gb_quat_th(&Gb_pose, &th);

  if (Gb_MGI6rTh(&pa10, &th, e1, e2, e3, &(qarmPosition->q), &qData, &qPose) != MGI_OK) {
    PRINT_ERROR("No solution without crossing singularity\n");
    return XARM_ERROR;
  }
  return XARM_OK;
}

XARM_STATUS xarmTestReachable(QARM_POSITION_STR* qarmPosition, TP_POSE* pose)
{
  Gb_6rParameters pa10;
  Gb_th th;
  Gb_q6 qPose;
  Gb_dataMGD qData;
  Gb_quat Gb_pose;
  Gb_statusMGI status;
  
  if (pa10.a2 != QARM_ARM_A2) {
    pa10.a2 =  QARM_ARM_A2;
    pa10.r4 =  QARM_ARM_R4;
    pa10.of1 = QARM_ARM_OF1;
    pa10.of2 = QARM_ARM_OF2;
    pa10.of3 = QARM_ARM_OF3;
    pa10.of4 = QARM_ARM_OF4;
    pa10.of5 = QARM_ARM_OF5;
    pa10.of6 = QARM_ARM_OF6;
    pa10.epsilon = QARM_ARM_EPSILON;
  }

  /* Transform the quaternion to the Gb quaterniom type */
  Gb_pose.x =  pose->p.x;
  Gb_pose.y =  pose->p.y;
  Gb_pose.z =  pose->p.z;
  Gb_pose.w =  pose->q.n; 
  Gb_pose.vx = pose->q.i; 
  Gb_pose.vy = pose->q.j; 
  Gb_pose.vz = pose->q.k;
 
  /* Gb quaternion to matrix rotation */
  Gb_quat_th(&Gb_pose, &th);
  status = Gb_MGI6rTh_O(&pa10, &th, &(qarmPosition->q), &qData, &qPose);

  if (status == MGI_OK || status == MGI_SINGULAR ) {
    printf("status of MGI =  MGI_OK\n");
    return XARM_OK;
  }
  if ( status == MGI_SINGULAR ) {
    printf("status of MGI = MGI_SINGULAR\n");
    return XARM_OK;
  } 
  printf("xarm Goal Pose unreacheable STATUS_MGI = %d\n", status);
  return XARM_ERROR;
}

XARM_STATUS xarmTestReachableBox(TP_POSE* pose) 
{
 if ( (pose->p.x < XARM_X_MIN) || (pose->p.x > XARM_X_MAX) || (pose->p.y < XARM_Y_MIN) || (pose->p.y > XARM_Y_MAX)) {
    printf("Next_thTool.p.x %f\n",pose->p.x);
    printf("Next_thTool.p.y %f\n",pose->p.y);
    return XARM_ERROR;
  }
  
  return XARM_OK;
}

XARM_STATUS xarmCalculOfAccVelPos(int tick, SEGMENT* seg, TP_VECTOR* v)
{
  double k1, k2, k3; 
  double Ts2, Ts3;
  int i;

  Ts2 = (double)(S_T*S_T)/(S_TD*S_TD);
  Ts3 = (double)(S_T*Ts2)/S_TD;
  i = tick;

//  printf(" TYPE of Segment %d\n",seg->type);
  switch(seg->type) {	
  case 1:	
    k1 = (double)(S_T*i)/S_TD;
    k2 = (double) i*i*Ts2/2.0;
    k3 = (double) i*i*i*Ts3/6.0;
    v->Acc = (seg->A0) + (seg->dir)*(seg->J)*k1;
    v->Vel = (seg->V0) + (seg->A0)*k1 + (seg->dir)*(seg->J)*k2;
    v->Pos = (seg->X0) + (seg->V0)*k1 + (seg->A0)*k2 + (seg->dir)*(seg->J)*k3;
    break;      
  case 2:
    k1 = (double)(S_T*i)/S_TD;
    k2 = i*i*Ts2/2;
    v->Acc = (seg->A0);
    v->Vel = (seg->V0) + (seg->A0)*k1;
    v->Pos = (seg->X0) + (seg->V0)*k1 + (seg->A0)*k2;
    break;  
  case 3:
    k1 = (double)(S_T*i)/S_TD;
    k2 = i*i*Ts2/2;
    k3 = i*i*i*Ts3/6;
    v->Acc = (seg->A0) - (seg->dir)*(seg->J)*k1;
    v->Vel = (seg->V0)+  (seg->A0)*k1 - (seg->dir)*(seg->J)*k2;
    v->Pos = (seg->X0) + (seg->V0)*k1 + (seg->A0)*k2 - (seg->dir)*(seg->J)*k3;
    break;  
  case 4:
    k1 = (double)(S_T*i)/S_TD;
    v->Acc = (seg->A0);
    v->Vel = (seg->V0);
    v->Pos = (seg->X0) + (seg->V0)*k1;
    break;  
  case 5:
    k1 = (double)(S_T*i)/S_TD;
    k2 = i*i*Ts2/2;
    k3 = i*i*i*Ts3/6;
    v->Acc = (seg->A0) - (seg->dir)*(seg->J)*k1;
    v->Vel = (seg->V0) + (seg->A0)*k1 - (seg->dir)*(seg->J)*k2;
    v->Pos = (seg->X0) + (seg->V0)*k1 + (seg->A0)*k2 - (seg->dir)*(seg->J)*k3;
    break;  
  case 6:
    k1 = (double)(S_T*i)/S_TD;
    k2 = i*i*Ts2/2;
    v->Acc = (seg->A0);
    v->Vel = (seg->V0) + (seg->A0)*k1;
    v->Pos = (seg->X0) + (seg->V0)*k1 + (seg->A0)*k2;
    break;  
  case 7:
    k1 = (double)(S_T*i)/S_TD;
    k2 = i*i*Ts2/2;
    k3 = i*i*i*Ts3/6;
    v->Acc = (seg->A0) + (seg->dir)*(seg->J)*k1;
    v->Vel = (seg->V0) + (seg->A0)*k1 + (seg->dir)*(seg->J)*k2;
    v->Pos = (seg->X0) + (seg->V0)*k1 + (seg->A0)*k2 + (seg->dir)*(seg->J)*k3;     
    break;      
  default:
    printf("xarm Bad type of Segment\n");
    return XARM_ERROR;
  }

  if (isnan(v->Pos) || isnan(v->Vel)  || isnan(v->Acc)) {
    printf("PROBLEME\n");
    printf("seg->X0 %g seg->V0 %g seg->A0 %g seg->J %g seg->dir %d\n",seg->X0 , seg->V0 ,seg->A0 ,seg->J ,seg->dir);

  }

  return XARM_OK;
}

void VerifyAllQueue(QUEUE* SegX, QUEUE* SegY, QUEUE* SegZ, QUEUE* SegQN, QUEUE* SegQI, QUEUE* SegQJ, QUEUE* SegQK, int* VerifyIfItsLastSegment)
{
      if (queueVerifyIfEmpty(SegX)!=0) {
	PRINT_ERROR("xarm QUEUE SegX is empty \n");
	*VerifyIfItsLastSegment ++;
      }
      if (queueVerifyIfEmpty(SegY)!=0) {
	PRINT_ERROR("xarm QUEUE SegY is empty \n");
	*VerifyIfItsLastSegment ++;
      }
      if (queueVerifyIfEmpty(SegZ)!=0) {
	PRINT_ERROR("xarm QUEUE SegZ is empty \n");
	*VerifyIfItsLastSegment ++;
      }
      if (queueVerifyIfEmpty(SegQN)!=0) {
	PRINT_ERROR("xarm QUEUE SegQN is empty \n");
	*VerifyIfItsLastSegment ++;
      }
      if (queueVerifyIfEmpty(SegQI)!=0) {
	PRINT_ERROR("xarm QUEUE SegQI is empty \n");
	*VerifyIfItsLastSegment ++;
      }
      if (queueVerifyIfEmpty(SegQJ)!=0) {
	PRINT_ERROR("xarm QUEUE SegQJ is empty \n");
	*VerifyIfItsLastSegment ++;
      }
      if (queueVerifyIfEmpty(SegQK)!=0) {
	PRINT_ERROR("xarm QUEUE SegQK is empty \n");
	*VerifyIfItsLastSegment ++;
      }
      return;
}

XARM_STATUS GeometricalCalculations( dmat PFrom, dmat PTo,dmat QFrom, dmat QTo,
				     double *MaxLDist,  double *MaxADist, dmat Dist,
				     dmat LDist, dmat AbsLDist, dmat ADist, dmat AbsADist, dmat AbsDist,
				     dmat Dir, int *report) 
{
 /* Linear Distance Calcul */
  if (matDif(PTo,PFrom,LDist)) {
    PRINT_ERROR("Linear Distance Calcul ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  /* Find Absolut Distance Deplacement */
  if(absVect(LDist,AbsLDist)) {
    PRINT_ERROR("Find Absolut Linear Distance Deplacement ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  /* Find Maximum Linear Distance Deplacement*/
  if (maxVect(AbsLDist, MaxLDist)) {
    PRINT_ERROR("Maximum Distance Element ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }

  /* Angular Distance Calcul */
  if (matDif(QTo,QFrom,ADist)) {
    PRINT_ERROR(" Angular Distance Calcul");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  /* Find Absolut Distance Deplacement */
  if(absVect(ADist,AbsADist)) {
    PRINT_ERROR(" Find Absolut Angular Distance Deplacement ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  /* Find Maximum Angular Distance Deplacement*/
  if (maxVect(AbsADist, MaxADist)) {
    PRINT_ERROR("Maximum Distance Element ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }

  /* Distance  = [Linear Distance ... Angular Distance ] */
  if( conca_nAB (XARM_POS_DIM,LDist,ADist,Dist) ) {
    PRINT_ERROR(" concatenate: Distance = [Linear Distance ... Angular Distance ]");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }

  /* Abs Distance = [Abs Linear Distance ... Abs Angular Distance ] */
  if( conca_nAB (XARM_POS_DIM,AbsLDist,AbsADist,AbsDist) ) {
    PRINT_ERROR(" concatenate: Abs Distance = [Abs Linear Distance ... Abs Angular Distance ]");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }

  /* Calcul Direction Vector */
  if(signVect(Dist,Dir)) {
    PRINT_ERROR(" Find Direction Motion");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }

  return XARM_OK;
}

XARM_STATUS CalculPointToPointJerkProfile(dmat PFrom, dmat QFrom, TP_POSELIMITS Limits, double MaxLDist, double MaxADist, double MaxDist,
					  dmat AbsDist, dmat Dir, TP_JERKS jerk[], TP_TIMES Acc[], TP_TIMES Vel[], TP_TIMES Pos[], TP_TIMES* TNE, int* report)
{
  double Tjc, Tac, Tvc;
  TP_JERKS Jerks;
  TP_TIMES T, TM;
  TP_COND IC, FC;
  double LTjc, LTac, LTvc, TimeLL;
  double ATjc, ATac, ATvc, TimeAL;
  int NOE;
  

  /* Gets Tjc, Tac and Tvc for Maximum Linear Distance Deplacement */
  if ( tpJerkProfile(MaxLDist, Limits.linear.maxJerk, Limits.linear.maxAcc, Limits.linear.maxVel, &LTjc, &LTac, &LTvc, &TimeLL)!=OK) {
    PRINT_ERROR(" jerk Profile Gets Times for Maximum Linear Distance");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  /* Gets Tjc, Tac and Tvc for Maximum Angular Distance Deplacement */
  if ( tpJerkProfile(MaxADist, Limits.angular.maxJerk, Limits.angular.maxAcc, Limits.angular.maxVel, &ATjc, &ATac, &ATvc, &TimeAL)!=OK) {
    PRINT_ERROR(" jerk Profile  Gets Times for Maximum Angular Distance");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  if (TimeAL <= TimeLL) {
    Tjc = LTjc;
    Tac = LTac;
    Tvc = LTvc;
    MaxDist = MaxLDist;
    Jerks.sel = 1;
    Jerks.J1 = Limits.linear.maxJerk;
  } else {
    Tjc = ATjc;
    Tac = ATac;
    Tvc = ATvc;
    MaxDist = MaxADist;
    Jerks.sel = 1;
    Jerks.J1 = Limits.angular.maxJerk;
  }
  /* Times Structure T construction */
  T.Tjpa = Tjc;
  T.Taca = Tac;
  T.Tjna = Tjc;
  T.Tvc  = Tvc;
  T.Tjnb = Tjc;
  T.Tacb = Tac;
  T.Tjpb = Tjc;

  /* Initial Conditions for Verify Times */
  IC.a = 0.0;
  IC.v = 0.0;
  IC.x = 0.0;

  /* Verify Times */
  if (tpVerifyTimes(DISTANCE_TOLERANCE, MaxDist, Jerks, IC, 1, T, &FC, &Acc[0], &Vel[0], &Pos[0], TP_ON)!=OK) {
    PRINT_ERROR(" Verify Times  1");
    printf("DISTANCE_TOLERANCE %f\n",DISTANCE_TOLERANCE);
    printf("MaxDist %f\n",MaxDist);
    printf("Jerks.J1 %f\n",Jerks.J1);    
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }

  /* Get Number of Elements needed for Vectors Definition */
  if (tpGetMonotonicTimes(T, &TM, &NOE)!=OK) {
    PRINT_ERROR(" Get MonotonicTimes ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  tpGetNumberOfElement (&TM, TNE);

  /* Calcul of Jerks Profile for next motion */
  /* IC PX */
  IC.a=0;
  IC.v=0;
  IC.x=PFrom.el[0][0];
  if (tpJerkProfileAdjusted(AbsDist.el[0][0], IC, T, TM, Dir.el[0][0], Limits.linear, &jerk[0], &Acc[0], &Vel[0], &Pos[0])!= OK) {
    PRINT_ERROR(" PX Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC PY */
  IC.a=0;
  IC.v=0;
  IC.x= PFrom.el[0][1];
  if (tpJerkProfileAdjusted(AbsDist.el[0][1], IC, T, TM, Dir.el[0][1], Limits.linear, &jerk[1], &Acc[1], &Vel[1], &Pos[1])!= OK) {
    PRINT_ERROR(" PY Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC PZ */
  IC.a=0;
  IC.v=0;
  IC.x=PFrom.el[0][2];
  if (tpJerkProfileAdjusted(AbsDist.el[0][2], IC, T, TM, Dir.el[0][2], Limits.linear, &jerk[2], &Acc[2], &Vel[2], &Pos[2])!= OK) {
    PRINT_ERROR(" PZ Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC QN */
  IC.a=0;
  IC.v=0;
  IC.x=QFrom.el[0][0];
  if (tpJerkProfileAdjusted(AbsDist.el[0][3], IC, T, TM, Dir.el[0][3], Limits.angular, &jerk[3], &Acc[3], &Vel[3], &Pos[3])!= OK) {
    PRINT_ERROR(" QN Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC QI */
  IC.a=0;
  IC.v=0;
  IC.x=QFrom.el[0][1];
  if (tpJerkProfileAdjusted(AbsDist.el[0][4], IC, T, TM, Dir.el[0][4], Limits.angular, &jerk[4], &Acc[4], &Vel[4], &Pos[4])!= OK) {
    PRINT_ERROR(" QI Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC QJ */
  IC.a=0;
  IC.v=0;
  IC.x=QFrom.el[0][2];
  if (tpJerkProfileAdjusted(AbsDist.el[0][5], IC, T, TM, Dir.el[0][5], Limits.angular, &jerk[5], &Acc[5], &Vel[5], &Pos[5])!= OK) {
    PRINT_ERROR(" QJ Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC QK */
  IC.a=0;
  IC.v=0;
  IC.x=QFrom.el[0][3];
  if (tpJerkProfileAdjusted(AbsDist.el[0][6], IC, T, TM, Dir.el[0][6], Limits.angular, &jerk[6], &Acc[6], &Vel[6], &Pos[6])!= OK) {
    PRINT_ERROR(" QK Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;     
  }

  return XARM_OK;
}

//CalculPointToPointJerkProfileV2(PFrom, QFrom, limitsExternalTrack, MaxLDist, MaxADist, MaxDist, AbsDist, Dir, &(motion1.jerk), &(motion1.Acc), &(motion1.Vel), &(motion1.Pos), &(motion1.TNE), report);

XARM_STATUS CalculPointToPointJerkProfileV2(dmat PFrom, dmat QFrom, TP_POSELIMITS Limits, double MaxLDist, double MaxADist, double MaxDist, dmat AbsDist, dmat Dir, TP_MOTION* motion, int* report)
{
  double Tjc, Tac, Tvc;
  TP_JERKS Jerks;
  TP_TIMES T, TM;
  TP_COND IC, FC;
  double LTjc, LTac, LTvc, TimeLL;
  double ATjc, ATac, ATvc, TimeAL;
  int NOE;
  

  /* Gets Tjc, Tac and Tvc for Maximum Linear Distance Deplacement */
  if ( tpJerkProfile(MaxLDist, Limits.linear.maxJerk, Limits.linear.maxAcc, Limits.linear.maxVel, &LTjc, &LTac, &LTvc, &TimeLL)!=OK) {
    PRINT_ERROR(" jerk Profile Gets Times for Maximum Linear Distance");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  /* Gets Tjc, Tac and Tvc for Maximum Angular Distance Deplacement */
  if ( tpJerkProfile(MaxADist, Limits.angular.maxJerk, Limits.angular.maxAcc, Limits.angular.maxVel, &ATjc, &ATac, &ATvc, &TimeAL)!=OK) {
    PRINT_ERROR(" jerk Profile  Gets Times for Maximum Angular Distance");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  if (TimeAL <= TimeLL) {
    Tjc = LTjc;
    Tac = LTac;
    Tvc = LTvc;
    MaxDist = MaxLDist;
    Jerks.sel = 1;
    Jerks.J1 = Limits.linear.maxJerk;
  } else {
    Tjc = ATjc;
    Tac = ATac;
    Tvc = ATvc;
    MaxDist = MaxADist;
    Jerks.sel = 1;
    Jerks.J1 = Limits.angular.maxJerk;
  }
  /* Times Structure T construction */
  T.Tjpa = Tjc;
  T.Taca = Tac;
  T.Tjna = Tjc;
  T.Tvc  = Tvc;
  T.Tjnb = Tjc;
  T.Tacb = Tac;
  T.Tjpb = Tjc;

  /* Initial Conditions for Verify Times */
  IC.a = 0.0;
  IC.v = 0.0;
  IC.x = 0.0;

  /* Verify Times */
  if (tpVerifyTimes(DISTANCE_TOLERANCE, MaxDist, Jerks, IC, 1, T, &FC, &(motion->Acc[0]), &(motion->Vel[0]), &(motion->Pos[0]), TP_ON)!=OK) {
    PRINT_ERROR(" Verify Times  1");
    printf("DISTANCE_TOLERANCE %f\n",DISTANCE_TOLERANCE);
    printf("MaxDist %f\n",MaxDist);
    printf("Jerks.J1 %f\n",Jerks.J1);    
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }

  /* Get Number of Elements needed for Vectors Definition */
  if (tpGetMonotonicTimes(T, &TM, &NOE)!=OK) {
    PRINT_ERROR(" Get MonotonicTimes ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;
  }
  tpGetNumberOfElement (&TM, &(motion->TNE));

  /* Calcul of Jerks Profile for next motion */
  /* IC PX */
  IC.a=0;
  IC.v=0;
  IC.x=PFrom.el[0][0];
  if (tpJerkProfileAdjusted(AbsDist.el[0][0], IC, T, TM, Dir.el[0][0], Limits.linear, &(motion->jerk[0]), &(motion->Acc[0]), &(motion->Vel[0]), &(motion->Pos[0]))!= OK) {
    PRINT_ERROR(" PX Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC PY */
  IC.a=0;
  IC.v=0;
  IC.x= PFrom.el[0][1];
  if (tpJerkProfileAdjusted(AbsDist.el[0][1], IC, T, TM, Dir.el[0][1], Limits.linear, &(motion->jerk[1]), &(motion->Acc[1]), &(motion->Vel[1]), &(motion->Pos[1]))!= OK) {
    PRINT_ERROR(" PY Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC PZ */
  IC.a=0;
  IC.v=0;
  IC.x=PFrom.el[0][2];
  if (tpJerkProfileAdjusted(AbsDist.el[0][2], IC, T, TM, Dir.el[0][2], Limits.linear, &(motion->jerk[2]), &(motion->Acc[2]), &(motion->Vel[2]), &(motion->Pos[2]))!= OK) {
    PRINT_ERROR(" PZ Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC QN */
  IC.a=0;
  IC.v=0;
  IC.x=QFrom.el[0][0];
  if (tpJerkProfileAdjusted(AbsDist.el[0][3], IC, T, TM, Dir.el[0][3], Limits.angular, &(motion->jerk[3]), &(motion->Acc[3]), &(motion->Vel[3]), &(motion->Pos[3]))!= OK) {
    PRINT_ERROR(" QN Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC QI */
  IC.a=0;
  IC.v=0;
  IC.x=QFrom.el[0][1];
  if (tpJerkProfileAdjusted(AbsDist.el[0][4], IC, T, TM, Dir.el[0][4], Limits.angular, &(motion->jerk[4]), &(motion->Acc[4]), &(motion->Vel[4]), &(motion->Pos[4]))!= OK) {
    PRINT_ERROR(" QI Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC QJ */
  IC.a=0;
  IC.v=0;
  IC.x=QFrom.el[0][2];
  if (tpJerkProfileAdjusted(AbsDist.el[0][5], IC, T, TM, Dir.el[0][5], Limits.angular, &(motion->jerk[5]), &(motion->Acc[5]), &(motion->Vel[5]), &(motion->Pos[5]))!= OK) {
    PRINT_ERROR(" QJ Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR; 
  }

  /* IC QK */
  IC.a=0;
  IC.v=0;
  IC.x=QFrom.el[0][3];
  if (tpJerkProfileAdjusted(AbsDist.el[0][6], IC, T, TM, Dir.el[0][6], Limits.angular, &(motion->jerk[6]), &(motion->Acc[6]), &(motion->Vel[6]), &(motion->Pos[6]))!= OK) {
    PRINT_ERROR(" QK Profile Vectors  ");
    *report = S_xarm_ERROR_TP;
    return XARM_ERROR;     
  }

  return XARM_OK;
}

XARM_STATUS CalculTransitionWithStopMotion(double PosStop[], TP_POSELIMITS limitsGoto, TP_COND INITIAL_COND[], TP_TIMES Acc[], TP_TIMES Vel[], TP_TIMES Pos[],
					   double OptimalTransitionTimesWithStop[], TP_AXIS_TIMES *AxisTimes_End, TP_AXIS_TIMES *AxisTimes_Start, 
					   int DirTransition_End[], int DirTransition_Start[], TP_TIMES Acc_End[], TP_TIMES Vel_End[], TP_TIMES Pos_End[],
					   TP_TIMES Acc_Start[], TP_TIMES Vel_Start[], TP_TIMES Pos_Start[], int* report)
{
   int  j;
   int TrajectoryType;
   TP_COND IC, FC;
   TP_LIMITS auxLimits;
   double TotalTime_End[XARM_NB_DIM];
   double TotalTime_Start[XARM_NB_DIM];
   TP_JERKS Jerks;
   double GD ;
 
  for(j=0;j<= XARM_NB_DIM-1;j++) {
	
    IC.a = INITIAL_COND[j].a;
    IC.v = INITIAL_COND[j].v;
    IC.x = 0;
    FC.a = 0;
    FC.v = 0;
    FC.x = (PosStop[j] - INITIAL_COND[j].x);
    Jerks.sel = 1;
    if (j<3) {
      auxLimits.maxJerk = limitsGoto.linear.maxJerk;
      auxLimits.maxAcc  = limitsGoto.linear.maxAcc;
      auxLimits.maxVel  = limitsGoto.linear.maxVel;
      Jerks.J1 = limitsGoto.linear.maxJerk;
    } else {
      auxLimits.maxJerk = limitsGoto.angular.maxJerk;
      auxLimits.maxAcc  = limitsGoto.angular.maxAcc;
      auxLimits.maxVel  = limitsGoto.angular.maxVel;
      Jerks.J1 = limitsGoto.angular.maxJerk; 	   
    }
    if (j==0) {
      /***************** PX *****************/
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_End->PX), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_End[0] = TrajectoryType;
      TotalTime_End[0] = (AxisTimes_End->PX.Tjpa + AxisTimes_End->PX.Taca + AxisTimes_End->PX.Tjna + AxisTimes_End->PX.Tvc + AxisTimes_End->PX.Tjnb + AxisTimes_End->PX.Tacb + AxisTimes_End->PX.Tjpb);

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x = INITIAL_COND[0].x ;
      GD =  FC.x* DirTransition_End[0];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_End[0],(AxisTimes_End->PX) , &FC, &Acc_End[0], &Vel_End[0], &Pos_End[0], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times  1");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      }      
    }
    if (j==1) { 
      /***************** PY *****************/
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_End->PY), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_End[1] = TrajectoryType;
      TotalTime_End [1] = (AxisTimes_End->PY.Tjpa + AxisTimes_End->PY.Taca + AxisTimes_End->PY.Tjna + AxisTimes_End->PY.Tvc + AxisTimes_End->PY.Tjnb + AxisTimes_End->PY.Tacb + AxisTimes_End->PY.Tjpb);

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x = INITIAL_COND[1].x ;
      GD =  FC.x* DirTransition_End[1];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_End[1],(AxisTimes_End->PY) , &FC, &Acc_End[1], &Vel_End[1], &Pos_End[1], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times  1");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      }      
    }
    if (j==2) {
      /***************** PZ *****************/ 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_End->PZ), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_End[2] = TrajectoryType;
      TotalTime_End[2] = (AxisTimes_End->PZ.Tjpa + AxisTimes_End->PZ.Taca + AxisTimes_End->PZ.Tjna + AxisTimes_End->PZ.Tvc + AxisTimes_End->PZ.Tjnb + AxisTimes_End->PZ.Tacb + AxisTimes_End->PZ.Tjpb); 

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x = INITIAL_COND[2].x ;
      GD =  FC.x* DirTransition_End[2];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_End[2],(AxisTimes_End->PZ) , &FC, &Acc_End[2], &Vel_End[2], &Pos_End[2], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times  1");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      }   
    }
    if (j==3) {
      /***************** QN *****************/  
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_End->QN), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_End[3] = TrajectoryType;
      TotalTime_End[3] = (AxisTimes_End->QN.Tjpa + AxisTimes_End->QN.Taca + AxisTimes_End->QN.Tjna + AxisTimes_End->QN.Tvc + AxisTimes_End->QN.Tjnb + AxisTimes_End->QN.Tacb + AxisTimes_End->QN.Tjpb);

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x = INITIAL_COND[3].x ;
      GD =  FC.x* DirTransition_End[3];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_End[3],(AxisTimes_End->QN) , &FC, &Acc_End[3], &Vel_End[3], &Pos_End[3], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times  1");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      }  
    }
    if (j==4) { 
      /***************** QI *****************/ 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_End->QI), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_End[4] = TrajectoryType;
      TotalTime_End[4] = (AxisTimes_End->QI.Tjpa + AxisTimes_End->QI.Taca + AxisTimes_End->QI.Tjna + AxisTimes_End->QI.Tvc + AxisTimes_End->QI.Tjnb + AxisTimes_End->QI.Tacb + AxisTimes_End->QI.Tjpb);

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x = INITIAL_COND[4].x ;
      GD =  FC.x* DirTransition_End[4];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_End[4],(AxisTimes_End->QI) , &FC, &Acc_End[4], &Vel_End[4], &Pos_End[4], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times  1");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      }  
    }
    if (j==5) {
      /***************** QJ *****************/  
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_End->QJ), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_End[5] = TrajectoryType;
      TotalTime_End[5] = (AxisTimes_End->QJ.Tjpa + AxisTimes_End->QJ.Taca + AxisTimes_End->QJ.Tjna + AxisTimes_End->QJ.Tvc + AxisTimes_End->QJ.Tjnb + AxisTimes_End->QJ.Tacb + AxisTimes_End->QJ.Tjpb);

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x = INITIAL_COND[5].x ;
      GD =  FC.x* DirTransition_End[5];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_End[5],(AxisTimes_End->QJ) , &FC, &Acc_End[5], &Vel_End[5], &Pos_End[5], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times  1");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      } 
    }
    if (j==6) {
      /***************** QK *****************/ 
      
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_End->QK), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_End[6] = TrajectoryType;
      TotalTime_End[6] = (AxisTimes_End->QK.Tjpa + AxisTimes_End->QK.Taca + AxisTimes_End->QK.Tjna + AxisTimes_End->QK.Tvc + AxisTimes_End->QK.Tjnb + AxisTimes_End->QK.Tacb + AxisTimes_End->QK.Tjpb);
      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x = INITIAL_COND[6].x ;
      GD =  FC.x* DirTransition_End[6];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_End[6],(AxisTimes_End->QK) , &FC, &Acc_End[6], &Vel_End[6], &Pos_End[6], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times QK 1");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      } 
    }
  }
 
  for(j=0;j<=XARM_NB_DIM-1;j++) {
	
    IC.a = 0;
    IC.v = 0;
    IC.x = 0;
    FC.a = Acc[j].Tjna;
    FC.v = Vel[j].Tjna;
    FC.x = Pos[j].Tjna - PosStop[j];
    Jerks.sel = 1;
    if (j<3) {
      auxLimits.maxJerk = limitsGoto.linear.maxJerk;
      auxLimits.maxAcc  = limitsGoto.linear.maxAcc;
      auxLimits.maxVel  = limitsGoto.linear.maxVel;
      Jerks.J1 = limitsGoto.linear.maxJerk; 	 
    } else {
      auxLimits.maxJerk = limitsGoto.angular.maxJerk;
      auxLimits.maxAcc  = limitsGoto.angular.maxAcc;
      auxLimits.maxVel  = limitsGoto.angular.maxVel;
      Jerks.J1 = limitsGoto.angular.maxJerk; 	   
    }
    if (j==0) { 
      /***************** PX *****************/
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_Start->PX), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_Start[0] = TrajectoryType;
      TotalTime_Start[0] = (AxisTimes_Start->PX.Tjpa + AxisTimes_Start->PX.Taca + AxisTimes_Start->PX.Tjna + AxisTimes_Start->PX.Tvc + AxisTimes_Start->PX.Tjnb + AxisTimes_Start->PX.Tacb + AxisTimes_Start->PX.Tjpb);
 
      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x =  PosStop[0];
      GD =  FC.x* DirTransition_Start[0];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_Start[0],(AxisTimes_Start->PX) , &FC, &Acc_Start[0], &Vel_Start[0], &Pos_Start[0], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times PX");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      }     
   }
    if (j==1) { 
      /***************** PY *****************/
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_Start->PY), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_Start[1] = TrajectoryType;
      TotalTime_Start [1] = (AxisTimes_Start->PY.Tjpa + AxisTimes_Start->PY.Taca + AxisTimes_Start->PY.Tjna + AxisTimes_Start->PY.Tvc + AxisTimes_Start->PY.Tjnb + AxisTimes_Start->PY.Tacb + AxisTimes_Start->PY.Tjpb);

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x =  PosStop[1];
      GD =  FC.x* DirTransition_Start[1];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC, DirTransition_Start[1],(AxisTimes_Start->PY) , &FC, &Acc_Start[1], &Vel_Start[1], &Pos_Start[1], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times PY");	
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      } 
    }
    if (j==2) { 
      /***************** PZ *****************/ 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_Start->PZ), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_Start[2] = TrajectoryType;
      TotalTime_Start[2] = (AxisTimes_Start->PZ.Tjpa + AxisTimes_Start->PZ.Taca + AxisTimes_Start->PZ.Tjna + AxisTimes_Start->PZ.Tvc + AxisTimes_Start->PZ.Tjnb + AxisTimes_Start->PZ.Tacb + AxisTimes_Start->PZ.Tjpb); 

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x =  PosStop[2];
      GD =  FC.x* DirTransition_Start[2];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_Start[2],(AxisTimes_Start->PZ) , &FC, &Acc_Start[2], &Vel_Start[2], &Pos_Start[2], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times PZ");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      } 
    }
    if (j==3) { 
      /***************** QN *****************/ 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_Start->QN), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_Start[3] = TrajectoryType;
      TotalTime_Start[3] = (AxisTimes_Start->QN.Tjpa + AxisTimes_Start->QN.Taca + AxisTimes_Start->QN.Tjna + AxisTimes_Start->QN.Tvc + AxisTimes_Start->QN.Tjnb + AxisTimes_Start->QN.Tacb + AxisTimes_Start->QN.Tjpb);

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x =  PosStop[3];
      GD =  FC.x* DirTransition_Start[3];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_Start[3],(AxisTimes_Start->QN) , &FC, &Acc_Start[3], &Vel_Start[3], &Pos_Start[3], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times QN");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      } 
    }
    if (j==4) { 
      /***************** QI *****************/
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_Start->QI), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_Start[4] = TrajectoryType;
      TotalTime_Start[4] = (AxisTimes_Start->QI.Tjpa + AxisTimes_Start->QI.Taca + AxisTimes_Start->QI.Tjna + AxisTimes_Start->QI.Tvc + AxisTimes_Start->QI.Tjnb + AxisTimes_Start->QI.Tacb + AxisTimes_Start->QI.Tjpb);
 
      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x =  PosStop[4];
      GD =  FC.x* DirTransition_Start[4];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_Start[4],(AxisTimes_Start->QI) , &FC, &Acc_Start[4], &Vel_Start[4], &Pos_Start[4], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times QI");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      } 
   }
    if (j==5) { 
      /***************** QJ *****************/ 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_Start->QJ), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_Start[5] = TrajectoryType;
      TotalTime_Start[5] = (AxisTimes_Start->QJ.Tjpa + AxisTimes_Start->QJ.Taca + AxisTimes_Start->QJ.Tjna + AxisTimes_Start->QJ.Tvc + AxisTimes_Start->QJ.Tjnb + AxisTimes_Start->QJ.Tacb + AxisTimes_Start->QJ.Tjpb);

      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x =  PosStop[5];
      GD =  FC.x* DirTransition_Start[5];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_Start[5],(AxisTimes_Start->QJ) , &FC, &Acc_Start[5], &Vel_Start[5], &Pos_Start[5], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times QJ");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      } 
    }
    if (j==6) {
      /***************** QK *****************/
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes_Start->QK), &TrajectoryType)!=0) {
	PRINT_ERROR("ERROR Jerk Profile Transition\n");
	return XARM_ERROR;
      }
      DirTransition_Start[6] = TrajectoryType;
      TotalTime_Start[6] = (AxisTimes_Start->QK.Tjpa + AxisTimes_Start->QK.Taca + AxisTimes_Start->QK.Tjna + AxisTimes_Start->QK.Tvc + AxisTimes_Start->QK.Tjnb + AxisTimes_Start->QK.Tacb + AxisTimes_Start->QK.Tjpb);
      /* Get initial conditions for each vectors Acc Vel and Pos */
      IC.x =  PosStop[6];
      GD =  FC.x* DirTransition_Start[6];
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC,  DirTransition_Start[6],(AxisTimes_Start->QK) , &FC, &Acc_Start[6], &Vel_Start[6], &Pos_Start[6], TP_ON)!=OK) {
	PRINT_ERROR(" Verify Times QK");
	*report = S_xarm_ERROR_TP;
	return XARM_ERROR;
      }
    }
  }

  for(j=0;j<=XARM_NB_DIM-1;j++) {
    OptimalTransitionTimesWithStop[j] =  TotalTime_End[j] + TotalTime_Start[j];
 //   printf("TotalTime_TransitionWithStop[%d] %f\n", j, OptimalTransitionTimesWithStop[j]); 
  }

  return XARM_OK;
}


XARM_STATUS FindMaxTimeAxis(TP_POSELIMITS limitsGoto, TP_COND INITIAL_COND[], TP_TIMES Acc[], TP_TIMES Vel[], TP_TIMES Pos[], double *MaxTime, int* MaxTimeAxis)
{
  int i=0;
  int TrajectoryType = 0;
  TP_COND IC, FC;
  TP_LIMITS auxLimits;
  TP_TIMES T_Jerk;
  double DistTransition [XARM_NB_DIM];
  int DirTransition[XARM_NB_DIM];
  TP_AXIS_TIMES AxisTimes;
  double TotalTime [7];

  for(i=0;i<=XARM_NB_DIM-1;i++) {

   IC.a = (double)((int)(INITIAL_COND[i].a * 1000000))/1000000;
   IC.v = (double)((int)(INITIAL_COND[i].v * 1000000))/1000000;
   IC.x = 0;
   FC.a = (double)((int)(Acc[i].Tjna * 1000000))/1000000;
   FC.v = (double)((int)(Vel[i].Tjna * 1000000))/1000000;
   FC.x = (double)((int)(((Pos[i].Tjna) - (INITIAL_COND[i].x)) * 1000000))/1000000;

   if (FC.a < 1e-06) {FC.a = 0.0;}
   if (IC.a < 1e-06) {IC.a = 0.0;}
//  printf("\n");
//  printf("\n");
//  printf("IC.a %f\n",IC.a);
//  printf("IC.v %f\n",IC.v);
//  printf("IC.x %f\n",IC.x);
//  printf("FC.a %f\n",FC.a);
//  printf("FC.v %f\n",FC.v);
//  printf("FC.x %f\n",FC.x);
//  
//  IC.a = INITIAL_COND[i].a;
//  IC.v = INITIAL_COND[i].v;
//  IC.x = 0;
//  FC.a = Acc[i].Tjna ;
//  FC.v = Vel[i].Tjna ;
//  FC.x = (Pos[i].Tjna) - (INITIAL_COND[i].x);
//  
//  printf("IC.a %f\n",IC.a);
//  printf("IC.v %f\n",IC.v);
//  printf("IC.x %f\n",IC.x);
//  printf("FC.a %f\n",FC.a);
//  printf("FC.v %f\n",FC.v);
//  printf("FC.x %f\n",FC.x);
  

   DistTransition [i] = FC.x;
   
   
   if (i<3) {
     auxLimits.maxJerk = limitsGoto.linear.maxJerk;
     auxLimits.maxAcc  = limitsGoto.linear.maxAcc;
      auxLimits.maxVel  = limitsGoto.linear.maxVel;
    } else {
      auxLimits.maxJerk = limitsGoto.angular.maxJerk;
      auxLimits.maxAcc  = limitsGoto.angular.maxAcc;
      auxLimits.maxVel  = limitsGoto.angular.maxVel;	   
    }
    if (i==0) { 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes.PX), &TrajectoryType)!=0) {
	printf("ERROR Jerk Profile Transition X\n");
	printf("IC.a =%f\n",IC.a);
	printf("IC.v =%f\n",IC.v);
	printf("IC.x =%f\n",IC.x);
	printf("FC.a =%f\n",FC.a);
	printf("FC.v =%f\n",FC.v);
	printf("FC.x =%f\n",FC.x);
	return XARM_ERROR;
      }
      DirTransition[0] = TrajectoryType;
      TotalTime [0] = (AxisTimes.PX.Tjpa + AxisTimes.PX.Taca + AxisTimes.PX.Tjna + AxisTimes.PX.Tvc + AxisTimes.PX.Tjnb + AxisTimes.PX.Tacb + AxisTimes.PX.Tjpb);
      /* Init of Max Values */
      *MaxTime = TotalTime [0];
      *MaxTimeAxis = 0;
      T_Jerk.Tjpa =  AxisTimes.PX.Tjpa;
      T_Jerk.Taca =  AxisTimes.PX.Taca;
      T_Jerk.Tjna =  AxisTimes.PX.Tjna;
      T_Jerk.Tvc  =  AxisTimes.PX.Tvc;
      T_Jerk.Tjnb =  AxisTimes.PX.Tjnb;
      T_Jerk.Tacb =  AxisTimes.PX.Tacb;
      T_Jerk.Tjpb =  AxisTimes.PX.Tjpb;
    }
    if (i==1) { 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes.PY), &TrajectoryType)!=0) {
	printf("ERROR Jerk Profile Transition Y\n");
	printf("IC.a =%f\n",IC.a);
	printf("IC.v =%f\n",IC.v);
	printf("IC.x =%f\n",IC.x);
	printf("FC.a =%f\n",FC.a);
	printf("FC.v =%f\n",FC.v);
	printf("FC.x =%f\n",FC.x);
	return XARM_ERROR;
      }
      DirTransition[1] = TrajectoryType;
      TotalTime [1] = (AxisTimes.PY.Tjpa + AxisTimes.PY.Taca + AxisTimes.PY.Tjna + AxisTimes.PY.Tvc + AxisTimes.PY.Tjnb + AxisTimes.PY.Tacb + AxisTimes.PY.Tjpb);
      if (TotalTime [1] > *MaxTime) {
	*MaxTime = TotalTime [1];
	*MaxTimeAxis = 1;
	T_Jerk.Tjpa =  AxisTimes.PY.Tjpa;
	T_Jerk.Taca =  AxisTimes.PY.Taca;
	T_Jerk.Tjna =  AxisTimes.PY.Tjna;
	T_Jerk.Tvc  =  AxisTimes.PY.Tvc;
	T_Jerk.Tjnb =  AxisTimes.PY.Tjnb;
	T_Jerk.Tacb =  AxisTimes.PY.Tacb;
	T_Jerk.Tjpb =  AxisTimes.PY.Tjpb;
      }
    }
    if (i==2) { 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes.PZ), &TrajectoryType)!=0) {
	printf("ERROR Jerk Profile Transition Z\n");
	printf("IC.a =%f\n",IC.a);
	printf("IC.v =%f\n",IC.v);
	printf("IC.x =%f\n",IC.x);
	printf("FC.a =%f\n",FC.a);
	printf("FC.v =%f\n",FC.v);
	printf("FC.x =%f\n",FC.x);
	return XARM_ERROR;
      }
      DirTransition[2] = TrajectoryType;
      TotalTime[2] = (AxisTimes.PZ.Tjpa + AxisTimes.PZ.Taca + AxisTimes.PZ.Tjna + AxisTimes.PZ.Tvc + AxisTimes.PZ.Tjnb + AxisTimes.PZ.Tacb + AxisTimes.PZ.Tjpb); 
      if (TotalTime[2] > *MaxTime) {
	*MaxTime = TotalTime[2];
	*MaxTimeAxis = 2;
	T_Jerk.Tjpa =  AxisTimes.PZ.Tjpa;
	T_Jerk.Taca =  AxisTimes.PZ.Taca;
	T_Jerk.Tjna =  AxisTimes.PZ.Tjna;
	T_Jerk.Tvc  =  AxisTimes.PZ.Tvc;
	T_Jerk.Tjnb =  AxisTimes.PZ.Tjnb;
	T_Jerk.Tacb =  AxisTimes.PZ.Tacb;
	T_Jerk.Tjpb =  AxisTimes.PZ.Tjpb;
      }
    }
    if (i==3) { 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes.QN), &TrajectoryType)!=0) {
	printf("ERROR Jerk Profile Transition QN\n");
	printf("IC.a =%f\n",IC.a);
	printf("IC.v =%f\n",IC.v);
	printf("IC.x =%f\n",IC.x);
	printf("FC.a =%f\n",FC.a);
	printf("FC.v =%f\n",FC.v);
	printf("FC.x =%f\n",FC.x);
	return XARM_ERROR;
      }
      DirTransition[3] = TrajectoryType;
      TotalTime[3] = (AxisTimes.QN.Tjpa + AxisTimes.QN.Taca + AxisTimes.QN.Tjna + AxisTimes.QN.Tvc + AxisTimes.QN.Tjnb + AxisTimes.QN.Tacb + AxisTimes.QN.Tjpb);

      if (TotalTime[3] > *MaxTime) {
	*MaxTime = TotalTime[3];
	*MaxTimeAxis = 3;
	T_Jerk.Tjpa =  AxisTimes.QN.Tjpa;
	T_Jerk.Taca =  AxisTimes.QN.Taca;
	T_Jerk.Tjna =  AxisTimes.QN.Tjna;
	T_Jerk.Tvc  =  AxisTimes.QN.Tvc;
	T_Jerk.Tjnb =  AxisTimes.QN.Tjnb;
	T_Jerk.Tacb =  AxisTimes.QN.Tacb;
	T_Jerk.Tjpb =  AxisTimes.QN.Tjpb;
      }
    }
    if (i==4) { 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes.QI), &TrajectoryType)!=0) {
	printf("ERROR Jerk Profile Transition QI\n");
	printf("IC.a =%f\n",IC.a);
	printf("IC.v =%f\n",IC.v);
	printf("IC.x =%f\n",IC.x);
	printf("FC.a =%f\n",FC.a);
	printf("FC.v =%f\n",FC.v);
	printf("FC.x =%f\n",FC.x);
	return XARM_ERROR;
      }
      DirTransition[4] = TrajectoryType;
      TotalTime[4] = (AxisTimes.QI.Tjpa + AxisTimes.QI.Taca + AxisTimes.QI.Tjna + AxisTimes.QI.Tvc + AxisTimes.QI.Tjnb + AxisTimes.QI.Tacb + AxisTimes.QI.Tjpb);
      if (TotalTime[4] > *MaxTime) {
	*MaxTime = TotalTime[4];
	*MaxTimeAxis = 4;
	T_Jerk.Tjpa =  AxisTimes.QI.Tjpa;
	T_Jerk.Taca =  AxisTimes.QI.Taca;
	T_Jerk.Tjna =  AxisTimes.QI.Tjna;
	T_Jerk.Tvc  =  AxisTimes.QI.Tvc;
	T_Jerk.Tjnb =  AxisTimes.QI.Tjnb;
	T_Jerk.Tacb =  AxisTimes.QI.Tacb;
	T_Jerk.Tjpb =  AxisTimes.QI.Tjpb;
      }
    }
    if (i==5) { 
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes.QJ), &TrajectoryType)!=0) {
	printf("ERROR Jerk Profile Transition QJ\n");
	printf("IC.a =%f\n",IC.a);
	printf("IC.v =%f\n",IC.v);
	printf("IC.x =%f\n",IC.x);
	printf("FC.a =%f\n",FC.a);
	printf("FC.v =%f\n",FC.v);
	printf("FC.x =%f\n",FC.x);
	return XARM_ERROR;
      }
      DirTransition[5] = TrajectoryType;
      TotalTime[5] = (AxisTimes.QJ.Tjpa + AxisTimes.QJ.Taca + AxisTimes.QJ.Tjna + AxisTimes.QJ.Tvc + AxisTimes.QJ.Tjnb + AxisTimes.QJ.Tacb + AxisTimes.QJ.Tjpb);
      if (TotalTime[5] > *MaxTime) {
	*MaxTime = TotalTime[5];
	*MaxTimeAxis = 5;
	T_Jerk.Tjpa =  AxisTimes.QJ.Tjpa;
	T_Jerk.Taca =  AxisTimes.QJ.Taca;
	T_Jerk.Tjna =  AxisTimes.QJ.Tjna;
	T_Jerk.Tvc  =  AxisTimes.QJ.Tvc;
	T_Jerk.Tjnb =  AxisTimes.QJ.Tjnb;
	T_Jerk.Tacb =  AxisTimes.QJ.Tacb;
	T_Jerk.Tjpb =  AxisTimes.QJ.Tjpb;
      }
    }
    if (i==6) {
      if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes.QK), &TrajectoryType)!=0) {
	printf("ERROR Jerk Profile Transition QK\n");
	printf("IC.a =%f\n",IC.a);
	printf("IC.v =%f\n",IC.v);
	printf("IC.x =%f\n",IC.x);
	printf("FC.a =%f\n",FC.a);
	printf("FC.v =%f\n",FC.v);
	printf("FC.x =%f\n",FC.x);
	return XARM_ERROR;
      }
      DirTransition[6] = TrajectoryType;
      TotalTime[6] = (AxisTimes.QK.Tjpa + AxisTimes.QK.Taca + AxisTimes.QK.Tjna + AxisTimes.QK.Tvc + AxisTimes.QK.Tjnb + AxisTimes.QK.Tacb + AxisTimes.QK.Tjpb);
      if (TotalTime[6] > *MaxTime) {
	*MaxTime = TotalTime[6];
	*MaxTimeAxis = 6;
	T_Jerk.Tjpa =  AxisTimes.QK.Tjpa;
	T_Jerk.Taca =  AxisTimes.QK.Taca;
	T_Jerk.Tjna =  AxisTimes.QK.Tjna;
	T_Jerk.Tvc  =  AxisTimes.QK.Tvc;
	T_Jerk.Tjnb =  AxisTimes.QK.Tjnb;
	T_Jerk.Tacb =  AxisTimes.QK.Tacb;
	T_Jerk.Tjpb =  AxisTimes.QK.Tjpb;
      }
    }
  }

  /* printf("\n"); */
  /* printf("MaxTimeAxis %d\n",MaxTimeAxis); */
  /* printf("MaxTime  %f\n",MaxTime); */
  /* printf("T_Jerk.Tjpa %f\n", T_Jerk.Tjpa); */
  /* printf("T_Jerk.Taca %f\n", T_Jerk.Taca); */
  /* printf("T_Jerk.Tjna %f\n", T_Jerk.Tjna); */
  /* printf("T_Jerk.Tvc %f\n", T_Jerk.Tvc); */
  /* printf("T_Jerk.Tjnb %f\n", T_Jerk.Tjnb); */
  /* printf("T_Jerk.Tacb %f\n", T_Jerk.Tacb); */
  /* printf("T_Jerk.Tjpb %f\n", T_Jerk.Tjpb); */
  /* printf("\n"); */
  return XARM_OK;
}
