/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 ** xarmJerkProfile.h
 **
 ** Author: Xavier Broquère
 ** Date: June 2007
 **
 **/


#ifndef XARMJERKPROFILE_H 
#define XARMJERKPROFILE_H 

#include "xarmConst.h"
#include "xarmStruct.h"
#include <stdlib.h>
#include "tpConst.h"
#include "tpStruct.h"
#include <stdio.h>
#include <math.h>



#ifndef ABS
#define ABS(a)        (((a) < 0) ? (-(a)) : (a))
#endif



XARM_STATUS VerifyInitialAndFinalConditions(TP_LIMITS* limitsGoto, TP_COND* IC, TP_COND* FC, TP_PARTICULAR_VELOCITY* PartVel, TP_COND* ICm, TP_COND* FCm);

XARM_STATUS CalculOfCriticalLength( TP_LIMITS* limitsGoto, TP_PARTICULAR_VELOCITY* PartVel, TP_COND* IC, TP_COND* FC, double* dc, int* zone); 

/************************************************/
/*             Functions of first type          */
/************************************************/

XARM_STATUS CalculOfDSVmaxType1(TP_LIMITS* limitsGoto, TP_PARTICULAR_VELOCITY* PartVel, TP_COND* ICm, TP_COND* FCm, double* DSVmax);

XARM_STATUS CalculOfDSAmaxType1(TP_LIMITS* limitsGoto, TP_PARTICULAR_VELOCITY* PartVel, TP_COND* ICm, TP_COND* FCm, double* DSAmax);

XARM_STATUS VerifyTimesType1(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_TIMES* T, double epsilon_erreur);

XARM_STATUS jerkProfile_Type1_VmaxAmin_7seg_maxi(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_VmaxAmin_5seg_maxi(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_VmaxAmin_6seg_maxi(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_Vmax_5seg_maxi(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_Vmax_6seg_maxi(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_VmaxAmin_4seg(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_inf_DSAmax_Z0(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_inf_DSAmax_Z1(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T, double* dc);

XARM_STATUS jerkProfile_Type1_inf_DSAmax_Z2(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T, double* dc);

XARM_STATUS Calcul_Of_DSAmax2_2_Type1(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, double* DSAmaxT12_2);

XARM_STATUS Calcul_Of_DSAmax2_22_Type1(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, double *DSAmaxT12_22);

XARM_STATUS jerkProfile_Type1_inf_DSAmax_inf_DSAmaxT12_22_Z2(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_inf_DSAmax_Z3(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_TIMES* T);

XARM_STATUS Calcul_Of_DSAmax2_31_Type1(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, double* DSAmaxT12_31);

XARM_STATUS Calcul_Of_DSAmax2_32_Type1(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, double* DSAmaxT12_32);

XARM_STATUS Calcul_Of_DSAmax2_33_Type1(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, double* DSAmaxT12_33);

XARM_STATUS Calcul_Of_DSAmax2_4_Type1(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, double *dc, double* DSAmaxT12_4);

XARM_STATUS jerkProfile_Type1_inf_DSAmax_sup_DSAmax2_31_Z3(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_inf_DSAmax_sup_DSAmax2_32_Z3(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, TP_TIMES* T);

XARM_STATUS jerkProfile_Type1_inf_DSAmax_sup_DSAmax2_33_Z3(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm,  TP_TIMES* T);

XARM_STATUS Jerk_Profile_dc(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_PARTICULAR_VELOCITY* PartVel, int *zone, TP_TIMES* T);


XARM_STATUS VerifyTimesType2(TP_LIMITS* limitsGoto, TP_COND* ICm, TP_COND* FCm, TP_TIMES* T);

/*************************************************/
/*                  Main Functions               */
/*************************************************/

extern int xarmJerkProfile(TP_COND IC, TP_COND FC, TP_LIMITS limitsGoto, TP_TIMES *T_Jerk, int *TrajectoryType);

extern XARM_STATUS xarmCalculOfCriticalLength(TP_COND IC, TP_COND FC, TP_LIMITS limitsGoto, double* dc);

XARM_STATUS xarmAdjustTimeSlowingJerk(TP_COND IC, TP_COND FC,double MaxTime, TP_LIMITS Limits, TP_TIMES *T, double* Jerk, TP_LIMITS* newLimits, int* DirTransition);

XARM_STATUS xarmCalculTimeProfileWithVcFixed(double V0, double Vf, double Vc, TP_LIMITS Limits, TP_TIMES_GLOBAL *T, int* dir_a, int* dir_b);

XARM_STATUS xarmCalculTimeProfileWithVcFixedWithAcc(double V0, double Vf, double A0, double Af, double Vc, TP_LIMITS Limits, TP_TIMES_GLOBAL *T, int* dir_a, int* dir_b);

XARM_STATUS xarmCalculOfDistance( double V0, double Jmax, TP_TIMES_GLOBAL* T, int* dir_a, int* dir_b, double* FinalDist);

XARM_STATUS xarmAdjustTimeSlowingVelocity(double V0, double Vf, double GoalDist, TP_LIMITS Limits, double TimeImp, TP_TIMES *T, int* dir_a, int* dir_b);

XARM_STATUS xarmAdjustTimeSlowingVelocityWithAcc(double V0, double Vf,double A0, double Af, double GoalDist, TP_LIMITS Limits, double TimeImp, TP_TIMES *T, int* dir_a, int* dir_b);

extern XARM_STATUS xarmAdjustTime(TP_COND IC, TP_COND FC,double MaxTime, TP_LIMITS Limits, TP_TIMES *T, double* Jerk, int* DirTransition_a, int* DirTransition_b);

extern XARM_STATUS xarmAdjustTimeWithAcc(TP_COND IC, TP_COND FC,double MaxTime, TP_LIMITS Limits, TP_TIMES *T, double* Jerk, int* DirTransition_a, int* DirTransition_b);

extern XARM_STATUS xarmVerifyTimes_Dir_ab(double dist_error, double GD, TP_JERKS Jerks, TP_COND IC, int Dir_a, int Dir_b, TP_TIMES T, TP_COND *FC, TP_TIMES *Acc, TP_TIMES *Vel, TP_TIMES *Pos);


//extern XARM_STATUS findTransitionTime(TP_COND IC[XARM_NB_DIM], TP_COND FC[XARM_NB_DIM], TP_POSELIMITS limitsGoto, int stopTimeM, int optimalTimeM[XARM_NB_DIM], int *impTimeM, TP_TIMES Times[XARM_NB_DIM], int Dir_a[XARM_NB_DIM], int Dir_b[XARM_NB_DIM]);

extern XARM_STATUS findTransitionTime( TP_POSELIMITS limitsGoto, TP_TRANSITION_MOTION* motion, int *impTimeM);

#endif
