/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 ** xarmTrackPoseCodels.c
 **
 ** Codels called by execution task xarmTrackPose
 **
 ** Author: 
 ** Date: 
 **
 **/

#include <portLib.h>
#include <stdlib.h>
#include "stdio.h"
#include "server/xarmHeader.h"
#include <math.h>
#include "server/qarmPosterLib.h"
#include "xarmConst.h"
#include "tpConst.h"
#include "Proto_gb.h"
#include "Proto_gbModeles.h"
#include "libeuler/pomEuler.h"
#include "xarmFunctions.h"
#include "tpFunctions.h"
#include "cFunctions.h"
#include "xarmJerkprofile.h"
//#include "xarmJerkprofile.c"
#include "queue.h"

/* Variable to point to point motions */
static dmat PFrom;                               /* Position From */
static dmat PTo;                                 /* Position To   */
static dmat QFrom;                               /* Quaternion From */
static dmat QTo;                                 /* Quaternion To */
static dmat Dist;                                /* Distance = To - From */
static dmat AbsDist;                             /* Distance Magnitud */
static dmat Dir;                                 /* Motion Direction */
static dmat ADist;                               /* Initial Angular Distance */ 
static dmat LDist;                               /* Initial Linear Distance */ 
static dmat AbsADist;                            /* Initial Absolute Angular Distance */ 
static dmat AbsLDist;                            /* Initial Absolute Linear Distance */ 
static int errnoMatDef;                          /* Error Auxiliar Variable */
static double MaxLDist;                          /* Maximum Linear Distance */
static double MaxADist;                          /* Maximum Angular Distance */
static double MaxDist;                           /* Maximum Distance */
static double Jmax, Amax, Vmax;


//   typedef struct TP_MOTION {
//     TP_TIMES Times[XARM_NB_DIM];
//     TP_TIMES TimesM[XARM_NB_DIM];
//     TP_TIMES Acc[XARM_NB_DIM];
//     TP_TIMES Vel[XARM_NB_DIM];
//     TP_TIMES Pos[XARM_NB_DIM];
//     TP_JERKS jerk[XARM_NB_DIM];
//     TP_COND  IC[XARM_NB_DIM];
//     TP_COND  FC[XARM_NB_DIM];
//     int      Dir[XARM_NB_DIM];
//     int      MotionDuration[XARM_NB_DIM];
//   } TP_MOTION;

static TP_MOTION motion1;
static TP_MOTION motion2;
static TP_TRANSITION_MOTION motionTr;


static TP_POSELIMITS  limitsExternalTrack;       /* Vel, Acc and Jerk limits for Goto functions*/
static Gb_q6 qExtRef;
static TP_POSE poseExtRef;
static POSTER_ID posterMhpTraj = NULL;
static XARM_EXTERNAL_TRACKING  MhpTraj;          /* MHP Trajectory */


/* Global trajectory variable */
//static int TrackPosePeriod = 5;
static int indexTrajectory;
static int timeToSwitch;                         /* Duration of motion sended to motion task (unit: s.10) */
//static double NB_TICK = 10.0;
static int timeAddedToSegments;
static int percentSwitch = 30;

    
static int mhpTrajectoryID;
static int motionIsStopped;

static int  RealTimeTrackPoseExecMode;

/*------------------------------------------------------------------------
 *
 * xarmTrackPoseEnd  --  Termination codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
xarmTrackPoseEnd(void)
{
  return OK;
}

/*------------------------------------------------------------------------
 *
 * xarmTrackPosePerm  --  1st codel of permanent activity (called BEFORE other activities)
 *
 * Description: 
 * 
 * Reports:      OK
 *              S_xarm_ERROR_MODE
 *              S_xarm_POSTER_NOT_FOUND
 *              S_xarm_POSTER_NOT_READ
 *              S_xarm_WRONG_VALUE
 *              S_xarm_ERROR_TP
 * 
 * Returns:    OK or ERROR
 */

STATUS
xarmTrackPosePerm(int *report)
{
  /* ... add your code here ... */
  return OK;
}

#define NEWDMAT_IS_OK(testError, mess, m_report)		  \
  do {								  \
    if (testError) {						  \
      PRINT_ERROR(mess);					  \
      *report = m_report;					  \
      return ETHER;						  \
    }								  \
  } while(0)							  \

/*------------------------------------------------------------------------
 * RealTimeTrackPose
 *
 * Description: 
 *
 * Reports:      OK
 *              S_xarm_POSTER_NOT_FOUND
 *              S_xarm_POSTER_NOT_READ
 *              S_xarm_ERROR_TP
 */

/* xarmRealTimeTrackPoseStart  -  codel START of RealTimeTrackPose
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmRealTimeTrackPoseStart(XARM_STRING *posPosterName, int *report)
{
  int i;

  /* We do nothing if xarm is sweet stopping */
  if (SDI_F->SweetStop == 1) {
    printf("XARM is sweet stopping\n");
    return START;
  }

  SDI_F->currentTimeOfTrajectory = 0;

 /* Find Poster */
  if (posterFind(posPosterName->str, &posterMhpTraj) == ERROR) {
    PRINT_ERROR("Can not find POSTER TRAJECTORY");
    *report =  S_xarm_POSTER_NOT_FOUND;
    return ETHER;
  }

  /* Verify poster NON NULL */
  if (posterMhpTraj == NULL) {
    PRINT_ERROR("Poster of planned trajectroy NULL");
    *report = S_xarm_POSTER_NOT_FOUND;
    return ETHER;
  }

  if (xarmXARM_EXTERNAL_TRACKINGPosterRead(posterMhpTraj, &MhpTraj)!=OK) {
    PRINT_ERROR("Can not read POSTER");
    *report =  S_xarm_POSTER_NOT_READ;
    SDI_F->SweetStop = 1;
    return ETHER;
  }

  /* set limits motion constraints and verify ratios */
  limitsExternalTrack.linear.maxJerk = SDI_F->limitsGoto.linear.maxJerk * MhpTraj.limitsRatio / 100.0; 
  limitsExternalTrack.linear.maxAcc = SDI_F->limitsGoto.linear.maxAcc * MhpTraj.limitsRatio / 100.0; 
  limitsExternalTrack.linear.maxVel = SDI_F->limitsGoto.linear.maxVel * MhpTraj.limitsRatio / 100.0; 

  limitsExternalTrack.angular.maxJerk = SDI_F->limitsGoto.angular.maxJerk * MhpTraj.limitsRatio / 100.0; 
  limitsExternalTrack.angular.maxAcc = SDI_F->limitsGoto.angular.maxAcc * MhpTraj.limitsRatio / 100.0; 
  limitsExternalTrack.angular.maxVel = SDI_F->limitsGoto.angular.maxVel * MhpTraj.limitsRatio / 100.0; 

  Jmax =  limitsExternalTrack.linear.maxJerk;
  Amax =  limitsExternalTrack.linear.maxAcc;
  Vmax =  limitsExternalTrack.linear.maxVel;
  if (Jmax<= 0 || Amax<= 0 || Vmax <= 0) {
    PRINT_ERROR("TrackingTrajectoryStart - ERROR - Jmax, Amax, Vmax Negatives or Zero - Linear");
    *report =  S_xarm_WRONG_VALUE; 
    return ETHER;
  } 
  if (Amax/Jmax  > Vmax/Amax) {
    PRINT_ERROR("TrackingTrajectoryStart - ERROR - Jmax, Amax, Vmax Values - Linear");
    *report =  S_xarm_WRONG_VALUE; 
    return ETHER;
  }
  Jmax =  limitsExternalTrack.angular.maxJerk;
  Amax =  limitsExternalTrack.angular.maxAcc;
  Vmax =  limitsExternalTrack.angular.maxVel;
  if (Jmax<= 0 || Amax<= 0 || Vmax <= 0) {
    PRINT_ERROR("TrackingTrajectoryStart - ERROR - Jmax, Amax, Vmax Negatives or Zero - Angular");
    *report =  S_xarm_WRONG_VALUE; 
    return ETHER;
  } 
  if (Amax/Jmax  > Vmax/Amax) {
    PRINT_ERROR("TrackingTrajectoryStart - ERROR - Jmax, Amax, Vmax Values - Angular");
    *report =  S_xarm_WRONG_VALUE; 
    return ETHER;
  }

 /* Initial PFrom and PTo */
  PFrom = newdmat(0,0,0,XARM_POS_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "QTo Definition", S_xarm_ERROR_TP);
  PTo = newdmat(0,0,0,XARM_POS_DIM-1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "PTo Definition", S_xarm_ERROR_TP);
  /* Initial QFrom and QTo */
  QFrom = newdmat(0,0,0,XARM_QUAT_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "QFrom Definition", S_xarm_ERROR_TP);
  QTo = newdmat(0,0,0,XARM_QUAT_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "QTo Definition", S_xarm_ERROR_TP);
  /* Distance Vector Definition */
  Dist = newdmat (0,0,0,XARM_NB_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Dist Definition", S_xarm_ERROR_TP);
  /* Absolute Distance Vector Definition */
  AbsDist = newdmat (0,0,0,XARM_NB_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "AbsDist Definition", S_xarm_ERROR_TP);
  /* Dir Vector Definition */
  Dir = newdmat (0,0,0,XARM_NB_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Dir Definition", S_xarm_ERROR_TP);
  /* Linear Distance Vector Definition */
  LDist = newdmat (0,0,0,XARM_POS_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Linear Distance Definition ", S_xarm_ERROR_TP);
  /* Absolute Linear Distance Vector Definition */
  AbsLDist = newdmat (0,0,0,XARM_POS_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Absolute Linear Distance Definition", S_xarm_ERROR_TP);
  /* Angular Distance Vector Definition */
  ADist = newdmat (0,0,0,XARM_QUAT_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Angular Distance Definition", S_xarm_ERROR_TP);
  /* Absolute Distance Vector Definition */
  AbsADist = newdmat (0,0,0,XARM_QUAT_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Absolute Angular Distance Definition", S_xarm_ERROR_TP);

  SDI_F->currentTimeOfTrajectory = 0;

  /* Lecture du poster */
  indexTrajectory = 0;

  if (xarmXARM_EXTERNAL_TRACKINGPosterRead(posterMhpTraj, &MhpTraj)!=OK) {
    PRINT_ERROR("Can not read POSTER");
    *report =  S_xarm_POSTER_NOT_READ;
    SDI_F->SweetStop = 1;
    return ETHER;
  }

  if ( MhpTraj.posterIsInitialized == 0) {
    printf("Mhp poster is not initialized\n");
    return ETHER;
  }

  qExtRef.q1 = TRUNC(MhpTraj.qRef[indexTrajectory].q1);
  qExtRef.q2 = TRUNC(MhpTraj.qRef[indexTrajectory].q2);  
  qExtRef.q3 = TRUNC(MhpTraj.qRef[indexTrajectory].q3); 
  qExtRef.q4 = TRUNC(MhpTraj.qRef[indexTrajectory].q4); 
  qExtRef.q5 = TRUNC(MhpTraj.qRef[indexTrajectory].q5); 
  qExtRef.q6 = TRUNC(MhpTraj.qRef[indexTrajectory].q6); 
  
  /* Compute the cartesian coordinates */
  xarmMGDQuat(&qExtRef, &poseExtRef);

  /*=====================================================*/
  /*                 First Trajectory                    */
  /*=====================================================*/ 
  
  /*Init From and To */
  /* Position From = Current Position */
  PFrom.el[0][0] = SDI_F->pose.p.x;
  PFrom.el[0][1] = SDI_F->pose.p.y;
  PFrom.el[0][2] = SDI_F->pose.p.z;
  /* Quaternion From = Current Orientation */
  QFrom.el[0][0] = SDI_F->pose.q.n;
  QFrom.el[0][1] = SDI_F->pose.q.i;  
  QFrom.el[0][2] = SDI_F->pose.q.j;
  QFrom.el[0][3] = SDI_F->pose.q.k;
  
  /* Position To = Goal Position 1 */
  PTo.el[0][0] = TRUNC(poseExtRef.p.x );
  PTo.el[0][1] = TRUNC(poseExtRef.p.y );
  PTo.el[0][2] = TRUNC(poseExtRef.p.z ); 
  /* Quaternion To = Goal Orientation 1 */
  QTo.el[0][0] = TRUNC(poseExtRef.q.n );  
  QTo.el[0][1] = TRUNC(poseExtRef.q.i );
  QTo.el[0][2] = TRUNC(poseExtRef.q.j );  
  QTo.el[0][3] = TRUNC(poseExtRef.q.k ); 


  motion1.IC[0].a = 0;
  motion1.IC[0].v = 0;
  motion1.IC[0].x = PFrom.el[0][0];
  motion1.IC[1].a = 0;
  motion1.IC[1].v = 0;
  motion1.IC[1].x = PFrom.el[0][1];
  motion1.IC[2].a = 0;
  motion1.IC[2].v = 0;
  motion1.IC[2].x = PFrom.el[0][2];
  motion1.IC[3].a = 0;
  motion1.IC[3].v = 0;
  motion1.IC[3].x = QFrom.el[0][0];
  motion1.IC[4].a = 0;
  motion1.IC[4].v = 0;
  motion1.IC[4].x = QFrom.el[0][1];
  motion1.IC[5].a = 0;
  motion1.IC[5].v = 0;
  motion1.IC[5].x = QFrom.el[0][2];
  motion1.IC[6].a = 0;
  motion1.IC[6].v = 0;
  motion1.IC[6].x = QFrom.el[0][3];

  motion1.FC[0].a = 0;
  motion1.FC[0].v = 0;
  motion1.FC[0].x = PTo.el[0][0];
  motion1.FC[1].a = 0;
  motion1.FC[1].v = 0;
  motion1.FC[1].x = PTo.el[0][1];
  motion1.FC[2].a = 0;
  motion1.FC[2].v = 0;
  motion1.FC[2].x = PTo.el[0][2];
  motion1.FC[3].a = 0;
  motion1.FC[3].v = 0;
  motion1.FC[3].x = QTo.el[0][0];
  motion1.FC[4].a = 0;
  motion1.FC[4].v = 0;
  motion1.FC[4].x = QTo.el[0][1];
  motion1.FC[5].a = 0;
  motion1.FC[5].v = 0;
  motion1.FC[5].x = QTo.el[0][2];
  motion1.FC[6].a = 0;
  motion1.FC[6].v = 0;
  motion1.FC[6].x = QTo.el[0][3];

  printf("START\n");
  printf("PFrom: %f %f %f %f %f %f %f\n",PFrom.el[0][0], PFrom.el[0][1], PFrom.el[0][2], QFrom.el[0][0], QFrom.el[0][1], QFrom.el[0][2], QFrom.el[0][3]);
  printf("PTo: %f %f %f %f %f %f %f\n",PTo.el[0][0], PTo.el[0][1], PTo.el[0][2], QTo.el[0][0], QTo.el[0][1], QTo.el[0][2], QTo.el[0][3]);

  if(GeometricalCalculations(PFrom, PTo, QFrom, QTo, &MaxLDist, &MaxADist, Dist, LDist, AbsLDist, ADist, AbsADist, AbsDist, Dir, report)!=OK) {
    PRINT_ERROR("xarm ERROR GeometricalCalculations\n");
    return ETHER;
  }
  CalculPointToPointJerkProfileV2(PFrom, QFrom, limitsExternalTrack, MaxLDist, MaxADist, MaxDist, AbsDist, Dir, &motion1, report);


//  printf("==================================================== \n");
//  printf("motion1 \n");
//  for(i=0;i<=XARM_NB_DIM-1;i++) {
//    printf("IC %f %f %f      FC  %f %f %f \n ",motion1.IC[i].a,motion1.IC[i].v,motion1.IC[i].x,motion1.FC[i].a,motion1.FC[i].v,motion1.FC[i].x);
//    printf("motion1.Pos[i].Tjpa %f\n", motion1.Pos[i].Tjpa);
//    printf("motion1.Pos[i].Taca %f\n", motion1.Pos[i].Taca);
//    printf("motion1.Pos[i].Tjna %f\n", motion1.Pos[i].Tjna);
//    printf("motion1.Pos[i].Tvc %f\n", motion1.Pos[i].Tvc);
//    printf("motion1.Pos[i].Tjnb %f\n", motion1.Pos[i].Tjnb);
//    printf("motion1.Pos[i].Tacb %f\n", motion1.Pos[i].Tacb); 
//    printf("motion1.Pos[i].Tjpb %f\n", motion1.Pos[i].Tjpb);  
//  }
//  printf("motion1.TNE.Tjpa %f\n",motion1.TNE.Tjpa);
//  printf("motion1.TNE.Taca %f\n",motion1.TNE.Taca);
//  printf("motion1.TNE.Tjna %f\n",motion1.TNE.Tjna);
//  printf("motion1.TNE.Tvc %f\n",motion1.TNE.Tvc);
//  printf("motion1.TNE.Tjnb %f\n",motion1.TNE.Tjnb);
//  printf("motion1.TNE.Tacb %f\n",motion1.TNE.Tacb);
//  printf("motion1.TNE.Tjpb %f\n",motion1.TNE.Tjpb);
//  printf("==================================================== \n");

  for(i=0;i<=XARM_NB_DIM-1;i++) {
    motion1.Dir[i] = Dir.el[0][i];
    /* We send segments to start the motion */
    queueAddDetails(&(SDI_F->Seg[i]), 1, motion1.TNE.Tjpa, motion1.jerk[i].J1, 0,                   0,                   motion1.IC[i].x,     motion1.Dir[i]);
    queueAddDetails(&(SDI_F->Seg[i]), 2, motion1.TNE.Taca, motion1.jerk[i].J1, motion1.Acc[i].Tjpa, motion1.Vel[i].Tjpa, motion1.Pos[i].Tjpa, motion1.Dir[i]);
    queueAddDetails(&(SDI_F->Seg[i]), 3, motion1.TNE.Tjna, motion1.jerk[i].J1, motion1.Acc[i].Taca, motion1.Vel[i].Taca, motion1.Pos[i].Taca, motion1.Dir[i]);
    queueAddDetails(&(SDI_F->Seg[i]), 4, motion1.TNE.Tvc,  motion1.jerk[i].J1, motion1.Acc[i].Tjna, motion1.Vel[i].Tjna, motion1.Pos[i].Tjna, motion1.Dir[i]);
  }

  timeAddedToSegments  = motion1.TNE.Tjpa + motion1.TNE.Taca + motion1.TNE.Tjna + motion1.TNE.Tvc;

  timeToSwitch = (int)(percentSwitch * timeAddedToSegments) / 100 ;
  printf("timeAddedToSegments %d timeToSwitch %d\n",timeAddedToSegments,timeToSwitch);
  /* Increment poster */
  SDI_F->dq.counter += 1;

  if ( MhpTraj.nbPoint > 1  ) {

    // indexTrajectory = 1; added to INTER
    SDI_F->xarmTrackSegment = XARM_TRACK_ON;
    printf("XARM  RealTimeTrackPose return EXEC and motion starts\n");
    RealTimeTrackPoseExecMode = 0;
    motionIsStopped = 0;
    mhpTrajectoryID = 0;
    SDI_F->extTrackingState.currentPoint = indexTrajectory;
    return EXEC;

  } else {
    /* We send segments to stop the trajectory */
    for(i=0;i<=XARM_NB_DIM-1;i++) {
      queueAddDetails(&(SDI_F->Seg[i]), 5, motion1.TNE.Tjnb, motion1.jerk[i].J1, motion1.Acc[i].Tvc , motion1.Vel[i].Tvc , motion1.Pos[i].Tvc , motion1.Dir[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 6, motion1.TNE.Tacb, motion1.jerk[i].J1, motion1.Acc[i].Tjnb, motion1.Vel[i].Tjnb, motion1.Pos[i].Tjnb, motion1.Dir[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 7, motion1.TNE.Tjpb, motion1.jerk[i].J1, motion1.Acc[i].Tacb, motion1.Vel[i].Tacb, motion1.Pos[i].Tacb, motion1.Dir[i]);
    }
    
    printf("XARM RealTimeTrackPose SINGLE POSE return END\n");
    SDI_F->xarmTrackSegment = XARM_TRACK_ON;
    return END; 
  }
}

/* xarmRealTimeTrackPoseMain  -  codel EXEC of RealTimeTrackPose
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmRealTimeTrackPoseMain(XARM_STRING *posPosterName, int *report)
{
  double cosTheta;
  TP_COND IC, FC;
  TP_TIMES TM;
  double distanceTolerance;
 
  //  double DiffTime;
  
  double maxTime;
  TP_LIMITS auxLimits;
  double GD;
  int NOE;
  int i;
 
  double maxTimeSec;
  int impTimeM;
  int  timeAdded = 0;
  int adjustJerkIsOk[XARM_NB_DIM];
  int adjustJerkIsOktt;

  if ( RealTimeTrackPoseExecMode == 0) {
  
    if (xarmXARM_EXTERNAL_TRACKINGPosterRead(posterMhpTraj, &MhpTraj)!=OK) {
      PRINT_ERROR("Can not read POSTER");
      *report =  S_xarm_POSTER_NOT_READ;
      SDI_F->SweetStop = 1;
      return END;
    }
  
    if ( MhpTraj.posterIsInitialized == 0) {
      printf("Mhp poster is not initialized\n");
      return END;
    }
  
    /* We do nothing if xarm is sweet stopping */
    if (SDI_F->SweetStop == 1) {
      printf("XARM is sweet stopping\n");
      SDI_F->currentTimeOfTrajectory = 0;
      // return START;
      return END;
    }
    
    if ((indexTrajectory == (MhpTraj.nbPoint - 1)) && motionIsStopped==0) {
      motionIsStopped = 1;
      
      /* We send segments to stop the trajectory */
      for(i=0;i<=XARM_NB_DIM-1;i++) {
	queueAddDetails(&(SDI_F->Seg[i]), 5, motion2.TNE.Tjnb, motion2.jerk[i].J1, motion2.Acc[i].Tvc , motion2.Vel[i].Tvc , motion2.Pos[i].Tvc , motion2.Dir[i]);
	queueAddDetails(&(SDI_F->Seg[i]), 6, motion2.TNE.Tacb, motion2.jerk[i].J1, motion2.Acc[i].Tjnb, motion2.Vel[i].Tjnb, motion2.Pos[i].Tjnb, motion2.Dir[i]);
	queueAddDetails(&(SDI_F->Seg[i]), 7, motion2.TNE.Tjpb, motion2.jerk[i].J1, motion2.Acc[i].Tacb, motion2.Vel[i].Tacb, motion2.Pos[i].Tacb, motion2.Dir[i]);
      }
    
      mhpTrajectoryID ++;
      SDI_F->extTrackingState.TrajectoryID = mhpTrajectoryID;
      /* SIGNALER A MHP LA FIN (MISE EN PAUSE) DU TRACKING */
      // return EXEC;
      return END;
    }
    
    if ((indexTrajectory == (MhpTraj.nbPoint - 1)) && motionIsStopped==1 && mhpTrajectoryID==MhpTraj.TrajectoryID) {
      indexTrajectory = 0;
      motionIsStopped = 0;
      SDI_F->currentTimeOfTrajectory = 0;
      // return START;
      return END;
    }
    
    if (SDI_F->currentTimeOfTrajectory < timeToSwitch) {
      printf("SDI_F->currentTimeOfTrajectory %d\n",SDI_F->currentTimeOfTrajectory );
      return EXEC;
    } 
    
    indexTrajectory ++;

    printf("========================\n");
    printf("  indexTrajectory  %d   \n",indexTrajectory);
    printf("========================\n");

    SDI_F->extTrackingState.currentPoint = indexTrajectory;
    
    
    RealTimeTrackPoseExecMode = 1;
    return EXEC;
  }

  if ( RealTimeTrackPoseExecMode == 1) {

    if (xarmXARM_EXTERNAL_TRACKINGPosterRead(posterMhpTraj, &MhpTraj)!=OK) {
      PRINT_ERROR("Can not read POSTER");
      *report =  S_xarm_POSTER_NOT_READ;
      SDI_F->SweetStop = 1;
      return END;
    }
    
    printf("MhpTraj.nbPoint %d \n",MhpTraj.nbPoint);
    for(i = 0; i<MhpTraj.nbPoint;i++) {
      printf("%d %f %f %f %f %f %f \n",i,TRUNC(MhpTraj.qRef[i].q1),TRUNC(MhpTraj.qRef[i].q2),TRUNC(MhpTraj.qRef[i].q3),TRUNC(MhpTraj.qRef[i].q4),TRUNC(MhpTraj.qRef[i].q5),TRUNC(MhpTraj.qRef[i].q6));
    }

    qExtRef.q1 = TRUNC(MhpTraj.qRef[indexTrajectory].q1);
    qExtRef.q2 = TRUNC(MhpTraj.qRef[indexTrajectory].q2);  
    qExtRef.q3 = TRUNC(MhpTraj.qRef[indexTrajectory].q3); 
    qExtRef.q4 = TRUNC(MhpTraj.qRef[indexTrajectory].q4); 
    qExtRef.q5 = TRUNC(MhpTraj.qRef[indexTrajectory].q5); 
    qExtRef.q6 = TRUNC(MhpTraj.qRef[indexTrajectory].q6); 
    
  /* Compute the cartesian coordinates */
  xarmMGDQuat(&qExtRef, &poseExtRef);
    
  /*=====================================================*/
  /*                Second Trajectory                    */
  /*=====================================================*/ 
    
  /* Position From = Goal Position 1 */
  PFrom.el[0][0] = motion1.FC[0].x;
  PFrom.el[0][1] = motion1.FC[1].x;
  PFrom.el[0][2] = motion1.FC[2].x;
  /* Quaternion From = Goal Orientation 1 */
  QFrom.el[0][0] = motion1.FC[3].x;
  QFrom.el[0][1] = motion1.FC[4].x;
  QFrom.el[0][2] = motion1.FC[5].x;
  QFrom.el[0][3] = motion1.FC[6].x;

  /* Position To = Goal Position 1 */
  PTo.el[0][0] = TRUNC(poseExtRef.p.x );
  PTo.el[0][1] = TRUNC(poseExtRef.p.y );
  PTo.el[0][2] = TRUNC(poseExtRef.p.z ); 
  /* Quaternion To = Goal Orientation 1 */
  QTo.el[0][0] = TRUNC(poseExtRef.q.n );  
  QTo.el[0][1] = TRUNC(poseExtRef.q.i );
  QTo.el[0][2] = TRUNC(poseExtRef.q.j );  
  QTo.el[0][3] = TRUNC(poseExtRef.q.k );

  motion2.IC[0].a = 0;
  motion2.IC[0].v = 0;
  motion2.IC[0].x = PFrom.el[0][0];
  motion2.IC[1].a = 0;
  motion2.IC[1].v = 0;
  motion2.IC[1].x = PFrom.el[0][1];
  motion2.IC[2].a = 0;
  motion2.IC[2].v = 0;
  motion2.IC[2].x = PFrom.el[0][2];
  motion2.IC[3].a = 0;
  motion2.IC[3].v = 0;
  motion2.IC[3].x = QFrom.el[0][0];
  motion2.IC[4].a = 0;
  motion2.IC[4].v = 0;
  motion2.IC[4].x = QFrom.el[0][1];
  motion2.IC[5].a = 0;
  motion2.IC[5].v = 0;
  motion2.IC[5].x = QFrom.el[0][2];
  motion2.IC[6].a = 0;
  motion2.IC[6].v = 0;
  motion2.IC[6].x = QFrom.el[0][3];
    
  motion2.FC[0].a = 0;
  motion2.FC[0].v = 0;
  motion2.FC[0].x = PTo.el[0][0];
  motion2.FC[1].a = 0;
  motion2.FC[1].v = 0;
  motion2.FC[1].x = PTo.el[0][1];
  motion2.FC[2].a = 0;
  motion2.FC[2].v = 0;
  motion2.FC[2].x = PTo.el[0][2];
  motion2.FC[3].a = 0;
  motion2.FC[3].v = 0;
  motion2.FC[3].x = QTo.el[0][0];
  motion2.FC[4].a = 0;
  motion2.FC[4].v = 0;
  motion2.FC[4].x = QTo.el[0][1];
  motion2.FC[5].a = 0;
  motion2.FC[5].v = 0;
  motion2.FC[5].x = QTo.el[0][2];
  motion2.FC[6].a = 0;
  motion2.FC[6].v = 0;
  motion2.FC[6].x = QTo.el[0][3];

  printf("INDEX_TRAJECTORY %d\n",indexTrajectory);
  printf("PFrom: %f %f %f %f %f %f %f\n",PFrom.el[0][0], PFrom.el[0][1], PFrom.el[0][2], QFrom.el[0][0], QFrom.el[0][1], QFrom.el[0][2], QFrom.el[0][3]);
  printf("PTo: %f %f %f %f %f %f %f\n",PTo.el[0][0], PTo.el[0][1], PTo.el[0][2], QTo.el[0][0], QTo.el[0][1], QTo.el[0][2], QTo.el[0][3]);


  cosTheta = QFrom.el[0][0]*QTo.el[0][0] + QFrom.el[0][1]*QTo.el[0][1] + QFrom.el[0][2]*QTo.el[0][2] + QFrom.el[0][3]*QTo.el[0][3];
  if (cosTheta<0) {
    QTo.el[0][0] = -QTo.el[0][0];
    QTo.el[0][1] = -QTo.el[0][1];
    QTo.el[0][2] = -QTo.el[0][2];
    QTo.el[0][3] = -QTo.el[0][3];
  }
  if(GeometricalCalculations(PFrom, PTo, QFrom, QTo, &MaxLDist, &MaxADist, Dist, LDist, AbsLDist, ADist, AbsADist, AbsDist, Dir, report)!=OK) {
    PRINT_ERROR("xarm ERROR GeometricalCalculations\n");
    return ETHER;
  }
  CalculPointToPointJerkProfileV2(PFrom, QFrom, limitsExternalTrack, MaxLDist, MaxADist, MaxDist, AbsDist, Dir, &motion2, report); 
  // CalculPointToPointJerkProfile(PFrom, QFrom, limitsExternalTrack, MaxLDist, MaxADist, MaxDist, AbsDist, Dir, motion2.jerk, motion2.Acc, motion2.Vel, motion2.Pos, &(motion2.TNE), report);

  for(i=0;i<=XARM_NB_DIM-1;i++) {
    motion2.Dir[i] = Dir.el[0][i];
  }

  motionTr.timeToStop = motion1.TNE.Tjnb + motion1.TNE.Tacb + motion1.TNE.Tjpb + motion2.TNE.Tjnb + motion2.TNE.Tacb + motion2.TNE.Tjpb;

  /*=====================================================*/
  /*              Transition Trajectory                  */
  /*=====================================================*/   

  for (i=0;i<7;i++) {    
    motionTr.IC[i].a =  TRUNC(motion1.Acc[i].Tvc);
    motionTr.IC[i].v =  TRUNC(motion1.Vel[i].Tvc); 
    motionTr.IC[i].x =  TRUNC(motion1.Pos[i].Tvc); 

    motionTr.FC[i].a =  TRUNC(motion2.Acc[i].Tjna);
    motionTr.FC[i].v =  TRUNC(motion2.Vel[i].Tjna); 
    motionTr.FC[i].x =  TRUNC(motion2.Pos[i].Tjna); 
  }


  /* Compute optimal motion for each quaternion axis */
  for (i=0; i < XARM_NB_DIM; i++) {
      
    if (i<3) {
      auxLimits.maxJerk = limitsExternalTrack.linear.maxJerk;
      auxLimits.maxAcc = limitsExternalTrack.linear.maxAcc;
      auxLimits.maxVel = limitsExternalTrack.linear.maxVel;
      //sensibility = LINEAR_SENSIBILITY;
      distanceTolerance = DISTANCE_TOLERANCE_LINEAR;
	
      motionTr.jerk[i].J1 = limitsExternalTrack.linear.maxJerk; 
      motionTr.jerk[i].sel = 1;
	
    } else {
      auxLimits.maxJerk = limitsExternalTrack.angular.maxJerk;
      auxLimits.maxAcc = limitsExternalTrack.angular.maxAcc;
      auxLimits.maxVel = limitsExternalTrack.angular.maxVel;
      //sensibility = ANGULAR_SENSIBILITY;
      distanceTolerance = DISTANCE_TOLERANCE_ANGULAR;
	
      motionTr.jerk[i].J1 = limitsExternalTrack.angular.maxJerk; 
      motionTr.jerk[i].sel = 1;
    }
      
      
    IC.a =  motionTr.IC[i].a;
    IC.v =  motionTr.IC[i].v;
    IC.x =  0.0;

    FC.a = motionTr.FC[i].a;
    FC.v = motionTr.FC[i].v; 
    FC.x = (motionTr.FC[i].x - motionTr.IC[i].x);
    
	
    if (xarmJerkProfile(IC, FC, auxLimits, &(motionTr.Times[i]), &(motionTr.Dir[i]))!=0) {
      printf("ERROR Jerk Profile on dim %d\n",i);
      return ETHER;
    }  
     
    /* Get initial conditions for each vectors Acc Vel and Pos */
      
    GD =  FC.x * motionTr.Dir[i];
      
    if (tpVerifyTimes( distanceTolerance, GD, motionTr.jerk[i], motionTr.IC[i], motionTr.Dir[i], motionTr.Times[i], &FC, &(motionTr.Acc[i]), &(motionTr.Vel[i]), &(motionTr.Pos[i]), TP_ON)!=OK) {
      printf(" Verify Times on dim %d\n",i);
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }

    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes( motionTr.Times[i], &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &(motionTr.TimesM[i]));

    motionTr.optimalTime[i] = motionTr.TimesM[i].Tjpa + motionTr.TimesM[i].Taca + motionTr.TimesM[i].Tjna + motionTr.TimesM[i].Tvc + motionTr.TimesM[i].Tjnb + motionTr.TimesM[i].Tacb + motionTr.TimesM[i].Tjpb; 
   
  }

  maxTime = 0.0;

  for (i=0; i < XARM_NB_DIM;i++) {
    
    if ( motionTr.optimalTime[i] > maxTime) {
      maxTime = motionTr.optimalTime[i] ;
    }
    // printf("time[%d] = %f\n",i,(double)motionTr.optimalTime[i]);
  }

  maxTimeSec = maxTime / 100;


  /* Compute interval time where slowing velocity works */
  printf("TRANSITION\n"); 
  if (findTransitionTime(limitsExternalTrack, &motionTr,  &impTimeM) != OK) {
    PRINT_ERROR("xarm CANNOT adjust transition times");
  } 
  adjustJerkIsOktt =0;
  if (impTimeM == motionTr.timeToStop ) {
    printf("on diminue le jerk\n");
    for (i = 0; i< XARM_NB_DIM; i++) {
      IC.a = motionTr.IC[i].a;
      IC.v = motionTr.IC[i].v;
      IC.x = 0;
      FC.a = motionTr.FC[i].a;
      FC.v =  motionTr.FC[i].v;
      FC.x =  motionTr.FC[i].x - motionTr.IC[i].x ;
 
      if(xarmAdjustTime( IC, FC, maxTimeSec, limitsExternalTrack.linear, &(motionTr.Times[i]),&(motionTr.jerk[i].J1),&(motionTr.Dir_a[i]),&(motionTr.Dir_b[i]) )!=0) {
        printf("adjust jerk ERROR on axis %d \n",i);
	adjustJerkIsOk[i] = 0;
      }
      printf("adjust jerk OK on axis %d \n",i);
      adjustJerkIsOk[i] = 1;
      adjustJerkIsOktt ++; 
    }
    printf("\nadjustJerkIsOktt %d\n\n",adjustJerkIsOktt);
  }

//  for (i = 0; i<XARM_NB_DIM; i++) {
//   printf("\n");
//   printf("IC FC %d :   %f %f %f %f %f %f\n",i, motionTr.IC[i].a,  motionTr.IC[i].v, motionTr.IC[i].x, motionTr.FC[i].a,  motionTr.FC[i].v, motionTr.FC[i].x);
//   printf("Dir_a %d Dir_b %d \n",motionTr.Dir_a[i],motionTr.Dir_b[i]);
//   printf("impTimeM %d motionTr.timeToStop %d \n",impTimeM,motionTr.timeToStop);
//
//   printf("\n");
//  }  

  /* We send segments to the motion task */

  for (i=0; i < XARM_NB_DIM;i++) {

    if (motionTr.timeToStop <= impTimeM &&  adjustJerkIsOktt <7 ) {
      printf("Stop and start motion on axis %d\n",i);
      queueAddDetails(&(SDI_F->Seg[i]), 5, motion1.TNE.Tjnb, motion1.jerk[i].J1, motion1.Acc[i].Tvc , motion1.Vel[i].Tvc , motion1.Pos[i].Tvc , motion1.Dir[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 6, motion1.TNE.Tacb, motion1.jerk[i].J1, motion1.Acc[i].Tjnb, motion1.Vel[i].Tjnb, motion1.Pos[i].Tjnb, motion1.Dir[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 7, motion1.TNE.Tjpb, motion1.jerk[i].J1, motion1.Acc[i].Tacb, motion1.Vel[i].Tacb, motion1.Pos[i].Tacb, motion1.Dir[i]);
      //  queueAddDetails(&(SDI_F->Seg[i]), 4, (impTimeM - motionTr.timeToStop), motion1.jerk[i].J1, 0.0, 0.0, motion1.Pos[i].Tacb, motion1.Dir[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 1, motion2.TNE.Tjpa, motion2.jerk[i].J1, motion2.IC[i].a,     motion2.IC[i].v,     motion2.IC[i].x,     motion2.Dir[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 2, motion2.TNE.Taca, motion2.jerk[i].J1, motion2.Acc[i].Tjpa, motion2.Vel[i].Tjpa, motion2.Pos[i].Tjpa, motion2.Dir[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 3, motion2.TNE.Tjna, motion2.jerk[i].J1, motion2.Acc[i].Taca, motion2.Vel[i].Taca, motion2.Pos[i].Taca, motion2.Dir[i]);

      timeAdded = motion1.TNE.Tjnb +  motion1.TNE.Tacb + motion1.TNE.Tjpb + motion2.TNE.Tjpa + motion2.TNE.Taca +  motion2.TNE.Tjna ;

    } else {

      if (xarmVerifyTimes_Dir_ab(DISTANCE_TOLERANCE_LINEAR, (motionTr.FC[i].x - motionTr.IC[i].x), motionTr.jerk[i], motionTr.IC[i], motionTr.Dir_a[i], motionTr.Dir_b[i], motionTr.Times[i] , &FC, &(motionTr.Acc[i]), &(motionTr.Vel[i]), &(motionTr.Pos[i]))!=OK) {
	PRINT_ERROR(" Verify Times ");
	*report = S_xarm_ERROR_TP;
	return END;
      }

      /* Get Number of Elements needed for Vectors Definition */
      if (tpGetMonotonicTimes( motionTr.Times[i], &TM, &NOE)!=OK) {
	PRINT_ERROR(" Get MonotonicTimes ");
	*report = S_xarm_ERROR_TP;
	return END;
      }
      tpGetNumberOfElement (&TM, &(motionTr.TimesM[i]));

      queueAddDetails(&(SDI_F->Seg[i]), 1, motionTr.TimesM[i].Tjpa, motionTr.jerk[i].J1, motionTr.IC[i].a,     motionTr.IC[i].v,     motionTr.IC[i].x,      motionTr.Dir_a[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 2, motionTr.TimesM[i].Taca, motionTr.jerk[i].J1, motionTr.Acc[i].Tjpa, motionTr.Vel[i].Tjpa, motionTr.Pos[i].Tjpa,  motionTr.Dir_a[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 3, motionTr.TimesM[i].Tjna, motionTr.jerk[i].J1, motionTr.Acc[i].Taca, motionTr.Vel[i].Taca, motionTr.Pos[i].Taca,  motionTr.Dir_a[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 4, motionTr.TimesM[i].Tvc,  motionTr.jerk[i].J1, motionTr.Acc[i].Tjna, motionTr.Vel[i].Tjna, motionTr.Pos[i].Tjna,  motionTr.Dir_a[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 1, motionTr.TimesM[i].Tjnb, motionTr.jerk[i].J1, motionTr.Acc[i].Tvc,  motionTr.Vel[i].Tvc,  motionTr.Pos[i].Tvc,   motionTr.Dir_b[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 2, motionTr.TimesM[i].Tacb, motionTr.jerk[i].J1, motionTr.Acc[i].Tjnb, motionTr.Vel[i].Tjnb, motionTr.Pos[i].Tjnb,  motionTr.Dir_b[i]);
      queueAddDetails(&(SDI_F->Seg[i]), 3, motionTr.TimesM[i].Tjpb, motionTr.jerk[i].J1, motionTr.Acc[i].Tacb, motionTr.Vel[i].Tacb, motionTr.Pos[i].Tacb,  motionTr.Dir_b[i]);
    
      //  printf(" motionTr.TimesM[i].Tjpa %f  motionTr.TimesM[i].Taca %f motionTr.TimesM[i].Tvc %f motionTr.TimesM[i].Tjnb %f  motionTr.TimesM[i].Tacb %f  motionTr.TimesM[i].Tjpb %f\n",motionTr.TimesM[i].Tjpa ,  motionTr.TimesM[i].Taca , motionTr.TimesM[i].Tvc , motionTr.TimesM[i].Tjnb ,  motionTr.TimesM[i].Tacb ,  motionTr.TimesM[i].Tjpb);
      timeAdded =motionTr.TimesM[i].Tjpa +motionTr.TimesM[i].Taca +  motionTr.TimesM[i].Tjna + motionTr.TimesM[i].Tvc +  motionTr.TimesM[i].Tjnb +  motionTr.TimesM[i].Tacb +  motionTr.TimesM[i].Tjpb   ;


    }

    /* Add Segments Tvc of the second motion*/
    queueAddDetails(&(SDI_F->Seg[i]), 4, motion2.TNE.Tvc, motion2.jerk[i].J1, motion2.Acc[i].Tjna, motion2.Vel[i].Tjna, motion2.Pos[i].Tjna, motion2.Dir[i]);

    timeAdded = timeAdded + motion2.TNE.Tvc;
    
  
   
  
  }
  printf("timeAddedToSegments %d timeAdded %d \n",timeAddedToSegments,timeAdded);
  timeToSwitch = ((percentSwitch * timeAdded) / 100) +  timeAddedToSegments ;
  timeAddedToSegments = timeAddedToSegments + timeAdded;
  motion1 = motion2;

  printf("timeAddedToSegments %d timeToSwitch %d\n",timeAddedToSegments,timeToSwitch);
  printf("Motion 1 updated\n");

  RealTimeTrackPoseExecMode = 0;
  return EXEC;
  }
  printf("Return EXEC without if\n");
  return EXEC;
}
  
/* xarmRealTimeTrackPoseEnd  -  codel END of RealTimeTrackPose
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmRealTimeTrackPoseEnd(XARM_STRING *posPosterName, int *report)
{
  freemat(PFrom);
  freemat(PTo);
  freemat(QFrom);
  freemat(QTo);
  freemat(Dist);
  freemat(AbsDist);
  freemat(Dir);
  freemat(LDist);
  freemat(AbsLDist);
  freemat(ADist);
  freemat(AbsADist);

  return ETHER;
}

/* xarmRealTimeTrackPoseInter  -  codel INTER of RealTimeTrackPose
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmRealTimeTrackPoseInter(XARM_STRING *posPosterName, int *report)
{


  return ETHER;

}



