/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*--------------------------------------------------------------------
                         -- C N R S --
         Laboratoire d'automatique et d'analyse des systemes
            Groupe Robotique et Intelligenxce Artificielle
                   7 Avenue du colonel Roche
                     31 077 Toulouse Cedex

  Fichier              : xarmConst.h
  Fonction             : Constants definition for xarm
  Date de creation     : Aout 2005
  Date de modification : Aout 2005
  Nb de lignes         : 30

  Auteur               : Ignacio HERRERA AGUILAR

----------------------------------------------------------------------*/
#ifndef XARMCONST_H
#define XARMCONST_H

#define C1_MAX_LINEAR_JERK          0.900
#define C1_MAX_LINEAR_ACCELERATION  0.300
#define C1_MAX_LINEAR_VELOCITY      0.150

#define C1_MAX_ANGULAR_JERK         0.600
#define C1_MAX_ANGULAR_ACCELERATION 0.200
#define C1_MAX_ANGULAR_VELOCITY     0.100


#define C2_MAX_LINEAR_JERK          0.090
#define C2_MAX_LINEAR_ACCELERATION  0.030
#define C2_MAX_LINEAR_VELOCITY      0.015

#define C2_MAX_ANGULAR_JERK         0.030
#define C2_MAX_ANGULAR_ACCELERATION 0.010
#define C2_MAX_ANGULAR_VELOCITY     0.005

#define XARM_NB_DIM              7
#define XARM_NB_AXIS             6
#define XARM_POS_DIM             3
#define XARM_QUAT_DIM            4

#define XARM_POS_X_INITIAL       0
#define XARM_POS_Y_INITIAL       0
#define XARM_POS_Z_INITIAL       1.230

#define XARM_QUAT_N_INITIAL      1
#define XARM_QUAT_I_INITIAL      0
#define XARM_QUAT_J_INITIAL      0
#define XARM_QUAT_K_INITIAL      0

/* #define XARM_BASE_HEIGHT         0.317 */
#define XARM_BASE_HEIGHT         0.000
#define XARM_6ARM_LENGTH         0.070
#define XARM_TOOL_LENGTH         0.205

#define XARM_SAMPLING_TIME       0.010

#define C1_XARM_KP_LINEAR          40
#define C1_XARM_TI_LINEAR           0.20

#define C1_XARM_KP_ANGULAR         30
#define C1_XARM_TI_ANGULAR          0.50

#define C2_XARM_KP_LINEAR          20
#define C2_XARM_TI_LINEAR           1.00

#define C2_XARM_KP_ANGULAR         10
#define C2_XARM_TI_ANGULAR          1.00

#define XARM_SINGULARITY_THRESHOLD    0.000000001

//#define XARM_Q1_VELOCITY_LIMIT   0.5
//#define XARM_Q2_VELOCITY_LIMIT   0.5
//#define XARM_Q3_VELOCITY_LIMIT   0.5
//#define XARM_Q4_VELOCITY_LIMIT   2.0
//#define XARM_Q5_VELOCITY_LIMIT   2.0
//#define XARM_Q6_VELOCITY_LIMIT   2.0


#define XARM_Q1_VELOCITY_LIMIT   0.8   /* maxi 57  /s => 0.9948 rad/s */
#define XARM_Q2_VELOCITY_LIMIT   0.8   /* maxi 57  /s => 0.9948 rad/s */
#define XARM_Q3_VELOCITY_LIMIT   1.6   /* maxi 114 /s => 1.9896 rad/s */
#define XARM_Q4_VELOCITY_LIMIT   5.0   /* maxi 360 /s => 6.2800 rad/s */
#define XARM_Q5_VELOCITY_LIMIT   5.0   /* maxi 360 /s => 6.2800 rad/s */
#define XARM_Q6_VELOCITY_LIMIT   5.0   /* maxi 360 /s => 6.2800 rad/s */



#define XARM_K1_WEIGHT            0.0001
#define XARM_K2_WEIGHT            0.001

#define XARM_LIN_ACC_ZERO        (double)C2_MAX_LINEAR_ACCELERATION/100
#define XARM_LIN_VEL_ZERO        (double)C2_MAX_LINEAR_VELOCITY/100
#define XARM_ANG_ACC_ZERO        (double)C2_MAX_ANGULAR_ACCELERATION/100
#define XARM_ANG_VEL_ZERO        (double)C2_MAX_ANGULAR_VELOCITY/100


/* Offset considered for XT -- External Trajectory */
#define XARM_OFFSET_X_XT            0.20
#define XARM_OFFSET_Y_XT            0.0
#define XARM_OFFSET_Z_XT            0.07

/* Axis Limits for GotoAbsolutPose request */
#define XARM_X_MAX          1.1
#define XARM_X_MIN          0.5
#define XARM_Y_MAX          0.5
#define XARM_Y_MIN          -0.5

#define XARM_X_MAX_JOY      0.85
#define XARM_X_MIN_JOY      0.6
#define XARM_Y_MAX_JOY      0.4
#define XARM_Y_MIN_JOY      -0.4
#define XARM_Z_MAX_JOY      0.5
#define XARM_Z_MIN_JOY      -0.2

#define XARM_JOY_VXGAIN     (-C1_MAX_LINEAR_VELOCITY/32768);
#define XARM_JOY_VYGAIN     (-C1_MAX_LINEAR_VELOCITY/32768);
#define XARM_JOY_VZGAIN     (-C1_MAX_LINEAR_VELOCITY/32768);
#define XARM_JOY_WXGAIN     (2*C1_MAX_ANGULAR_VELOCITY/32768);
#define XARM_JOY_WYGAIN     (-2*C1_MAX_ANGULAR_VELOCITY/32768);
#define XARM_JOY_WZGAIN     (2*C1_MAX_ANGULAR_VELOCITY/32768);

#define XARM_MOUSE3D_VXGAIN     (C1_MAX_LINEAR_VELOCITY/460.0);
#define XARM_MOUSE3D_VYGAIN     (-C1_MAX_LINEAR_VELOCITY/460.0);
#define XARM_MOUSE3D_VZGAIN     (C1_MAX_LINEAR_VELOCITY/460.0);
#define XARM_MOUSE3D_WXGAIN     (2*C1_MAX_ANGULAR_VELOCITY/460.0);
#define XARM_MOUSE3D_WYGAIN     (2*C1_MAX_ANGULAR_VELOCITY/460.0);
#define XARM_MOUSE3D_WZGAIN     (2*C1_MAX_ANGULAR_VELOCITY/460.0);

#define JOYSTICK_SENSIBILITY 0.001
#define JOYSTICK_SENSIBILITY_ROT 0.01

#endif /* XARMCONST_H */
