/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*--------------------------------------------------------------------
                         -- C N R S --
         Laboratoire d'automatique et d'analyse des systemes
            Groupe Robotique et Intelligenxce Artificielle
                   7 Avenue du colonel Roche
                     31 077 Toulouse Cedex

  Fichier              : xarmStruct.h
  Fonction             : Struct definition for xarm
  Date de creation     : Aout 2005
  Date de modification : Aout 2005
  Nb de lignes         :

  Auteur               : Ignacio HERRERA AGUILAR

----------------------------------------------------------------------*/
#include "codels/tpStruct.h"
#include "xarmConst.h"
#include "codels/matrixStruct.h"
#include <gbStruct.h>

#ifndef XARMSTRUCT_H
#define XARMSTRUCT_H


#define XARM_MAX_NB_POSITIONS   200
#define XARM_MAX_NB_CONFIGURATIONS 50
#define XARM_MAX_POSTER_NAME 200

#define XARM_STRING_SIZE 256

typedef struct XARM_STRING{
  char str[XARM_STRING_SIZE];
}XARM_STRING;


typedef enum XARM_ENUM_MODE {
  XARM_INITIAL_STATE,
  XARM_SINGLE_POSE,
  XARM_FIXE_TRAJECTORY,
  XARM_EXTERNAL_POSE,
  XARM_EXTERNAL_CONF,
  XARM_EXTERNAL_DRIVE,
  XARM_STOP
} XARM_ENUM_MODE;

typedef enum XARM_TRACK_SEGMENT {
  XARM_TRACK_OFF,
  XARM_TRACK_ON
} XARM_TRACK_SEGMENT;

typedef struct XARM_TRACK_STATE {
  int state; /* xarmTrackState.state = 0 si xarmTrackSegment == XARM_TACK_OFF et  xarmTrackState.state = 1 si xarmTrackSegment == XARM_TACK_ON */
  int unused;
} XARM_TRACK_STATE;

typedef enum XARM_ETAT_ENUM {
  XARM_OFF,
  XARM_INIT,
  XARM_ON
} XARM_ETAT_ENUM;

typedef enum XARM_ENUM_SINGULAR {
  XARM_SINGULARITY_CROSS,
  XARM_SINGULARITY_NO_CROSS
} XARM_ENUM_SINGULAR;

typedef struct XARM_Gb_q6 {
  double q1;
  double q2;
  double q3;
  double q4;
  double q5;
  double q6;
} XARM_Gb_q6;


typedef struct XARM_FILE_STR {
  char file[100];
} XARM_FILE_STR;

typedef struct XARM_COND_STR {
  TP_COND x;
  TP_COND y;
  TP_COND z;
  TP_COND n;
  TP_COND i;
  TP_COND j;
  TP_COND k;
} XARM_COND_STR;

typedef struct XARM_PI_CONTROL_GAIN_STR {
  double Linear_Kp;                 /* Linear Control gain */
  double Linear_Ti;                 /* Linear Integral Time */
  double Angular_Kp;                /* Linear Control gain  */
  double Angular_Ti;                /* Linear Integral Time */
} XARM_PI_CONTROL_GAIN_STR;

typedef struct XARM_DIM_CONTROL_STR {
  double Ts;                 /* Sampling period */
  double Kp;                 /* Control gain */
  double Ti;                 /* Integral Time */
  double Error_1;            /* e[k-1] */
  double AccMax;             /* Maximum acceleration */
  double VelMax;             /* Maximum velocity */
  double Uk;                 /* u[k] */
} XARM_DIM_CONTROL_STR;

typedef struct XARM_POSE_CONTROL_STR {
  XARM_DIM_CONTROL_STR PX;
  XARM_DIM_CONTROL_STR PY;
  XARM_DIM_CONTROL_STR PZ;
  XARM_DIM_CONTROL_STR TX;
  XARM_DIM_CONTROL_STR TY;
  XARM_DIM_CONTROL_STR TZ;
} XARM_POSE_CONTROL_STR;


typedef struct XARM_TRAJECTORY {
  double trajectory[6][XARM_MAX_NB_POSITIONS];
  double timeTraj[1][XARM_MAX_NB_POSITIONS];
  int nb_positions;
  int withOutUseAlignment;
} XARM_TRAJECTORY;

typedef struct XARM_EXTERNAL_REF {
  double px, py, pz ;     /* Position */
  double qn, qi, qj, qk;  /* Orientation */
  double vx, vy, vz;      /* Lineal velocity */
  double wx, wy, wz;      /* Angular velocity */
} XARM_EXTERNAL_REF;

typedef struct XARM_EXT_POSTER {
  /*char posterName[XARM_MAX_POSTER_NAME];*/
  XARM_STRING posterName;
  int posterId;
  int withOutUseAlignment;
} XARM_EXT_POSTER;

/*  nx  ox  ax  pz
    ny  oy  ay  py
    nz  oz  az  pz  */

typedef struct XARM_THETA_STR{
  double nx, ny, nz;
  double ox, oy, oz;
  double ax, ay, az;
  double px, py, pz; /* this is the position in the frame of the base of the arm */
} XARM_THETA_STR;

typedef struct XARM_JUST_DO_IT{
  XARM_ETAT_ENUM GoForIt; /* XARM_OFF   0? --->   Not Ready
		             XARM_ON    2? --->   Ready
		          */
  int Counter;            /* 0 --- 100 per cent */
  int NElements;          /* Nb of elements */
  int IElements;          /* Index Elements */
} XARM_JUST_DO_IT;

/* Xavier */
typedef enum XARM_ENUM_CONSIGN_TYPE {
  XARM_QUATERNION_CONSIGN_TYPE,
  XARM_ANGLE_CONSIGN_TYPE,
  XARM_EXTERN_CONSIGN_TYPE,
  XARM_NOT_SELECTED_CONSIGN_TYPE
} XARM_ENUM_CONSIGN_TYPE;

typedef enum XARM_STATUS {
  XARM_OK    = 0,
  XARM_ERROR = 1
} XARM_STATUS;

typedef struct XARM_ARM_PA10 {
  double r0;                                /* Height base arm */
  double r6;                                /* Sixth link length */
  double r7;                                /* Tool + link length */
  XARM_THETA_STR r8z;                       /* Rotation for correct the last joint */
  TP_POSELIMITS Limits;                     /* Linear and Angular limits */
  Gb_q6 qPos;                               /* Q Position */
  Gb_q6 qVel;                               /* Q velocity */
  Gb_th th;                                 /* Theta Matrix */
  QARM_POSITION_STR qarmPosition;           /* Structure for poster Position Qarm reading */
  Gb_jac Jac06;                             /* Jacobian Matrix R0 - R6 */
  dmat Jac;                                 /* Jacobian Matrix */
  dmat InvJac;                              /* Inverted Jacobian Matrix */
  double K_WEIGHT;                          /* K for SR- Inverse */
  dmat PoseVelocity;                        /* Pose Velocity Vector */
  dmat AxisVelocity;                        /* Axis Velocity Vector */
  dmat AxisVelocityLimits;                  /* Axis Velocity Limits Vector */
} XARM_ARM_PA10;

typedef struct XARM_SOIL {
  double angularVelocity;
  double angularPosition;
  double linearVelocity;
  double linearPosition;
} XARM_SOIL;

typedef struct XARM_EXTERNAL_TRACKING {
  int TrajectoryID;
  int posterIsInitialized; /* 0:not initialised    1:initIsDone */
  int stopExternalTracking; /* 0:not stopped    1:stopped */
  int limitsRatio; /* 0 --- 100 per cent of Jerk Acceleration and Velocity limits */
  int nbPoint;
  Gb_q6 qRef[XARM_MAX_NB_CONFIGURATIONS];
  // XARM_EXTERNAL_REF cartesianRef;
} XARM_EXTERNAL_TRACKING;

typedef struct XARM_EXTERNAL_STATE {
  int  TrajectoryID;
  int currentPoint;
}XARM_EXTERNAL_STATE;

#endif /* XARMSTRUCT_H */
