/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 ** xarmMotionCodels.c
 **
 ** Codels called by execution task xarmMotion
 **
 ** Author: Xavier BROQUERE
 ** Date: October 2007
 **
 **/

#include <portLib.h>
#include <stdlib.h>
#include "stdio.h"
#include <math.h>
#include "server/xarmHeader.h"

#include "server/qarmPosterLib.h"
//#include "server/joystickPosterLib.h"
#include "xarmConst.h"
#include "tpConst.h"
#include "Proto_gb.h"
#include "Proto_gbModeles.h"
#include "libeuler/pomEuler.h"
#include "xarmFunctions.h"
#include "xarmFunctions.c"
#include "matrix.h"
#include "tpFunctions.h"
#include "tpFunctions.c"
#include "cFunctions.h"
#include "xarmJerkprofile.h"
#include "xarmJerkprofile.c"
#include "queue.c"
#include "joystick.h"

static int initDone = 0;
static POSTER_ID posterQarmPos;                   /* Poster ID Position Qarm  */
static SEGMENT LSeg[XARM_NB_DIM];
//static SEGMENT LSegY;
//static SEGMENT LSegZ;
//static SEGMENT LSegQN;
//static SEGMENT LSegQI;
//static SEGMENT LSegQJ;
//static SEGMENT LSegQK;

static TP_TICK LTick;
static TP_LOCALVECTORS Vectors;
static TP_VEL Omega;                             /* Omega */   
static int indexElement;
static int VerifyIfItsLastSegment;
static int initTakeSeg;
static dmat auxJac;                              /* Auxiliar Jacobian Matrix */ 

/* Needed by Control Loop */
static TP_MAT MatQ;                              /* Matrix for Q Position */
static TP_MAT MatQPoint;                         /* Matrix for QPoint */
static TP_MAT MatPReal;                          /* Matrix for Position Measured */
static TP_MAT MatQReal;                          /* Matrix for Orientation Measured */
static TP_MAT MatErrorP;                         /* Matrix for Error Position */   
static TP_MAT MatErrorQ;                         /* Matrix for Error Quaternion */
static TP_MAT MatQRealPrevious;

/* Needed by Visual Servoing */
static XARM_EXTERNAL_REF extPose;                /* Pose Poster for external control */
static double ui_y_1;                            /* Integrator for visual servoing in Y */
static double ei_y_1;                            /* Error for visual servoin in Y */

/* Only to save the data */
static TP_VECTORS VROWS;
static TP_VECTORS VROWSREAL;
static double ControlOut_Vx[MAX_ELEMENTS_LIMIT];
static double ControlOut_Vy[MAX_ELEMENTS_LIMIT];
static double ControlOut_Vz[MAX_ELEMENTS_LIMIT];
static double ControlOut_Wx[MAX_ELEMENTS_LIMIT];
static double ControlOut_Wy[MAX_ELEMENTS_LIMIT];
static double ControlOut_Wz[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q1[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q2[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q3[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q4[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q5[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q6[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q1Vel[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q2Vel[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q3Vel[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q4Vel[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q5Vel[MAX_ELEMENTS_LIMIT];
static double SaveQPos_q6Vel[MAX_ELEMENTS_LIMIT];

static int cptSaveData;
static int indexToSaveData;
static int fileIsOpen;

static POSTER_ID posPosterId = NULL;
static int posterIsPOM_POS;
static POM_SENSOR_POS baseArmPos;
static T3D  baseArmPosT3d;
static double ErrorDim;

static  double VxRef, VyRef, VzRef, WxRef, WyRef, WzRef;
static  TP_COND IC, FC;
static  TP_LIMITS auxLimits;

//static  TP_AXIS_TIMES AxisTimes;
static  TP_TIMES AxisTimes[XARM_NB_DIM];

static  TP_TIMES Acc[XARM_NB_DIM], Vel[XARM_NB_DIM], Pos[XARM_NB_DIM];
static  double GD;
static  int Dir[XARM_NB_DIM];
static  TP_JERKS Jerks;
static  TP_TIMES  TM, TNE;
static  int NOE;  
static  double NB_TICK = 10.0;
static  double StopPos[XARM_NB_DIM];
static  double Time[XARM_NB_DIM];
static  double MaxTimeM;
static  int SweetStopInit;
static double VjoyRef[1000];
static int cptVjoy;
# define JOYSTICK_DEVICE_PATH "/dev/input/js0"
#define DELAY_SAVE_DATA 1 /* Save Data of the arm after the end of Track Segment (in tick) */
static POSTER_ID posterMouse3d = NULL;                   /* Poster ID Mouse3d */
static POSTER_ID posterViman =  NULL;                   /* Poster ID Joystick  */


static Gb_objectPose vimanObjectPoseOld;
static TP_QUAT  vimanObjectQuatOld;

static void getJoystickSpeeds(double *vx, double *vy, double *vz, double* wx, double* wy, double* wz);
static void getMouse3dSpeeds(double *vx, double *vy, double *vz, double* wx, double* wy, double* wz);
static void getVimanObjectMotion(double* px, double* py, double* pz, double* qn, double* qi, double* qj, double* qk);

static void getJoystickSpeeds(double *vx, double *vy, double *vz, double* wx, double* wy, double* wz)
{
  double vxl, vyl, vzl, wxl, wyl, wzl;
  Gb_th thTool, thToolInv;
  Gb_v3 velTrans, velRot,  velTransOut, velRotOut;
  //Gb_th Next_thTool;


  if (SDI_F->joystick.buttons[5]) {
    vxl = SDI_F->joystick.axes[1] * XARM_JOY_VXGAIN;
    vyl = SDI_F->joystick.axes[0] * XARM_JOY_VYGAIN;
    vzl = SDI_F->joystick.axes[3] * XARM_JOY_VZGAIN; 
    wxl = SDI_F->joystick.axes[4] * XARM_JOY_WYGAIN;
    wyl = SDI_F->joystick.axes[5] * XARM_JOY_WYGAIN;
    wzl = SDI_F->joystick.axes[2] * XARM_JOY_WZGAIN;




    Gb_th_produit(&(SDI_F->pa10Arm.qarmPosition.th06), xarmR6Rt, &thTool);
    //    Gb_th_inverse(&thTool, &thToolInv);
    Gb_v3_set(&velTrans, vxl, vyl, vzl);
    Gb_v3_set(&velRot, wxl, wyl, wzl);
    Gb_th_x_v3 (&thTool, &velTrans, &velTransOut);
    Gb_th_x_v3 (&thTool, &velRot, &velRotOut);

  //  *vx =  baseArmPosT3d.matrix.matrix[0]*vxl + baseArmPosT3d.matrix.matrix[4]*vyl + baseArmPosT3d.matrix.matrix[8]*vzl ;
  //  *vy =  baseArmPosT3d.matrix.matrix[1]*vxl + baseArmPosT3d.matrix.matrix[5]*vyl + baseArmPosT3d.matrix.matrix[9]*vzl ;
  //  *vz =  baseArmPosT3d.matrix.matrix[2]*vxl + baseArmPosT3d.matrix.matrix[6]*vyl + baseArmPosT3d.matrix.matrix[10]*vzl ;
 
    *vx = velTransOut.x;
    *vy = velTransOut.y;
    *vz = velTransOut.z;

    *wx = velRotOut.x;
    *wy = velRotOut.y;
    *wz = velRotOut.z;

  }
  else {
    *vx = 0.0;
    *vy = 0.0;
    *vz = 0.0;
    *wx = 0.0;
    *wy = 0.0;
    *wz = 0.0;
  }
}


static void getMouse3dSpeeds(double *vx, double *vy, double *vz, double* wx, double* wy, double* wz)
{

  double vxl, vyl, vzl, wxl, wyl, wzl;
  if (SDI_F->mouse3dData.b0) {
    vxl = SDI_F->mouse3dData.z * XARM_MOUSE3D_VXGAIN;
    vyl = SDI_F->mouse3dData.x * XARM_MOUSE3D_VYGAIN;
    vzl = SDI_F->mouse3dData.y * XARM_MOUSE3D_VZGAIN;
    wxl = 0.0;
    wyl = 0.0;
    wzl = 0.0;

    *vx =  baseArmPosT3d.matrix.matrix[0]*vxl + baseArmPosT3d.matrix.matrix[4]*vyl + baseArmPosT3d.matrix.matrix[8]*vzl ;
    *vy =  baseArmPosT3d.matrix.matrix[1]*vxl + baseArmPosT3d.matrix.matrix[5]*vyl + baseArmPosT3d.matrix.matrix[9]*vzl ;
    *vz =  baseArmPosT3d.matrix.matrix[3]*vxl + baseArmPosT3d.matrix.matrix[6]*vyl + baseArmPosT3d.matrix.matrix[10]*vzl ;
    *wx =  baseArmPosT3d.matrix.matrix[0]*wxl + baseArmPosT3d.matrix.matrix[4]*wyl + baseArmPosT3d.matrix.matrix[8]*wzl ;
    *wy =  baseArmPosT3d.matrix.matrix[1]*wxl + baseArmPosT3d.matrix.matrix[5]*wyl + baseArmPosT3d.matrix.matrix[9]*wzl ;
    *wz =  baseArmPosT3d.matrix.matrix[3]*wxl + baseArmPosT3d.matrix.matrix[6]*wyl + baseArmPosT3d.matrix.matrix[10]*wzl ; 

  } else {
    vxl = SDI_F->mouse3dData.z * XARM_MOUSE3D_VXGAIN;
    vyl = SDI_F->mouse3dData.x * XARM_MOUSE3D_VYGAIN;
    vzl = SDI_F->mouse3dData.y * XARM_MOUSE3D_VZGAIN;
    wxl = SDI_F->mouse3dData.Rz * XARM_MOUSE3D_WXGAIN;
    wyl = SDI_F->mouse3dData.Rx * XARM_MOUSE3D_WYGAIN;
    wzl = SDI_F->mouse3dData.Ry * XARM_MOUSE3D_WZGAIN; 

    *vx =  baseArmPosT3d.matrix.matrix[0]*vxl + baseArmPosT3d.matrix.matrix[4]*vyl + baseArmPosT3d.matrix.matrix[8]*vzl ;
    *vy =  baseArmPosT3d.matrix.matrix[1]*vxl + baseArmPosT3d.matrix.matrix[5]*vyl + baseArmPosT3d.matrix.matrix[9]*vzl ;
    *vz =  baseArmPosT3d.matrix.matrix[3]*vxl + baseArmPosT3d.matrix.matrix[6]*vyl + baseArmPosT3d.matrix.matrix[10]*vzl ;
    *wx =  baseArmPosT3d.matrix.matrix[0]*wxl + baseArmPosT3d.matrix.matrix[4]*wyl + baseArmPosT3d.matrix.matrix[8]*wzl ;
    *wy =  baseArmPosT3d.matrix.matrix[1]*wxl + baseArmPosT3d.matrix.matrix[5]*wyl + baseArmPosT3d.matrix.matrix[9]*wzl ;
    *wz =  baseArmPosT3d.matrix.matrix[3]*wxl + baseArmPosT3d.matrix.matrix[6]*wyl + baseArmPosT3d.matrix.matrix[10]*wzl ; 
  }
}

//static XARM_THETA_STR vimanObjectPoseOld;

static void getVimanObjectMotion(double* px, double* py, double* pz, double* qn, double* qi, double* qj, double* qk) {

  Gb_th thTool;
  TP_QUAT GBQuatL;
  double normaQ;
  Gb_dep dep;
  Gb_quat quat;

  if( SDI_F->gbObjectPose.visible == 1) {

    *px = SDI_F->gbObjectPose.th.px - 0.18 - 0.03;
    *py = SDI_F->gbObjectPose.th.py;
    *pz = SDI_F->gbObjectPose.th.pz - 0.924;

    thTool.vx.x =  SDI_F->gbObjectPose.th.nx;
    thTool.vx.y =  SDI_F->gbObjectPose.th.ny;
    thTool.vx.z =  SDI_F->gbObjectPose.th.nz;
			  
    thTool.vy.x =  SDI_F->gbObjectPose.th.ox;
    thTool.vy.y =  SDI_F->gbObjectPose.th.oy;
    thTool.vy.z =  SDI_F->gbObjectPose.th.oz;
			  
    thTool.vz.x =  SDI_F->gbObjectPose.th.ax;
    thTool.vz.y =  SDI_F->gbObjectPose.th.ay;
    thTool.vz.z =  SDI_F->gbObjectPose.th.az;

    /* Calcul and normalization of goal quaternion in the base frame */ 
    if (RotMat2Quat(&thTool, &GBQuatL) != OK) { 
      PRINT_ERROR("Pb - RotMatToQuat"); 
      *qn = 1.0;
      *qi = 0.0;
      *qj = 0.0;
      *qk = 0.0;
      
    } else {
      normaQ = GBQuatL.n * GBQuatL.n
	+ GBQuatL.i * GBQuatL.i
	+ GBQuatL.j * GBQuatL.j
	+ GBQuatL.k * GBQuatL.k;
      normaQ=sqrt(normaQ);
      if(normaQ==0) {
	PRINT_ERROR(" - ERROR - Quaternion Norm = 0");
	*qn = 1.0;
	*qi = 0.0;
	*qj = 0.0;
	*qk = 0.0;
      } else {
      *qn = GBQuatL.n /normaQ;
      *qi = GBQuatL.i /normaQ;
      *qj = GBQuatL.j /normaQ;
      *qk = GBQuatL.k /normaQ;
      }
    }
    quat.x = *px;
    quat.y = *py;
    quat.z = *pz;
    quat.w = *qn;
    quat.vx = *qi;
    quat.vy = *qj;
    quat.vz = *qk;
    Gb_quat_dep(&quat, &dep);

    //  printf("qn %f qi %f qj %f qk %f\n",*qn, *qi, *qj, *qk);
    //printf("dep x %f y %f z %f rx %f ry %f rz %f a %f \n", dep.x, dep.y, dep.z , dep.rx, dep.ry, dep.rz, dep.a);

    if( (ABS(*px -   vimanObjectPoseOld.th.px) < 0.002 ) &&  (ABS(*py -   vimanObjectPoseOld.th.py) < 0.002 ) &&  (ABS(*pz -   vimanObjectPoseOld.th.pz)) < 0.002 ) { 
      *px = vimanObjectPoseOld.th.px ;
      *py = vimanObjectPoseOld.th.py ;
      *pz = vimanObjectPoseOld.th.pz ; 
    } else {
      vimanObjectPoseOld.th.px  = SDI_F->gbObjectPose.th.px - 0.18 - 0.03;
      vimanObjectPoseOld.th.py  = SDI_F->gbObjectPose.th.py;
      vimanObjectPoseOld.th.pz  = SDI_F->gbObjectPose.th.pz - 0.924;
      vimanObjectQuatOld.n =  *qn ;
      vimanObjectQuatOld.i =  *qi ;
      vimanObjectQuatOld.j =  *qj ;
      vimanObjectQuatOld.k =  *qk ;
    
    }
  } else {
    *px = vimanObjectPoseOld.th.px ;
    *py = vimanObjectPoseOld.th.py ;
    *pz = vimanObjectPoseOld.th.pz ;  
    *qn = vimanObjectQuatOld.n;
    *qi = vimanObjectQuatOld.i;
    *qj = vimanObjectQuatOld.j;
    *qk = vimanObjectQuatOld.k; 
  }
}

/*------------------------------------------------------------------------
 *
 * xarmMotionEnd  --  Termination codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
xarmMotionEnd(void)
{
  initDone = 0;
  SDI_F->dq.counter    += 1;
  SDI_F->dq.qVelocityC.q1 = 0.0;
  SDI_F->dq.qVelocityC.q2 = 0.0;
  SDI_F->dq.qVelocityC.q3 = 0.0;
  SDI_F->dq.qVelocityC.q4 = 0.0;
  SDI_F->dq.qVelocityC.q5 = 0.0;
  SDI_F->dq.qVelocityC.q6 = 0.0;

  //freemat();
  freemat(SDI_F->pa10Arm.InvJac);
  freemat(SDI_F->pa10Arm.Jac);
  freemat(auxJac);
  freemat(SDI_F->pa10Arm.PoseVelocity); 
  freemat(SDI_F->pa10Arm.AxisVelocity);
  freemat(SDI_F->pa10Arm.AxisVelocityLimits);

  printf("xarm Memory cleared \n");

  return OK;
}

/*------------------------------------------------------------------------
 *
 * xarmMotionPerm  --  1st codel of permanent activity (called BEFORE other activities)
 *
 * Description: 
 * 
 * Reports:      OK
 *              S_xarm_ERROR_MODE
 *              S_xarm_POSTER_NOT_FOUND
 *              S_xarm_POSTER_NOT_READ
 *              S_xarm_WRONG_VALUE
 *              S_xarm_ERROR_TP
 * 
 * Returns:    OK or ERROR
 */
  int cpt = 0;
STATUS
xarmMotionPerm(int *report)
{
  T3D t3d;
  double normaQ;
  TP_QUAT Quat;
  TP_QUAT DQuat;
  FILE *fileptr;
  int i; /* compteur */
  double D; /* variable temporaire */
  double Vx,Vy,Vz,Wx,Wy,Wz;
  double Qn, Qi, Qj, Qk, Rn, Ri, Rj, Rk;
  double coef;
  double auxCoef;  
  double cosTheta;
  double ei_y, ui_y;     /* Visual Servoing in Y */
  double Kii_y = 0.005;   /* 0.0025 */
  double Kpi_y = 0.01;      /* 0.1 */
  
  if (!initDone) {
    return OK;
  }

  /* Read Poster Data and Status from QARM */
  if (xarmQARM_POSITION_STRPosterRead(posterQarmPos, &(SDI_F->pa10Arm.qarmPosition))!=OK) {
    PRINT_ERROR("Can not read POSITION_POSTER");
    *report =  S_xarm_POSTER_NOT_READ;
    return ERROR;
  }
  
  xarmTh06_Tool(&(SDI_F->pa10Arm.qarmPosition.th06), &(SDI_F->pa10Arm.th), &(SDI_F->pose));

  /* th = qarmPosition.th06; */
  SDI_F->pa10Arm.Jac06 = SDI_F->pa10Arm.qarmPosition.jac;
  SDI_F->pa10Arm.qPos  = SDI_F->pa10Arm.qarmPosition.q;

  /* Arm position */
  SDI_F->posArm.nx = SDI_F->pa10Arm.th.vx.x;                                                    
  SDI_F->posArm.ny = SDI_F->pa10Arm.th.vx.y;							 
  SDI_F->posArm.nz = SDI_F->pa10Arm.th.vx.z;							 										 
  SDI_F->posArm.ox = SDI_F->pa10Arm.th.vy.x;							 
  SDI_F->posArm.oy = SDI_F->pa10Arm.th.vy.y;							 
  SDI_F->posArm.oz = SDI_F->pa10Arm.th.vy.z;							 										 
  SDI_F->posArm.ax = SDI_F->pa10Arm.th.vz.x;							 
  SDI_F->posArm.ay = SDI_F->pa10Arm.th.vz.y;							 
  SDI_F->posArm.az = SDI_F->pa10Arm.th.vz.z;							 										 
  SDI_F->posArm.px =  SDI_F->pa10Arm.qarmPosition.th06.vp.x + SDI_F->pa10Arm.r6 * SDI_F->pa10Arm.qarmPosition.th06.vz.x;	 
  SDI_F->posArm.py =  SDI_F->pa10Arm.qarmPosition.th06.vp.y + SDI_F->pa10Arm.r6 * SDI_F->pa10Arm.qarmPosition.th06.vz.y;	 
  SDI_F->posArm.pz =  SDI_F->pa10Arm.r0 + SDI_F->pa10Arm.qarmPosition.th06.vp.z + SDI_F->pa10Arm.r6 * SDI_F->pa10Arm.qarmPosition.th06.vz.z;	 
										 
  /* Grip position */								 
  SDI_F->posGrip.nx =  SDI_F->pa10Arm.th.vx.x;							 
  SDI_F->posGrip.ny =  SDI_F->pa10Arm.th.vx.y;							 
  SDI_F->posGrip.nz =  SDI_F->pa10Arm.th.vx.z;							 										 
  SDI_F->posGrip.ox =  SDI_F->pa10Arm.th.vy.x;							 
  SDI_F->posGrip.oy =  SDI_F->pa10Arm.th.vy.y;							 
  SDI_F->posGrip.oz =  SDI_F->pa10Arm.th.vy.z;							 										 
  SDI_F->posGrip.ax =  SDI_F->pa10Arm.th.vz.x;							 
  SDI_F->posGrip.ay =  SDI_F->pa10Arm.th.vz.y;							 
  SDI_F->posGrip.az =  SDI_F->pa10Arm.th.vz.z;							 										 
  SDI_F->posGrip.px =  SDI_F->pose.p.x;						 
  SDI_F->posGrip.py =  SDI_F->pose.p.y;						 
  SDI_F->posGrip.pz =  SDI_F->pose.p.z;    

  SDI_F->posGripQuat.x =  SDI_F->pose.p.x;						 
  SDI_F->posGripQuat.y =  SDI_F->pose.p.y;						 
  SDI_F->posGripQuat.z =  SDI_F->pose.p.z;  
  SDI_F->posGripQuat.w =  SDI_F->pose.q.n;						 
  SDI_F->posGripQuat.vx =  SDI_F->pose.q.i;						 
  SDI_F->posGripQuat.vy =  SDI_F->pose.q.j;  
  SDI_F->posGripQuat.vz =  SDI_F->pose.q.k;  
                                      

  strcpy(SDI_F->pomArmPos.seName,"R6Arm");

  // typedef struct T3DMatrix {
  //   short type; 			/* T3D_MATRIX */
  //   short flags;
  //   int err;
  //   double matrix[16];		/* homogeneous matrix */
  //   double v[21];			/* !!!! covariance matrix on _Euler_ angles !!!! */
  // } T3DMatrix;
  // 
  // typedef union T3D {
  //   short type; 	  T3D_MATRIX,		/* must not be changed */
  //   T3DAny any;
  //   T3DMatrix matrix;
  //   T3DEuler euler;
  //   T3DVector vector;
  // } T3D;

  t3dSetOk(&t3d);
  t3d.matrix.flags = T3D_ALLOW_CONVERSION;

  t3d.type = T3D_MATRIX;
  t3d.matrix.type = T3D_MATRIX;
  t3d.matrix.matrix[0]  = SDI_F->pa10Arm.th.vx.x;
  t3d.matrix.matrix[1]  = SDI_F->pa10Arm.th.vy.x;
  t3d.matrix.matrix[2]  = SDI_F->pa10Arm.th.vz.x;
  t3d.matrix.matrix[3]  = SDI_F->posArm.px;
  t3d.matrix.matrix[4]  = SDI_F->pa10Arm.th.vx.y;
  t3d.matrix.matrix[5]  = SDI_F->pa10Arm.th.vy.y;
  t3d.matrix.matrix[6]  = SDI_F->pa10Arm.th.vz.y;
  t3d.matrix.matrix[7]  = SDI_F->posArm.py;
  t3d.matrix.matrix[8]  = SDI_F->pa10Arm.th.vx.z;
  t3d.matrix.matrix[9]  = SDI_F->pa10Arm.th.vy.z;
  t3d.matrix.matrix[10] = SDI_F->pa10Arm.th.vz.z;
  t3d.matrix.matrix[11] = SDI_F->posArm.pz ;
  t3d.matrix.matrix[12] = 0;
  t3d.matrix.matrix[13] = 0;
  t3d.matrix.matrix[14] = 0;
  t3d.matrix.matrix[15] = 1;

  pomEulerVFromT3D (&t3d, &(SDI_F->pomArmPos.seConfig));
  
  if (initTakeSeg ==0) {

    /* Init Trajectory Variables */ 
    MatQ.cols = 6;
    MatQ.rows = 1;
    MatQPoint.cols = 6;
    MatQPoint.rows = 1;
    MatErrorP.cols = 3;
    MatErrorP.rows = 1;
    MatErrorQ.cols = 5;
    MatErrorQ.rows = 1;
    MatPReal.cols  = 3;
    MatPReal.rows  = 1;
    MatQReal.cols  = 4;
    MatQReal.rows  = 1;
    MatQRealPrevious.cols  = 4;
    MatQRealPrevious.rows  = 1;

    MatQRealPrevious.el[0][0] = SDI_F->pose.q.n;
    MatQRealPrevious.el[0][1] = SDI_F->pose.q.i;
    MatQRealPrevious.el[0][2] = SDI_F->pose.q.j;
    MatQRealPrevious.el[0][3] = SDI_F->pose.q.k;
  }
  
  /* Jacobian Matrix Calcul */
  if (CalculOfJacobianMatrix(SDI_F->pa10Arm.Jac, SDI_F->pa10Arm.Jac06, SDI_F->pa10Arm.r6, SDI_F->pa10Arm.r7) !=OK) {
    PRINT_ERROR(" JACOBIAN MATRIX CALCUL ");
  }
  
  /* Calcul acceleration, velocity and position references to control loop */ 
  switch(SDI_F->xarmTrackSegment) {

  case XARM_TRACK_OFF:

    /* For safety */
    SDI_F->mode = XARM_INITIAL_STATE;
    SDI_F->SweetStop = 0;
    SDI_F->xarmTrackState.state = 0;
    initTakeSeg = 0;
    VerifyIfItsLastSegment = 0;
    SweetStopInit = 0;
    // indexElement =0;
    SDI_F->currentTimeOfTrajectory = 0;

/*    if ((SDI_F->saveFiles == 1) && cptSaveData < DELAY_SAVE_DATA)  { */
/*       indexToSaveData ++; */
/*       i = indexElement-1; */

/*       SaveQPos_q1[indexToSaveData] = SDI_F->pa10Arm.qarmPosition.q.q1; */
/*       SaveQPos_q2[indexToSaveData] = SDI_F->pa10Arm.qarmPosition.q.q2; */
/*       SaveQPos_q3[indexToSaveData] = SDI_F->pa10Arm.qarmPosition.q.q3; */
/*       SaveQPos_q4[indexToSaveData] = SDI_F->pa10Arm.qarmPosition.q.q4; */
/*       SaveQPos_q5[indexToSaveData] = SDI_F->pa10Arm.qarmPosition.q.q5; */
/*       SaveQPos_q6[indexToSaveData] = SDI_F->pa10Arm.qarmPosition.q.q6; */
     
/*       fprintf(fileptr,"%d %f %f %f %f %f %f %f %f %f ", indexToSaveData, VROWS.PX_Acc.el[0][i],VROWS.PX_Vel.el[0][i],VROWS.PX_Pos.el[0][i],VROWS.PY_Acc.el[0][i],VROWS.PY_Vel.el[0][i],VROWS.PY_Pos.el[0][i],VROWS.PZ_Acc.el[0][i],VROWS.PZ_Vel.el[0][i],VROWS.PZ_Pos.el[0][i]); */
/*       fprintf(fileptr,"%f %f %f %f %f %f %f %f %f %f %f %f ",VROWS.QN_Acc.el[0][i],VROWS.QN_Vel.el[0][i],VROWS.QN_Pos.el[0][i],VROWS.QI_Acc.el[0][i],VROWS.QI_Vel.el[0][i],VROWS.QI_Pos.el[0][i],VROWS.QJ_Acc.el[0][i],VROWS.QJ_Vel.el[0][i],VROWS.QJ_Pos.el[0][i],VROWS.QK_Acc.el[0][i],VROWS.QK_Vel.el[0][i],VROWS.QK_Pos.el[0][i]); */
/*       fprintf(fileptr,"%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f ;\n",SDI_F->pose.p.x, SDI_F->pose.p.y, SDI_F->pose.p.z, SDI_F->pose.q.n, SDI_F->pose.q.i, SDI_F->pose.q.j, SDI_F->pose.q.k, 1.0, ControlOut_Vx[i], ControlOut_Vy[i], ControlOut_Vz[i], ControlOut_Wx[i], ControlOut_Wy[i], ControlOut_Wz[i], SaveQPos_q1[indexToSaveData], SaveQPos_q2[indexToSaveData], SaveQPos_q3[indexToSaveData], SaveQPos_q4[indexToSaveData], SaveQPos_q5[indexToSaveData], SaveQPos_q6[indexToSaveData], SaveQPos_q1Vel[i], SaveQPos_q2Vel[i], SaveQPos_q3Vel[i], SaveQPos_q4Vel[i], SaveQPos_q5Vel[i], SaveQPos_q6Vel[i]); */
/*       cptSaveData ++; */
/*     } */

/*     if ((SDI_F->saveFiles == 1) && cptSaveData >= DELAY_SAVE_DATA && fileIsOpen==1)  { */
/*       fclose(fileptr); */
/*       fileIsOpen = 0; */
/*       indexElement = 0; */
/*       printf("File RefTP.dat is written !\n"); */
/*     } */


    break;

  case XARM_TRACK_ON:

    SDI_F->xarmTrackState.state = 1; 
    SDI_F->dq.counter  ++ ;

    SaveQPos_q1[indexElement] = SDI_F->pa10Arm.qarmPosition.q.q1;
    SaveQPos_q2[indexElement] = SDI_F->pa10Arm.qarmPosition.q.q2;
    SaveQPos_q3[indexElement] = SDI_F->pa10Arm.qarmPosition.q.q3;
    SaveQPos_q4[indexElement] = SDI_F->pa10Arm.qarmPosition.q.q4;
    SaveQPos_q5[indexElement] = SDI_F->pa10Arm.qarmPosition.q.q5;
    SaveQPos_q6[indexElement] = SDI_F->pa10Arm.qarmPosition.q.q6;

    if (VerifyIfItsLastSegment > 0  || (SDI_F->SweetStop == 1)) {
      /* Verify if Motion is stopping */
      if ( ABS(Vectors.X.Vel) > SWEET_STOP_SOIL ||  ABS(Vectors.Y.Vel) > SWEET_STOP_SOIL ||  ABS(Vectors.Z.Vel) > SWEET_STOP_SOIL  ||  ABS(Vectors.QN.Vel) > SWEET_STOP_SOIL  ||  ABS(Vectors.QI.Vel) > SWEET_STOP_SOIL  ||  ABS(Vectors.QJ.Vel) > SWEET_STOP_SOIL  ||  ABS(Vectors.QK.Vel) > SWEET_STOP_SOIL || ABS(Vectors.X.Acc) > SWEET_STOP_SOIL ||  ABS(Vectors.Y.Acc) > SWEET_STOP_SOIL ||  ABS(Vectors.Z.Acc) > SWEET_STOP_SOIL  ||  ABS(Vectors.QN.Acc) > SWEET_STOP_SOIL  ||  ABS(Vectors.QI.Acc) > SWEET_STOP_SOIL  ||  ABS(Vectors.QJ.Acc) > SWEET_STOP_SOIL  ||  ABS(Vectors.QK.Acc) > SWEET_STOP_SOIL  ) {
	
	if (SweetStopInit == 0) {
	  SDI_F->SweetStop = 1;
	  printf("XARM_SWEET_STOP\n");
	  initTakeSeg = 0;
	  VerifyIfItsLastSegment = 0;
	  /* Initialize queues */
	  queueInit(&(SDI_F->Seg[0]), "SegX");
	  queueInit(&(SDI_F->Seg[1]), "SegY");
	  queueInit(&(SDI_F->Seg[2]), "SegZ");
	  queueInit(&(SDI_F->Seg[3]), "SegQN");
	  queueInit(&(SDI_F->Seg[4]), "SegQI");
	  queueInit(&(SDI_F->Seg[5]), "SegQJ");
	  queueInit(&(SDI_F->Seg[6]), "SegQK"); 
	  MaxTimeM = 0;

	  /* Compute optimal motion for each quaternion axis */
	  for (i=0; i < XARM_NB_DIM; i++) {
	    
	    if (i<3) {
	      auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
	      auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
	      auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
	      Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
	      Jerks.sel = 1; 
	    } else {
	      auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
	      auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
	      auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
	      Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
	      Jerks.sel = 1; 
	    }
	    
	    if(i==0) {
	      /**************** PX ****************/                                     
	      IC.a = SDI_F->motionConditions.x.a; 
	      IC.v = SDI_F->motionConditions.x.v;
	      IC.x = SDI_F->motionConditions.x.x;
	    }
	    if(i==1) {
	      /**************** PY ****************/                                     
	      IC.a = SDI_F->motionConditions.y.a; 
	      IC.v = SDI_F->motionConditions.y.v;
	      IC.x = SDI_F->motionConditions.y.x;
	    }
	    if(i==2) {
	      /**************** PZ ****************/                                     
	      IC.a = SDI_F->motionConditions.z.a; 
	      IC.v = SDI_F->motionConditions.z.v;
	      IC.x = SDI_F->motionConditions.z.x;
	    }
	    if(i==3) {
	      /**************** QN ****************/                                     
	      IC.a = SDI_F->motionConditions.n.a; 
	      IC.v = SDI_F->motionConditions.n.v;
	      IC.x = SDI_F->motionConditions.n.x;
	    }
	    if(i==4) {
	      /**************** QI ****************/                                     
	      IC.a = SDI_F->motionConditions.i.a; 
	      IC.v = SDI_F->motionConditions.i.v;
	      IC.x = SDI_F->motionConditions.i.x;
	    }
	    if(i==5) {
	      /**************** QJ ****************/                                     
	      IC.a = SDI_F->motionConditions.j.a; 
	      IC.v = SDI_F->motionConditions.j.v;
	      IC.x = SDI_F->motionConditions.j.x;
	    }
	    if(i==6) {
	      /**************** QK ****************/                                     
	      IC.a = SDI_F->motionConditions.k.a; 
	      IC.v = SDI_F->motionConditions.k.v;
	      IC.x = SDI_F->motionConditions.k.x;
	    }

	    D = IC.x;
	    IC.x = 0.0;

	    //if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.linear, &criticalLength)!= OK) {
	    //if(xarmCalculOfCriticalLength( IC, FC, auxLimits, &StopPos[i])!= OK) {
	    //  PRINT_ERROR("ERROR Calcul Of Critical Length\n");
	    //  return ETHER;
	    //}
	    //if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[i]), &Dir[i])!=0) {
	    //  PRINT_ERROR("ERROR Jerk Profile\n");
	    //  return ETHER;
	    //}

	    tpStopPosition(IC, auxLimits, &StopPos[i], &AxisTimes[i]);
	    /* Get initial conditions for each vectors Acc Vel and Pos */

	    Dir[i] = SIGN(IC.v);
	    IC.x = D;
	    GD =  StopPos[i]* Dir[i];

	    if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC,  Dir[i], AxisTimes[i] , &FC, &Acc[i], &Vel[i], &Pos[i], TP_ON)!=OK) {
	      PRINT_ERROR(" Verify Times axis ");
	      *report = S_xarm_ERROR_TP;
	      SDI_F->mode = XARM_STOP;
	    } 

	    StopPos[i] = FC.x;
	    /* Get Number of Elements needed for Vectors Definition */
	    if (tpGetMonotonicTimes( AxisTimes[i], &TM, &NOE)!=OK) {
	      PRINT_ERROR(" Get MonotonicTimes ");
	      *report = S_xarm_ERROR_TP;
	      SDI_F->mode = XARM_STOP;
	    }
	    tpGetNumberOfElement (&TM, &TNE);
	    Time[i] = TNE.Tjpa + TNE.Taca + TNE.Tjna + TNE.Tvc + TNE.Tjnb + TNE.Tacb + TNE.Tjpb;

	    if (Time[i] > MaxTimeM) {
	      MaxTimeM = Time[i];
	    }
	    
	    queueAddDetails(&(SDI_F->Seg[i]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[i]);
	    queueAddDetails(&(SDI_F->Seg[i]), 2, TNE.Taca, Jerks.J1, Acc[i].Tjpa, Vel[i].Tjpa, Pos[i].Tjpa,  Dir[i]);
	    queueAddDetails(&(SDI_F->Seg[i]), 3, TNE.Tjna, Jerks.J1, Acc[i].Taca, Vel[i].Taca, Pos[i].Taca,  Dir[i]);
	    queueAddDetails(&(SDI_F->Seg[i]), 4, TNE.Tvc,  Jerks.J1, Acc[i].Tjna, Vel[i].Tjna, Pos[i].Tjna,  Dir[i]);
	    queueAddDetails(&(SDI_F->Seg[i]), 5, TNE.Tjnb, Jerks.J1, Acc[i].Tvc,  Vel[i].Tvc,  Pos[i].Tvc,   Dir[i]);
	    queueAddDetails(&(SDI_F->Seg[i]), 6, TNE.Tacb, Jerks.J1, Acc[i].Tjnb, Vel[i].Tjnb, Pos[i].Tjnb,  Dir[i]);
	    queueAddDetails(&(SDI_F->Seg[i]), 7, TNE.Tjpb, Jerks.J1, Acc[i].Tacb, Vel[i].Tacb, Pos[i].Tacb,  Dir[i]);
	  }

	  
	  /* Add Zero to adjust number of elements in vectors */
	  queueAddDetails(&(SDI_F->Seg[0]), 4, (MaxTimeM - Time[0] + 5), Jerks.J1, 0, 0, StopPos[0], 1);   
	  queueAddDetails(&(SDI_F->Seg[1]), 4, (MaxTimeM - Time[1] + 5), Jerks.J1, 0, 0, StopPos[1], 1);
	  queueAddDetails(&(SDI_F->Seg[2]), 4, (MaxTimeM - Time[2] + 5), Jerks.J1, 0, 0, StopPos[2], 1);
	  queueAddDetails(&(SDI_F->Seg[3]), 4,(MaxTimeM - Time[3] + 5), Jerks.J1, 0, 0, StopPos[3], 1);
	  queueAddDetails(&(SDI_F->Seg[4]), 4,(MaxTimeM - Time[4] + 5), Jerks.J1, 0, 0, StopPos[4], 1);
	  queueAddDetails(&(SDI_F->Seg[5]), 4,(MaxTimeM - Time[5] + 5), Jerks.J1, 0, 0, StopPos[5], 1);
	  queueAddDetails(&(SDI_F->Seg[6]), 4,(MaxTimeM - Time[6] + 5), Jerks.J1, 0, 0, StopPos[6], 1);
	  
	  SweetStopInit = 1;
	  initTakeSeg = 0;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_ON;
	} /* End SweetStopInit */
      }
      else {
	SDI_F->SweetStop = 0;
	SweetStopInit = 0;
	initTakeSeg = 0;
	VerifyIfItsLastSegment = 0;
	printf("XARM_INITIAL_STATE\n");
	SDI_F->mode = XARM_INITIAL_STATE;
	SDI_F->consignType = XARM_NOT_SELECTED_CONSIGN_TYPE;
	SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;

	/* Initialize queues */
	queueInit(&(SDI_F->Seg[0]), "SegX");
	queueInit(&(SDI_F->Seg[1]), "SegY");
	queueInit(&(SDI_F->Seg[2]), "SegZ");
	queueInit(&(SDI_F->Seg[3]), "SegQN");
	queueInit(&(SDI_F->Seg[4]), "SegQI");
	queueInit(&(SDI_F->Seg[5]), "SegQJ");
	queueInit(&(SDI_F->Seg[6]), "SegQK"); 
 
	if (SDI_F->saveFiles == 1) {
	  if ((fileptr = fopen("RefTP.dat","wb"))==NULL) {
	    PRINT_ERROR("Open File RefTP.dat");
	    return ERROR;
	  }
	  fileIsOpen = 1;

	  fprintf(fileptr,"# i VROWS.PX_Acc VROWS.PX_Vel VROWS.PX_Pos VROWS.PY_Acc VROWS.PY_Vel VROWS.PY_Pos VROWS.PZ_Acc VROWS.PZ_Vel VROWS.PZ_Pos");
	  fprintf(fileptr," VROWS.QN_Acc VROWS.QN_Vel VROWS.QN_Pos VROWS.QI_Acc VROWS.QI_Vel VROWS.QI_Pos VROWS.QJ_Acc VROWS.QJ_Vel VROWS.QJ_Pos VROWS.QK_Acc VROWS.QK_Vel VROWS.QK_Pos VROWSREAL.PX_Pos VROWSREAL.PY_Pos VROWSREAL.PZ_Pos VROWSREAL.QN_Pos VROWSREAL.QI_Pos VROWSREAL.QJ_Pos VROWSREAL.QK_Pos VxJoystick ControlOut_Vx ControlOut_Vy ControlOut_Vz ControlOut_Wx ControlOut_Wy ControlOut_Wz SaveQPos_q1 SaveQPos_q2 SaveQPos_q3 SaveQPos_q4 SaveQPos_q5 SaveQPos_q6 SaveQPos_q1Vel SaveQPos_q2Vel SaveQPos_q3Vel SaveQPos_q4Vel SaveQPos_q5Vel SaveQPos_q6Vel ;\n");
	  printf("indexelement %d \n",indexElement);
	  for(i=0;i<indexElement;i++) {
	    fprintf(fileptr,"%d %f %f %f %f %f %f %f %f %f ",i, VROWS.PX_Acc.el[0][i],VROWS.PX_Vel.el[0][i],VROWS.PX_Pos.el[0][i],VROWS.PY_Acc.el[0][i],VROWS.PY_Vel.el[0][i],VROWS.PY_Pos.el[0][i],VROWS.PZ_Acc.el[0][i],VROWS.PZ_Vel.el[0][i],VROWS.PZ_Pos.el[0][i]);
	    fprintf(fileptr,"%f %f %f %f %f %f %f %f %f %f %f %f ",VROWS.QN_Acc.el[0][i],VROWS.QN_Vel.el[0][i],VROWS.QN_Pos.el[0][i],VROWS.QI_Acc.el[0][i],VROWS.QI_Vel.el[0][i],VROWS.QI_Pos.el[0][i],VROWS.QJ_Acc.el[0][i],VROWS.QJ_Vel.el[0][i],VROWS.QJ_Pos.el[0][i],VROWS.QK_Acc.el[0][i],VROWS.QK_Vel.el[0][i],VROWS.QK_Pos.el[0][i]);
	    fprintf(fileptr,"%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f ;\n",VROWSREAL.PX_Pos.el[0][i], VROWSREAL.PY_Pos.el[0][i], VROWSREAL.PZ_Pos.el[0][i], VROWSREAL.QN_Pos.el[0][i], VROWSREAL.QI_Pos.el[0][i], VROWSREAL.QJ_Pos.el[0][i], VROWSREAL.QK_Pos.el[0][i], VjoyRef[i], ControlOut_Vx[i], ControlOut_Vy[i], ControlOut_Vz[i], ControlOut_Wx[i], ControlOut_Wy[i], ControlOut_Wz[i], SaveQPos_q1[i], SaveQPos_q2[i], SaveQPos_q3[i], SaveQPos_q4[i], SaveQPos_q5[i], SaveQPos_q6[i], SaveQPos_q1Vel[i], SaveQPos_q2Vel[i], SaveQPos_q3Vel[i], SaveQPos_q4Vel[i], SaveQPos_q5Vel[i], SaveQPos_q6Vel[i]);
	  }
	  cptSaveData = 0;
	  indexToSaveData = i;
	  fclose(fileptr);
	  printf("File RefTP.dat is written !\n");
	  indexElement = 0;
	  return OK;
	  
	} else {
	  indexElement = 0;
	  return OK;
	}
      }
    }
    
    
    if (initTakeSeg == 0) {
    //  indexElement = 0;
      LTick.X = 1;
      LTick.Y = 1;
      LTick.Z = 1;
      LTick.QN = 1;
      LTick.QI = 1;
      LTick.QJ = 1;
      LTick.QK = 1;
      
      /* Verify if there is an empty queue when this mode is enable */
      VerifyAllQueue(&(SDI_F->Seg[0]), &(SDI_F->Seg[1]), &(SDI_F->Seg[2]), &(SDI_F->Seg[3]), &(SDI_F->Seg[4]), &(SDI_F->Seg[5]), &(SDI_F->Seg[6]), &VerifyIfItsLastSegment);
	     
      /* Init XARM - PA10 Control Params */
      //     if (InitParamsPa10ControlC1(&SDI_F->PIGain, &SDI_F->pa10Control) != OK) {
      if (InitParamsPa10Control(&SDI_F->PIGain, &SDI_F->limitsGoto, &SDI_F->pa10Control) != OK) {
	PRINT_ERROR(" Init Params Pa10 Control ");
	*report = S_xarm_ERROR_TP;
	SDI_F->mode = XARM_STOP;
	//return ERROR;
      }
      /* Init Controls XARM - PA10 */
      if (InitControlsPa10(&SDI_F->pa10Control) != OK) {
	PRINT_ERROR(" Init Params Pa10 Control ");
	*report = S_xarm_ERROR_TP;
	SDI_F->mode = XARM_STOP;
	SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	return OK;
      }
      /* Take first segment */
      for(i=0; i<XARM_NB_DIM; i++) {
	if(queueGetAndRemove(&(SDI_F->Seg[i]), &LSeg[i])!=0) {
	  PRINT_ERROR("xarmQueueEmpty\n");
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	}
      }
      initTakeSeg = 1;
    }
 
    /* Calcul Acc, Vel and Pos for this tick */
    /* PX */      
    if ( LTick.X <= LSeg[0].time ) {
      if (xarmCalculOfAccVelPos( LTick.X, &LSeg[0], &(Vectors.X))!= 0) {
	PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	*report=S_xarm_ERROR_TP;
	SDI_F->mode = XARM_STOP;
	SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	return OK;
	//return ERROR;
      }
    } else {
      if (queueVerifyIfEmpty(&(SDI_F->Seg[0]))!=0) {
	//	printf("xarm QUEUE Seg[0] is empty \n");
	VerifyIfItsLastSegment ++;
      } else {
	if(queueGetAndRemove(&(SDI_F->Seg[0]), &LSeg[0])!=0) {
	 // PRINT_ERROR("xarmQueueEmpty\n");
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;	
	  //return ERROR;
	}
	LTick.X = 1;
	if (xarmCalculOfAccVelPos( LTick.X, &LSeg[0], &(Vectors.X))!=0) {
	  PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	  *report=S_xarm_ERROR_TP;
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
      }
    }
    /* PY */
    if ( LTick.Y <= LSeg[1].time ) {
      if (xarmCalculOfAccVelPos( LTick.Y, &LSeg[1], &(Vectors.Y))!=0) {
	PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	*report=S_xarm_ERROR_TP;
	SDI_F->mode = XARM_STOP;
	SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	return OK;
	//return ERROR;
      }
    } else {
      if (queueVerifyIfEmpty(&(SDI_F->Seg[1]))!=0) {
//	printf("xarm QUEUE Seg[1] is empty \n");
	VerifyIfItsLastSegment ++;
      } else {
	if(queueGetAndRemove(&(SDI_F->Seg[1]), &LSeg[1])!=0) {
	//  PRINT_ERROR("xarmQueueEmpty\n");
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
	LTick.Y = 1;
	if (xarmCalculOfAccVelPos( LTick.Y, &LSeg[1], &(Vectors.Y))!=0) {
	  PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	  *report=S_xarm_ERROR_TP;
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
      }
    }
    /* PZ */
    if ( LTick.Z <= LSeg[2].time ) {
      if (xarmCalculOfAccVelPos( LTick.Z, &LSeg[2], &(Vectors.Z))!=0) {
	PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	*report=S_xarm_ERROR_TP;
	SDI_F->mode = XARM_STOP;
	SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	return OK;
	//return ERROR;
      }
    } else {
      if (queueVerifyIfEmpty(&(SDI_F->Seg[2]))!=0) {
//	printf("xarm QUEUE Seg[2] is empty \n");
	VerifyIfItsLastSegment ++;
      } else {
	if(queueGetAndRemove(&(SDI_F->Seg[2]), &LSeg[2])!=0) {
	//  PRINT_ERROR("xarmQueueEmpty\n");
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
	LTick.Z = 1;
	if (xarmCalculOfAccVelPos( LTick.Z, &LSeg[2], &(Vectors.Z))!=0) {
	  PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	  *report=S_xarm_ERROR_TP;
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
      }
    }
    /* QN */
    if ( LTick.QN <= LSeg[3].time ) {
      if (xarmCalculOfAccVelPos( LTick.QN, &LSeg[3], &(Vectors.QN))!=0) {
	PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	*report=S_xarm_ERROR_TP;
	SDI_F->mode = XARM_STOP;
	SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	return OK;
	//return ERROR;
      }
    } else {
      if (queueVerifyIfEmpty(&(SDI_F->Seg[3]))!=0) {
//	printf("xarm QUEUE SegQN is empty \n");
	VerifyIfItsLastSegment ++;
      } else {
	if(queueGetAndRemove(&(SDI_F->Seg[3]), &LSeg[3])!=0) {
//	  PRINT_ERROR("xarmQueueEmpty\n");
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
	LTick.QN = 1;
	if (xarmCalculOfAccVelPos( LTick.QN, &LSeg[3], &(Vectors.QN))!=0) {
	  PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	  *report=S_xarm_ERROR_TP;
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}  
      }
    }
    /* QI */
    if ( LTick.QI <= LSeg[4].time ) {
      if (xarmCalculOfAccVelPos( LTick.QI, &LSeg[4], &(Vectors.QI))!=0) {
	PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	*report=S_xarm_ERROR_TP;
	SDI_F->mode = XARM_STOP;
	SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	return OK;
	//return ERROR;
      }
    } else {
      if (queueVerifyIfEmpty(&(SDI_F->Seg[4]))!=0) {
//	printf("xarm QUEUE SegQI is empty \n");
	VerifyIfItsLastSegment ++;
      } else {
	if(queueGetAndRemove(&(SDI_F->Seg[4]), &LSeg[4])!=0) {
//	  PRINT_ERROR("xarmQueueEmpty\n");
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
	LTick.QI = 1;
	if (xarmCalculOfAccVelPos( LTick.QI, &LSeg[4], &(Vectors.QI))!=0) {
	  PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	  *report=S_xarm_ERROR_TP;
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	} 	
      }
    }
    /* QJ */
    if ( LTick.QJ <= LSeg[5].time ) {
      if (xarmCalculOfAccVelPos( LTick.QJ, &LSeg[5], &(Vectors.QJ))!=0) {
	PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	*report=S_xarm_ERROR_TP;
	SDI_F->mode = XARM_STOP;
	SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	return OK;
	//return ERROR;
      }
    } else {
      if (queueVerifyIfEmpty(&(SDI_F->Seg[5]))!=0) {
//	printf("xarm QUEUE SegQJ is empty \n");
	VerifyIfItsLastSegment ++;
      } else {
	if(queueGetAndRemove(&(SDI_F->Seg[5]), &LSeg[5])!=0) {
//	  PRINT_ERROR("xarmQueueEmpty\n");
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
	LTick.QJ = 1;
	if (xarmCalculOfAccVelPos( LTick.QJ, &LSeg[5], &(Vectors.QJ))!=0) {
	  PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	  *report=S_xarm_ERROR_TP;
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
      }
    }
    /* QK */
    if ( LTick.QK <= LSeg[6].time ) {
      if (xarmCalculOfAccVelPos( LTick.QK, &LSeg[6], &(Vectors.QK))!=0) {
	PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	*report=S_xarm_ERROR_TP;
	SDI_F->mode = XARM_STOP;
	SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	return OK;
	//return ERROR;
      }
    } else {
      if (queueVerifyIfEmpty(&(SDI_F->Seg[6]))!=0) {
//	printf("xarm QUEUE SegQK is empty \n");
	VerifyIfItsLastSegment ++;
      } else {
	if(queueGetAndRemove(&(SDI_F->Seg[6]), &LSeg[6])!=0) {
//	  PRINT_ERROR("xarmQueueEmpty\n");
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
	LTick.QK = 1;
	if (xarmCalculOfAccVelPos( LTick.QK, &LSeg[6], &(Vectors.QK))!=0) {
	  PRINT_ERROR("xarm Motion ERROR Calcul of Acc, Vel and Pos ");
	  *report=S_xarm_ERROR_TP;
	  SDI_F->mode = XARM_STOP;
	  SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
	  return OK;
	  //return ERROR;
	}
      }
    }
    /* Verify if there is another segment when the last time of the curent segment is reached */
    if ( LTick.X == LSeg[0].time ) {
      if (queueVerifyIfEmpty(&(SDI_F->Seg[0]))!=0) {
//	printf("xarm QUEUE SegX is empty \n");
	VerifyIfItsLastSegment ++;
      }
    }
    if ( LTick.Y == LSeg[1].time ) {      
      if (queueVerifyIfEmpty(&(SDI_F->Seg[1]))!=0) {
//	printf("xarm QUEUE SegY is empty \n");
	VerifyIfItsLastSegment ++;
      }
    }
    if ( LTick.Z == LSeg[2].time ) {  
      if (queueVerifyIfEmpty(&(SDI_F->Seg[2]))!=0) {
//	printf("xarm QUEUE SegZ is empty \n");
	VerifyIfItsLastSegment ++;
      }
    }
    if ( LTick.QN == LSeg[3].time ) {  
      if (queueVerifyIfEmpty(&(SDI_F->Seg[3]))!=0) {
//	printf("xarm QUEUE SegQN is empty \n");
	VerifyIfItsLastSegment ++;
      }
    }
    if ( LTick.QI == LSeg[4].time ) {  
      if (queueVerifyIfEmpty(&(SDI_F->Seg[4]))!=0) {
//	printf("xarm QUEUE SegQI is empty \n");
	VerifyIfItsLastSegment ++;
      }
    }
    if ( LTick.QJ == LSeg[5].time ) {  
      if (queueVerifyIfEmpty(&(SDI_F->Seg[5]))!=0) {
//	printf("xarm QUEUE SegQJ is empty \n");
	VerifyIfItsLastSegment ++;
      }
    }
    if ( LTick.QK == LSeg[6].time ) {  
      if (queueVerifyIfEmpty(&(SDI_F->Seg[6]))!=0) {
//	printf("xarm QUEUE SegQK is empty \n");
	VerifyIfItsLastSegment ++;
      }
    }      
    

    /* Save Data to plot curves */
    if (SDI_F->saveFiles == 1) {
      VROWS.PX_Acc.el[0][indexElement]     = Vectors.X.Acc;
      VROWS.PX_Vel.el[0][indexElement]     = Vectors.X.Vel;
      VROWS.PX_Pos.el[0][indexElement]     = Vectors.X.Pos; 
      VROWS.PY_Acc.el[0][indexElement]     = Vectors.Y.Acc;
      VROWS.PY_Vel.el[0][indexElement]     = Vectors.Y.Vel;
      VROWS.PY_Pos.el[0][indexElement]     = Vectors.Y.Pos; 
      VROWS.PZ_Acc.el[0][indexElement]     = Vectors.Z.Acc;
      VROWS.PZ_Vel.el[0][indexElement]     = Vectors.Z.Vel;
      VROWS.PZ_Pos.el[0][indexElement]     = Vectors.Z.Pos; 
      VROWS.QN_Acc.el[0][indexElement]     = Vectors.QN.Acc;
      VROWS.QN_Vel.el[0][indexElement]     = Vectors.QN.Vel;
      VROWS.QN_Pos.el[0][indexElement]     = Vectors.QN.Pos; 
      VROWS.QI_Acc.el[0][indexElement]     = Vectors.QI.Acc;
      VROWS.QI_Vel.el[0][indexElement]     = Vectors.QI.Vel;
      VROWS.QI_Pos.el[0][indexElement]     = Vectors.QI.Pos; 
      VROWS.QJ_Acc.el[0][indexElement]     = Vectors.QJ.Acc;
      VROWS.QJ_Vel.el[0][indexElement]     = Vectors.QJ.Vel;
      VROWS.QJ_Pos.el[0][indexElement]     = Vectors.QJ.Pos; 
      VROWS.QK_Acc.el[0][indexElement]     = Vectors.QK.Acc;
      VROWS.QK_Vel.el[0][indexElement]     = Vectors.QK.Vel;
      VROWS.QK_Pos.el[0][indexElement]     = Vectors.QK.Pos; 
      VROWSREAL.PX_Pos.el[0][indexElement] = SDI_F->pose.p.x;
      VROWSREAL.PY_Pos.el[0][indexElement] = SDI_F->pose.p.y;
      VROWSREAL.PZ_Pos.el[0][indexElement] = SDI_F->pose.p.z;
      VROWSREAL.QN_Pos.el[0][indexElement] = SDI_F->pose.q.n;
      VROWSREAL.QI_Pos.el[0][indexElement] = SDI_F->pose.q.i;
      VROWSREAL.QJ_Pos.el[0][indexElement] = SDI_F->pose.q.j;
      VROWSREAL.QK_Pos.el[0][indexElement] = SDI_F->pose.q.k;
    }

    SDI_F->mode = XARM_FIXE_TRAJECTORY;
    break;

  default:
    PRINT_ERROR("Wrong xarmTrackSegment Mode");
    *report=S_xarm_ERROR_MODE;
    SDI_F->mode = XARM_STOP;
    //return ERROR;
  }
  
  /* Send velocity reference to qarm */ 
  switch(SDI_F->mode) {
  case XARM_INITIAL_STATE:
    SDI_F->dq.counter    += 1;
    memset(&(SDI_F->dq.qVelocityC), 0, sizeof(Gb_q6));
    /*     SDI_F->dq.qVelocityC.q1 = 0.0; */
    /*     SDI_F->dq.qVelocityC.q2 = 0.0; */
    /*     SDI_F->dq.qVelocityC.q3 = 0.0; */
    /*     SDI_F->dq.qVelocityC.q4 = 0.0; */
    /*     SDI_F->dq.qVelocityC.q5 = 0.0; */
    /*     SDI_F->dq.qVelocityC.q6 = 0.0; */
    /* Update Motion Conditions */                                     
    SDI_F->motionConditions.x.a = 0;
    SDI_F->motionConditions.x.v = 0; 
    SDI_F->motionConditions.x.x = SDI_F->pose.p.x;
    SDI_F->motionConditions.y.a = 0; 
    SDI_F->motionConditions.y.v = 0; 
    SDI_F->motionConditions.y.x = SDI_F->pose.p.y; 
    SDI_F->motionConditions.z.a = 0; 
    SDI_F->motionConditions.z.v = 0; 
    SDI_F->motionConditions.z.x = SDI_F->pose.p.z; 
    SDI_F->motionConditions.n.a = 0;
    SDI_F->motionConditions.n.v = 0; 
    SDI_F->motionConditions.n.x = SDI_F->pose.q.n;
    SDI_F->motionConditions.i.a = 0;
    SDI_F->motionConditions.i.v = 0;
    SDI_F->motionConditions.i.x = SDI_F->pose.q.i;
    SDI_F->motionConditions.j.a = 0;
    SDI_F->motionConditions.j.v = 0;
    SDI_F->motionConditions.j.x = SDI_F->pose.q.j;
    SDI_F->motionConditions.k.a = 0;
    SDI_F->motionConditions.k.v = 0;
    SDI_F->motionConditions.k.x = SDI_F->pose.q.k;
    SDI_F->xarmTrackState.state = 0;
    break;

  case XARM_SINGLE_POSE:
  case XARM_EXTERNAL_POSE:
  case XARM_EXTERNAL_CONF:
  case XARM_EXTERNAL_DRIVE:
  case XARM_FIXE_TRAJECTORY:

    /* Update Motion Conditions */                                     
    SDI_F->motionConditions.x.a = Vectors.X.Acc;  
    SDI_F->motionConditions.x.v = Vectors.X.Vel;  
    SDI_F->motionConditions.x.x = Vectors.X.Pos;  
    SDI_F->motionConditions.y.a = Vectors.Y.Acc;  
    SDI_F->motionConditions.y.v = Vectors.Y.Vel;  
    SDI_F->motionConditions.y.x = Vectors.Y.Pos;  
    SDI_F->motionConditions.z.a = Vectors.Z.Acc;  
    SDI_F->motionConditions.z.v = Vectors.Z.Vel;  
    SDI_F->motionConditions.z.x = Vectors.Z.Pos;  
    SDI_F->motionConditions.n.a = Vectors.QN.Acc; 
    SDI_F->motionConditions.n.v = Vectors.QN.Vel; 
    SDI_F->motionConditions.n.x = Vectors.QN.Pos; 
    SDI_F->motionConditions.i.a = Vectors.QI.Acc; 
    SDI_F->motionConditions.i.v = Vectors.QI.Vel; 
    SDI_F->motionConditions.i.x = Vectors.QI.Pos; 
    SDI_F->motionConditions.j.a = Vectors.QJ.Acc; 
    SDI_F->motionConditions.j.v = Vectors.QJ.Vel; 
    SDI_F->motionConditions.j.x = Vectors.QJ.Pos; 
    SDI_F->motionConditions.k.a = Vectors.QK.Acc; 
    SDI_F->motionConditions.k.v = Vectors.QK.Vel; 
    SDI_F->motionConditions.k.x = Vectors.QK.Pos; 

    /* Quaternion NORMALIZATION */
    normaQ = Vectors.QN.Pos * Vectors.QN.Pos +  Vectors.QI.Pos * Vectors.QI.Pos +  Vectors.QJ.Pos * Vectors.QJ.Pos +  Vectors.QK.Pos * Vectors.QK.Pos;
    normaQ=sqrt(normaQ);
    if (isnan(normaQ) || normaQ == 0 || isinf(normaQ) ) { 
      printf("normaQ %g  Vectors.QN.Pos %g  Vectors.QI.Pos %g  Vectors.QJ.Pos %g  Vectors.QK.Pos %g\n",normaQ , Vectors.QN.Pos,  Vectors.QI.Pos, Vectors.QJ.Pos,  Vectors.QK.Pos); 
    }
    Vectors.QN.Pos = Vectors.QN.Pos/normaQ;
    Vectors.QI.Pos = Vectors.QI.Pos/normaQ;
    Vectors.QJ.Pos = Vectors.QJ.Pos/normaQ;
    Vectors.QK.Pos = Vectors.QK.Pos/normaQ;
    
    /* Omega Construction */    
    /* Quaternion */
    Quat.n = Vectors.QN.Pos;
    Quat.i = Vectors.QI.Pos;
    Quat.j = Vectors.QJ.Pos;
    Quat.k = Vectors.QK.Pos;
    /* Quaternion Derive */
    DQuat.n = Vectors.QN.Vel;
    DQuat.i = Vectors.QI.Vel;
    DQuat.j = Vectors.QJ.Vel;
    DQuat.k = Vectors.QK.Vel;	 
    /* Convertion */
    if (quat2omega(Quat, DQuat, &Omega)!= OK) {
      PRINT_ERROR("Quaternion 2 Omega convertion \n");
      *report = S_xarm_ERROR_TP;
      return ERROR; 
    }
   
    extPose.px = 0.0;
    extPose.py = 0.0;
    extPose.pz = 0.0;
    extPose.qn = 0.0;
    extPose.qi = 0.0;
    extPose.qj = 0.0;
    extPose.qk = 0.0;
    extPose.vx = 0.0;
    extPose.vy = 0.0;
    extPose.vz = 0.0;
    extPose.wx = 0.0;
    extPose.wy = 0.0;
    extPose.wz = 0.0;
  
    /* qVel */
    SDI_F->pa10Arm.qVel.q1 = 0;
    SDI_F->pa10Arm.qVel.q2 = 0;
    SDI_F->pa10Arm.qVel.q3 = 0;
    SDI_F->pa10Arm.qVel.q4 = 0;
    SDI_F->pa10Arm.qVel.q5 = 0;
    SDI_F->pa10Arm.qVel.q6 = 0;

    MatPReal.el[0][0] = SDI_F->pose.p.x;
    MatPReal.el[0][1] = SDI_F->pose.p.y; 
    MatPReal.el[0][2] = SDI_F->pose.p.z;

    /* Control Loop for  PX */
    ErrorDim = SDI_F->pose.p.x - Vectors.X.Pos;
    MatErrorP.el[0][0] = ErrorDim;
    if (CalculLawOfControl(&SDI_F->pa10Control.PX,ErrorDim)!= OK) {
      PRINT_ERROR(" PX Law of Control Pa10 Control ");
      *report = S_xarm_ERROR_TP;
      return ERROR;
    }
    Vx = Vectors.X.Vel - SDI_F->pa10Control.PX.Uk;

    /* Control Loop for  PY */
    /* Visual Servoing */
    /* fprintf(stdout, "YC %f \n",extPose.py); */
    ei_y = extPose.py;
    //  printf("Erreur PY line = %f\n", extPose.py);
    ui_y = (Kpi_y + Kii_y)*ei_y - Kpi_y*ei_y_1 + ui_y_1;
    if (ui_y > 0.05) ui_y = 0.05;
    if (ui_y <-0.05) ui_y =-0.05;
    ei_y_1 = ei_y;
    ui_y_1 = ui_y;


    ErrorDim = SDI_F->pose.p.y - Vectors.Y.Pos + ui_y;
    MatErrorP.el[0][1] = ErrorDim;
    if (CalculLawOfControl(&SDI_F->pa10Control.PY,ErrorDim)!= OK) {
      PRINT_ERROR(" PY Law of Control Pa10 Control ");
      *report = S_xarm_ERROR_TP;
      return ERROR;
    }
    Vy = Vectors.Y.Vel - SDI_F->pa10Control.PY.Uk;

    /* Control Loop for  PZ */
    ErrorDim = SDI_F->pose.p.z - Vectors.Z.Pos;
    MatErrorP.el[0][2] = ErrorDim;
    if (CalculLawOfControl(&SDI_F->pa10Control.PZ,ErrorDim)!= OK) {
      PRINT_ERROR(" PZ Law of Control Pa10 Control ");
      *report = S_xarm_ERROR_TP;
      return ERROR;
    }
    Vz = Vectors.Z.Vel - SDI_F->pa10Control.PZ.Uk;

    /* Quaternion Auxiliar Variables */
    Qn = SDI_F->pose.q.n;
    Qi = SDI_F->pose.q.i;
    Qj = SDI_F->pose.q.j;
    Qk = SDI_F->pose.q.k;
    Rn = Vectors.QN.Pos;
    Ri = Vectors.QI.Pos;
    Rj = Vectors.QJ.Pos;     
    Rk = Vectors.QK.Pos;

    /* VERIFY CONTINUITY*/
    if (indexElement != 0) {
      cosTheta = MatQRealPrevious.el[0][0]*Qn + MatQRealPrevious.el[0][1]*Qi + MatQRealPrevious.el[0][2]*Qj + MatQRealPrevious.el[0][3]*Qk;
    } else {
      cosTheta = Qn;
    }
    if (cosTheta<0) {
      Qn = -Qn;
      Qi = -Qi;
      Qj = -Qj;
      Qk = -Qk;
    }

    MatQReal.el[0][0] = Qn;
    MatQReal.el[0][1] = Qi;
    MatQReal.el[0][2] = Qj;
    MatQReal.el[0][3] = Qk;
    /* Save data for next tick */
    MatQRealPrevious.el[0][0] = Qn;
    MatQRealPrevious.el[0][1] = Qi;
    MatQRealPrevious.el[0][2] = Qj;
    MatQRealPrevious.el[0][3] = Qk;

    /* QuaternionError */
    MatErrorQ.el[0][0] = Rn - Qn;
    MatErrorQ.el[0][1] = Ri - Qi;
    MatErrorQ.el[0][2] = Rj - Qj;
    MatErrorQ.el[0][3] = Rk - Qk;

    MatErrorQ.el[0][4] = extPose.py ;

    /*Quaternion Error = Rn*Qijk - Qn*Rijk - Rijk X Qijk */ 
    /* Control Loop for  Wx */
    ErrorDim = Rn*Qi-Qn*Ri+Rj*Qk-Rk*Qj;
    if (CalculLawOfControl(&SDI_F->pa10Control.TX,ErrorDim)!= OK) {
      PRINT_ERROR("Wx  Law of Control Pa10 Control ");
      printf("Qn %g Qi %g Qj %g Qk %g\n",Qn, Qi, Qj, Qk);
      printf("Rn %g Ri %g Rj %g Rk %g\n",Rn, Ri, Rj, Rk);
      printf("Omega.wx %g\n",Omega.wx );
      *report = S_xarm_ERROR_TP;
      return ERROR;
    }
    Wx =Omega.wx - SDI_F->pa10Control.TX.Uk;
    
    /* Control Loop for  Wy */
    ErrorDim = Rn*Qj-Qn*Rj+Rk*Qi-Ri*Qk;
    if (CalculLawOfControl(&SDI_F->pa10Control.TY,ErrorDim)!= OK) {
      PRINT_ERROR(" Wy Law of Control Pa10 Control ");
      printf("Qn %g Qi %g Qj %g Qk %g\n",Qn, Qi, Qj, Qk);
      printf("Rn %g Ri %g Rj %g Rk %g\n",Rn, Ri, Rj, Rk);
      printf("Omega.wy %g\n",Omega.wy );
      *report = S_xarm_ERROR_TP;
      return ERROR;
    }
    Wy = Omega.wy - SDI_F->pa10Control.TY.Uk;
    
    /* Control Loop for  Wz */
    ErrorDim = Rn*Qk-Qn*Rk+Ri*Qj-Rj*Qi;
    if (CalculLawOfControl(&SDI_F->pa10Control.TZ,ErrorDim)!= OK) {
      PRINT_ERROR(" Wz Law of Control Pa10 Control ");
      printf("Qn %g Qi %g Qj %g Qk %g\n",Qn, Qi, Qj, Qk);
      printf("Rn %g Ri %g Rj %g Rk %g\n",Rn, Ri, Rj, Rk);
      printf("Omega.wz %g\n",Omega.wz );
      *report = S_xarm_ERROR_TP;
      return ERROR;
    }
    Wz = Omega.wz - SDI_F->pa10Control.TZ.Uk;


    ControlOut_Vx[indexElement] = Vx;
    ControlOut_Vy[indexElement] = Vy;
    ControlOut_Vz[indexElement] = Vz;
    ControlOut_Wx[indexElement] = Wx;
    ControlOut_Wy[indexElement] = Wy;
    ControlOut_Wz[indexElement] = Wz;

    /* ******************BEGIN INVERSE JACOBIAN MATRIX ******************* */
    if (SRSQ_Inv(SDI_F->pa10Arm.Jac, SDI_F->pa10Arm.InvJac,auxJac, SDI_F->pa10Arm.K_WEIGHT)!= OK) {
      PRINT_ERROR(" Jacobian Inversion  ");
      *report = S_xarm_ERROR_TP;
      return ERROR;
    }
    /* ****************** END  INVERSE JACOBIAN MATRIX ******************* */

    SDI_F->pa10Arm.PoseVelocity.el[0][0] = Vx;
    SDI_F->pa10Arm.PoseVelocity.el[1][0] = Vy;
    SDI_F->pa10Arm.PoseVelocity.el[2][0] = Vz;
    SDI_F->pa10Arm.PoseVelocity.el[3][0] = Wx;
    SDI_F->pa10Arm.PoseVelocity.el[4][0] = Wy;
    SDI_F->pa10Arm.PoseVelocity.el[5][0] = Wz;

    SDI_F->pa10Arm.AxisVelocity.el[0][0] = 0;
    SDI_F->pa10Arm.AxisVelocity.el[1][0] = 0;
    SDI_F->pa10Arm.AxisVelocity.el[2][0] = 0;
    SDI_F->pa10Arm.AxisVelocity.el[3][0] = 0;
    SDI_F->pa10Arm.AxisVelocity.el[4][0] = 0;
    SDI_F->pa10Arm.AxisVelocity.el[5][0] = 0;

    MatQ.el[0][0] = SDI_F->pa10Arm.qPos.q1;
    MatQ.el[0][1] = SDI_F->pa10Arm.qPos.q2;
    MatQ.el[0][2] = SDI_F->pa10Arm.qPos.q3;
    MatQ.el[0][3] = SDI_F->pa10Arm.qPos.q4;
    MatQ.el[0][4] = SDI_F->pa10Arm.qPos.q5;
    MatQ.el[0][5] = SDI_F->pa10Arm.qPos.q6;

    MatQPoint.el[0][0] = Vx;
    MatQPoint.el[0][1] = Vy;
    MatQPoint.el[0][2] = Vz;
    MatQPoint.el[0][3] = Wx;
    MatQPoint.el[0][4] = Wy;
    MatQPoint.el[0][5] = Wz;

    /* Gets Axis Velocity */
    if(matmul(SDI_F->pa10Arm.InvJac,SDI_F->pa10Arm.PoseVelocity,SDI_F->pa10Arm.AxisVelocity)) {
      PRINT_ERROR("Axis Velocity calcul ");
      *report = S_xarm_ERROR_TP;
      return ERROR;
    }

    /* Limit axis velocity */
    coef = 0;
    for (i=0;i<XARM_NB_AXIS;i++) {
      auxCoef = ABS(SDI_F->pa10Arm.AxisVelocity.el[i][0]/SDI_F->pa10Arm.AxisVelocityLimits.el[0][i]);
      if (auxCoef>coef) {
	coef = auxCoef;
      }
    }
    if (coef>1.0) {
      for (i=0;i<XARM_NB_AXIS;i++) {
	SDI_F->pa10Arm.AxisVelocity.el[i][0]=SDI_F->pa10Arm.AxisVelocity.el[i][0]/coef;
      }
    }    

    /* qVel */
    SDI_F->pa10Arm.qVel.q1 = SDI_F->pa10Arm.AxisVelocity.el[0][0];
    SDI_F->pa10Arm.qVel.q2 = SDI_F->pa10Arm.AxisVelocity.el[1][0];
    SDI_F->pa10Arm.qVel.q3 = SDI_F->pa10Arm.AxisVelocity.el[2][0];
    SDI_F->pa10Arm.qVel.q4 = SDI_F->pa10Arm.AxisVelocity.el[3][0];
    SDI_F->pa10Arm.qVel.q5 = SDI_F->pa10Arm.AxisVelocity.el[4][0];
    SDI_F->pa10Arm.qVel.q6 = SDI_F->pa10Arm.AxisVelocity.el[5][0];

    /* Verify INF and NAN */
    if(isnan(SDI_F->pa10Arm.qVel.q1) || isnan(SDI_F->pa10Arm.qVel.q2) || isnan(SDI_F->pa10Arm.qVel.q3) ||
       isnan(SDI_F->pa10Arm.qVel.q4) || isnan(SDI_F->pa10Arm.qVel.q5) || isnan(SDI_F->pa10Arm.qVel.q6) ) {
      PRINT_ERROR("VELOCITY NAN - STOP TRACKING ");
      PRINT_DEBUG("indexElement: %d \n",indexElement);
      SDI_F->pa10Arm.qVel.q1 = 0;
      SDI_F->pa10Arm.qVel.q2 = 0;
      SDI_F->pa10Arm.qVel.q3 = 0;
      SDI_F->pa10Arm.qVel.q4 = 0;
      SDI_F->pa10Arm.qVel.q5 = 0;
      SDI_F->pa10Arm.qVel.q6 = 0;
      return ERROR;
    }  
    if(isinf(SDI_F->pa10Arm.qVel.q1) || isinf(SDI_F->pa10Arm.qVel.q2) || isinf(SDI_F->pa10Arm.qVel.q3) ||
       isinf(SDI_F->pa10Arm.qVel.q4) || isinf(SDI_F->pa10Arm.qVel.q5) || isinf(SDI_F->pa10Arm.qVel.q6) ) {
      PRINT_ERROR("VELOCITY INF - STOP TRACKING ");
      PRINT_DEBUG("indexElement: %d \n",indexElement);
      SDI_F->pa10Arm.qVel.q1 = 0;
      SDI_F->pa10Arm.qVel.q2 = 0;
      SDI_F->pa10Arm.qVel.q3 = 0;
      SDI_F->pa10Arm.qVel.q4 = 0;
      SDI_F->pa10Arm.qVel.q5 = 0;
      SDI_F->pa10Arm.qVel.q6 = 0;
      return ERROR;
    }
  
    /* Verify Limits */
    if (SDI_F->pa10Arm.qVel.q1> XARM_Q1_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q1= XARM_Q1_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q1<-XARM_Q1_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q1=-XARM_Q1_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q2> XARM_Q2_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q2= XARM_Q2_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q2<-XARM_Q2_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q2=-XARM_Q2_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q3> XARM_Q3_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q3= XARM_Q3_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q3<-XARM_Q3_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q3=-XARM_Q3_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q1> XARM_Q4_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q1= XARM_Q4_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q1<-XARM_Q4_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q1=-XARM_Q4_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q2> XARM_Q5_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q2= XARM_Q5_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q2<-XARM_Q5_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q2=-XARM_Q5_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q3> XARM_Q6_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q3= XARM_Q6_VELOCITY_LIMIT;
    if (SDI_F->pa10Arm.qVel.q3<-XARM_Q6_VELOCITY_LIMIT)  SDI_F->pa10Arm.qVel.q3=-XARM_Q6_VELOCITY_LIMIT;

    SDI_F->dq.counter    += 1;
    //    memcpy(&(SDI_F->dq.qVelocityC), &(SDI_F->pa10Arm.qVel), sizeof(Gb_q6));
    SDI_F->dq.qVelocityC = SDI_F->pa10Arm.qVel;
    /*     SDI_F->dq.qVelocityC.q1 = SDI_F->pa10Arm.qVel.q1; */
    /*     SDI_F->dq.qVelocityC.q2 = SDI_F->pa10Arm.qVel.q2; */
    /*     SDI_F->dq.qVelocityC.q3 = SDI_F->pa10Arm.qVel.q3; */
    /*     SDI_F->dq.qVelocityC.q4 = SDI_F->pa10Arm.qVel.q4; */
    /*     SDI_F->dq.qVelocityC.q5 = SDI_F->pa10Arm.qVel.q5; */
    /*     SDI_F->dq.qVelocityC.q6 = SDI_F->pa10Arm.qVel.q6; */ 

    SaveQPos_q1Vel[indexElement] = SDI_F->pa10Arm.qVel.q1;
    SaveQPos_q2Vel[indexElement] = SDI_F->pa10Arm.qVel.q2;
    SaveQPos_q3Vel[indexElement] = SDI_F->pa10Arm.qVel.q3;
    SaveQPos_q4Vel[indexElement] = SDI_F->pa10Arm.qVel.q4;
    SaveQPos_q5Vel[indexElement] = SDI_F->pa10Arm.qVel.q5;
    SaveQPos_q6Vel[indexElement] = SDI_F->pa10Arm.qVel.q6;

    /* Next Element */
    indexElement++;
    SDI_F->currentTimeOfTrajectory ++;
    LTick.X  ++;
    LTick.Y  ++;
    LTick.Z  ++;
    LTick.QN  ++;
    LTick.QI  ++;
    LTick.QJ  ++;
    LTick.QK  ++;

    break;

  case XARM_STOP:
    PRINT_ERROR("STOP CURRENT ACTIVITY - GO TO INITIAL STATE");
    SDI_F->mode = XARM_INITIAL_STATE;
    SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
    SDI_F->xarmTrackState.state = 1;
    SDI_F->dq.counter    += 1;
    SDI_F->dq.qVelocityC.q1 = 0.0;
    SDI_F->dq.qVelocityC.q2 = 0.0;
    SDI_F->dq.qVelocityC.q3 = 0.0;
    SDI_F->dq.qVelocityC.q4 = 0.0;
    SDI_F->dq.qVelocityC.q5 = 0.0;
    SDI_F->dq.qVelocityC.q6 = 0.0;

    /* Initialize queues */
    queueInit(&(SDI_F->Seg[0]), "SegX");
    queueInit(&(SDI_F->Seg[1]), "SegY");
    queueInit(&(SDI_F->Seg[2]), "SegZ");
    queueInit(&(SDI_F->Seg[3]), "SegQN");
    queueInit(&(SDI_F->Seg[4]), "SegQI");
    queueInit(&(SDI_F->Seg[5]), "SegQJ");
    queueInit(&(SDI_F->Seg[6]), "SegQK");

    break;
  default:
    PRINT_ERROR("Wrong Mode ");
    *report=S_xarm_ERROR_MODE;
    return ERROR;
  }
  
  return OK;
}

/*------------------------------------------------------------------------
 *
 * xarmMotionPerm2  --  2nd codel of permanent activity (called AFTER other activities)
 *
 * Description: 
 * 
 * Reports:      OK
 *              S_xarm_ERROR_MODE
 *              S_xarm_POSTER_NOT_FOUND
 *              S_xarm_POSTER_NOT_READ
 *              S_xarm_WRONG_VALUE
 *              S_xarm_ERROR_TP
 * 
 * Returns:    OK or ERROR
 */

STATUS
xarmMotionPerm2(int *report)
{
  /* Add your code here */
  return OK;
}

#define NEWDMAT_IS_OK(testError, mess, m_report)		  \
  do {								  \
    if (testError) {						  \
      PRINT_ERROR(mess);					  \
      *report = m_report;					  \
      return ETHER;						  \
    }								  \
  } while(0)							  \


/*------------------------------------------------------------------------
 * Init
 *
 * Description: 
 *
 * Reports:      OK
 */

/* xarmInitMain  -  codel EXEC of Init
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmInitMain(int *report)
{
  int errnoMatDef;

  /* ENABLE SAVE DATA */
  SDI_F->saveFiles = 0; /* 0: no save , 1:save */
  SDI_F->printData = 0;
  cptSaveData = DELAY_SAVE_DATA +1;

   /* INIT XARM R6/Rt */
  InitxarmR6Rt(&SDI_F->pa10Arm.r0, &SDI_F->pa10Arm.r6, &SDI_F->pa10Arm.r7, &SDI_F->pa10Arm.r8z);

  /* INIT MODE */
  SDI_F->mode = XARM_INITIAL_STATE;
  SDI_F->xarmTrackSegment = XARM_TRACK_OFF;

 /* INIT POSTERS */
  if (posterFind(QARM_POSITION_POSTER_NAME, &posterQarmPos) == ERROR) {
    PRINT_ERROR("Can not find POSTER POSITION");
    printf ("Can not find POSTER POSITION %s", QARM_POSITION_POSTER_NAME);
    *report =  S_xarm_POSTER_NOT_FOUND;
    return ETHER;
  }

  /* Jacobian Matrix Definition */
  SDI_F->pa10Arm.Jac = newdmat (0,XARM_NB_AXIS - 1,0,XARM_NB_AXIS - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Jacobian Matrix Definition", S_xarm_ERROR_TP);
  /* Inverted Jacobian Matrix Definition */
  SDI_F->pa10Arm.InvJac = newdmat (0,XARM_NB_AXIS - 1,0,XARM_NB_AXIS - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Inverted Jacobian Matrix Definition", S_xarm_ERROR_TP);
  /* Aux Jacobian Matrix Definition */
  auxJac = newdmat (0,XARM_NB_AXIS - 1,0,XARM_NB_AXIS - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Auxiliar Jacobian Matrix Definition", S_xarm_ERROR_TP);

  /* Pose Velocity Vector Definition */
  SDI_F->pa10Arm.PoseVelocity = newdmat (0,XARM_NB_AXIS - 1,0,0,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Pose Velocity Vector Definition", S_xarm_ERROR_TP);
  /* Axis Velocity Vector Definition */
  SDI_F->pa10Arm.AxisVelocity = newdmat (0,XARM_NB_AXIS - 1,0,0,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Axis Velocity Vector Definition",S_xarm_ERROR_TP);
  /* Axis Velocity Limits Vector Definition */
  SDI_F->pa10Arm.AxisVelocityLimits = newdmat (0,XARM_NB_AXIS - 1,0,0,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Axis Velocity Vector Definition",S_xarm_ERROR_TP);
 
  SDI_F->pa10Arm.AxisVelocityLimits.el[0][0] = XARM_Q1_VELOCITY_LIMIT;
  SDI_F->pa10Arm.AxisVelocityLimits.el[0][1] = XARM_Q2_VELOCITY_LIMIT;
  SDI_F->pa10Arm.AxisVelocityLimits.el[0][2] = XARM_Q3_VELOCITY_LIMIT;
  SDI_F->pa10Arm.AxisVelocityLimits.el[0][3] = XARM_Q4_VELOCITY_LIMIT;
  SDI_F->pa10Arm.AxisVelocityLimits.el[0][4] = XARM_Q5_VELOCITY_LIMIT;
  SDI_F->pa10Arm.AxisVelocityLimits.el[0][5] = XARM_Q6_VELOCITY_LIMIT;

   /* qVel */
  SDI_F->pa10Arm.qVel.q1 = 0;
  SDI_F->pa10Arm.qVel.q2 = 0;
  SDI_F->pa10Arm.qVel.q3 = 0;
  SDI_F->pa10Arm.qVel.q4 = 0;
  SDI_F->pa10Arm.qVel.q5 = 0;
  SDI_F->pa10Arm.qVel.q6 = 0;

  SDI_F->dq.counter = 0;
  //SDI_F->dq.counterMax = 50;
  SDI_F->dq.counterMax = 60;

  /* Initialize jerk, acceleration and velocity limits */
  SDI_F->limitsGoto.linear.maxJerk  = C1_MAX_LINEAR_JERK;
  SDI_F->limitsGoto.linear.maxAcc   = C1_MAX_LINEAR_ACCELERATION;
  SDI_F->limitsGoto.linear.maxVel   = C1_MAX_LINEAR_VELOCITY;
  SDI_F->limitsGoto.angular.maxJerk = C1_MAX_ANGULAR_JERK;
  SDI_F->limitsGoto.angular.maxAcc  = C1_MAX_ANGULAR_ACCELERATION;
  SDI_F->limitsGoto.angular.maxVel  = C1_MAX_ANGULAR_VELOCITY;

  SDI_F->limitsExtern.linear.maxJerk  = C2_MAX_LINEAR_JERK;
  SDI_F->limitsExtern.linear.maxAcc   = C2_MAX_LINEAR_ACCELERATION;
  SDI_F->limitsExtern.linear.maxVel   = C2_MAX_LINEAR_VELOCITY;
  SDI_F->limitsExtern.angular.maxJerk = C2_MAX_ANGULAR_JERK;
  SDI_F->limitsExtern.angular.maxAcc  = C2_MAX_ANGULAR_ACCELERATION;
  SDI_F->limitsExtern.angular.maxVel  = C2_MAX_ANGULAR_VELOCITY;

  /* Initialize queues */
  queueInit(&(SDI_F->Seg[0]), "SegX");
  queueInit(&(SDI_F->Seg[1]), "SegY");
  queueInit(&(SDI_F->Seg[2]), "SegZ");
  queueInit(&(SDI_F->Seg[3]), "SegQN");
  queueInit(&(SDI_F->Seg[4]), "SegQI");
  queueInit(&(SDI_F->Seg[5]), "SegQJ");
  queueInit(&(SDI_F->Seg[6]), "SegQK");
 
  /* Init PIGain */
  SDI_F->PIGain.Linear_Kp  = C1_XARM_KP_LINEAR;
  SDI_F->PIGain.Linear_Ti  = C1_XARM_TI_LINEAR;;
  SDI_F->PIGain.Angular_Kp = C1_XARM_KP_ANGULAR;
  SDI_F->PIGain.Angular_Ti = C1_XARM_TI_ANGULAR;

  initPa10Parameters();

  SDI_F->consignType = XARM_NOT_SELECTED_CONSIGN_TYPE;
  SDI_F->singularity = XARM_SINGULARITY_CROSS;
  

  initDone = 1;
  return ETHER;
}




/*------------------------------------------------------------------------
 * DefGotoAbsolutXYZThetaPosition
 *
 * Description: 
 *
 * Reports:      OK
 *              S_xarm_ERROR_DEF_ABSOLUT_XYZ_THETA_POSITION
 *              S_xarm_WRONG_ABSOLUT_POSE
 */

/* xarmDefGotoAbsolutXYZThetaPositionMain  -  codel EXEC of DefGotoAbsolutXYZThetaPosition
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmDefGotoAbsolutXYZThetaPositionMain(TP_ABSOLUT_POSITION *goalAbsolutPosition, int *report)
{

  Gb_th invPosGripOrigin_th;
  Gb_th goalAbsolutPose_th;
  Gb_th relativeDisplacement_th;
  double normaQ;
  Gb_th thTool;
  Gb_th Next_thTool;
  TP_QUAT GBQuatL;
  POSTER_ID posterQarmPos;
  QARM_POSITION_STR qarmPosition2;
 
  goalAbsolutPose_th.vx.x = cos(goalAbsolutPosition->theta);
  goalAbsolutPose_th.vx.y = sin(goalAbsolutPosition->theta);
  goalAbsolutPose_th.vx.z = 0;

  goalAbsolutPose_th.vy.x = -goalAbsolutPose_th.vx.y;
  goalAbsolutPose_th.vy.y = goalAbsolutPose_th.vx.x;
  goalAbsolutPose_th.vy.z = 0;

  goalAbsolutPose_th.vz.x = 0;
  goalAbsolutPose_th.vz.y = 0;
  goalAbsolutPose_th.vz.z = 1;

  goalAbsolutPose_th.vp.x = goalAbsolutPosition->x;
  goalAbsolutPose_th.vp.y = goalAbsolutPosition->y;
  goalAbsolutPose_th.vp.z = goalAbsolutPosition->z;
  /* this is the position in the main frame of of the goalAbsolutPosition */

  //  printf(" \n");
  //  printf("posAbsGoal x %f\n",(goalAbsolutPose_th.vp.x));
  //  printf("posAbsGoal y %f\n",(goalAbsolutPose_th.vp.y));
  //  printf("posAbsGoal z %f\n",(goalAbsolutPose_th.vp.z));
 
  /* posGripOrigin_th Inversion */
  invPosGripOrigin_th.vx.x = SDI_F->posGripOrigin_th.vx.x;
  invPosGripOrigin_th.vx.y = SDI_F->posGripOrigin_th.vy.x;
  invPosGripOrigin_th.vx.z = SDI_F->posGripOrigin_th.vz.x;   /* 0 */

  invPosGripOrigin_th.vy.x = SDI_F->posGripOrigin_th.vx.y;
  invPosGripOrigin_th.vy.y = SDI_F->posGripOrigin_th.vy.y;
  invPosGripOrigin_th.vy.z = SDI_F->posGripOrigin_th.vz.y;   /* 0 */

  invPosGripOrigin_th.vz.x = SDI_F->posGripOrigin_th.vx.z;   /* 0 */
  invPosGripOrigin_th.vz.y = SDI_F->posGripOrigin_th.vy.z;   /* 0 */
  invPosGripOrigin_th.vz.z = SDI_F->posGripOrigin_th.vz.z;   /* 1 */

  invPosGripOrigin_th.vp.x = -((invPosGripOrigin_th.vx.x)*(SDI_F->posGripOrigin_th.vp.x)+(invPosGripOrigin_th.vy.x)*(SDI_F->posGripOrigin_th.vp.y)+(invPosGripOrigin_th.vz.x)*(SDI_F->posGripOrigin_th.vp.z)) ;
  invPosGripOrigin_th.vp.y = -((invPosGripOrigin_th.vx.y)*(SDI_F->posGripOrigin_th.vp.x)+(invPosGripOrigin_th.vy.y)*(SDI_F->posGripOrigin_th.vp.y)+(invPosGripOrigin_th.vz.y)*(SDI_F->posGripOrigin_th.vp.z)) ;
  invPosGripOrigin_th.vp.z = -((invPosGripOrigin_th.vx.z)*(SDI_F->posGripOrigin_th.vp.x)+(invPosGripOrigin_th.vy.z)*(SDI_F->posGripOrigin_th.vp.y)+(invPosGripOrigin_th.vz.z)*(SDI_F->posGripOrigin_th.vp.z));

  Gb_th_produit(&invPosGripOrigin_th, &goalAbsolutPose_th, &relativeDisplacement_th);

  //  printf(" x_rel %f\n",relativeDisplacement_th.vp.x);
  //  printf(" y_rel %f\n",relativeDisplacement_th.vp.y);
  //  printf(" z_rel %f\n",relativeDisplacement_th.vp.z);
 
  /* INIT POSTERS */
  if (posterFind(QARM_POSITION_POSTER_NAME, &posterQarmPos) == ERROR) {
    PRINT_ERROR("Can not find POSTER POSITION");
    printf ("Can not find POSTER POSITION %s", QARM_POSITION_POSTER_NAME);
    *report =  S_xarm_POSTER_NOT_FOUND;
    return ETHER;
  }

 /* Read Poster Data and Status from QARM */
  if (xarmQARM_POSITION_STRPosterRead(posterQarmPos, &qarmPosition2)!=OK) {
    PRINT_ERROR("Can not read POSITION_POSTER");
    *report =  S_xarm_POSTER_NOT_READ;
    return ETHER;
  }

  /* Init xarm R6/Rt */
  Gb_th_produit(&(qarmPosition2.th06), xarmR6Rt, &thTool); 
  thTool.vp.z += XARM_BASE_HEIGHT;  
 
 /* Calcul of goal Matrix rotation in the base frame */
  Gb_th_produit(&thTool, &relativeDisplacement_th , &Next_thTool); 

  /* Calcul and normalization of goal quaternion in the base frame */ 
  if (RotMat2Quat(&Next_thTool, &GBQuatL) != OK) { 
    PRINT_ERROR("Pb - xarmDefGotoAbsolut_XYZ_Theta_PositionMain"); 
    return ETHER; 
  } 
  normaQ = GBQuatL.n * GBQuatL.n
    + GBQuatL.i * GBQuatL.i
    + GBQuatL.j * GBQuatL.j
    + GBQuatL.k * GBQuatL.k;
  normaQ=sqrt(normaQ);
  if(normaQ==0) {
    PRINT_ERROR("DefGotoAbsolut_XYZ_Theta_PositionMain - ERROR - Quaternion Norm = 0");
    *report =  S_xarm_WRONG_VALUE; 
    return ETHER;
  }
  GBQuatL.n = GBQuatL.n /normaQ;
  GBQuatL.i = GBQuatL.i /normaQ;
  GBQuatL.j = GBQuatL.j /normaQ;
  GBQuatL.k = GBQuatL.k /normaQ;


  if ( (Next_thTool.vp.x < XARM_X_MIN) || (Next_thTool.vp.x > XARM_X_MAX) || (Next_thTool.vp.y < XARM_Y_MIN) || (Next_thTool.vp.y > XARM_Y_MAX)) {
    printf("Next_thTool.vp.x %f\n",Next_thTool.vp.x);
    printf("Next_thTool.vp.y %f\n",Next_thTool.vp.y);
    printf("Next_thTool.vp.z %f\n",Next_thTool.vp.z);
    *report =  S_xarm_WRONG_ABSOLUT_POSE;
    return ETHER;
  }
  else {
    /* Update the goal position and orientation */
    /* Position To = Current Position */
    SDI_F->matTrajQ.p.x = Next_thTool.vp.x;
    SDI_F->matTrajQ.p.y = Next_thTool.vp.y;
    SDI_F->matTrajQ.p.z = Next_thTool.vp.z;
    /* Quaternion To = Current Orientation */
    SDI_F->matTrajQ.q.n = GBQuatL.n;
    SDI_F->matTrajQ.q.i = GBQuatL.i;
    SDI_F->matTrajQ.q.j = GBQuatL.j;
    SDI_F->matTrajQ.q.k = GBQuatL.k;
    
    printf("\n Goal Quaternion : \n");
    printf(" X %f\n",(SDI_F->matTrajQ.p.x));
    printf(" Y %f\n",(SDI_F->matTrajQ.p.y));
    printf(" Z %f\n",(SDI_F->matTrajQ.p.z));
    printf(" n %f\n",(SDI_F->matTrajQ.q.n));
    printf(" i %f\n",(SDI_F->matTrajQ.q.i));
    printf(" j %f\n",(SDI_F->matTrajQ.q.j));
    printf(" k %f\n",(SDI_F->matTrajQ.q.k));
    
    /* Init of Consign Type */
    SDI_F->consignType = XARM_QUATERNION_CONSIGN_TYPE;
  }
  return ETHER;
}


/* xarmDefGotoAbsolutXYZPositionMain  -  codel EXEC of DefGotoAbsolutXYZPosition
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmDefGotoAbsolutXYZPositionMain(TP_ABSOLUT_POSITION *goalAbsolutPosition, int *report)
{
 Gb_th invPosGripOrigin_th;
  Gb_th goalAbsolutPose_th;
  Gb_th relativeDisplacement_th;
  //  double normaQ;
  Gb_th thTool;
  Gb_th Next_thTool;
  TP_QUAT GBQuatL;
  POSTER_ID posterQarmPos;
  QARM_POSITION_STR qarmPosition2;
 
  goalAbsolutPose_th.vx.x = 1;
  goalAbsolutPose_th.vx.y = 0;
  goalAbsolutPose_th.vx.z = 0;

  goalAbsolutPose_th.vy.x = -goalAbsolutPose_th.vx.y;
  goalAbsolutPose_th.vy.y = goalAbsolutPose_th.vx.x;
  goalAbsolutPose_th.vy.z = 0;

  goalAbsolutPose_th.vz.x = 0;
  goalAbsolutPose_th.vz.y = 0;
  goalAbsolutPose_th.vz.z = 1;

  goalAbsolutPose_th.vp.x = goalAbsolutPosition->x;
  goalAbsolutPose_th.vp.y = goalAbsolutPosition->y;
  goalAbsolutPose_th.vp.z = goalAbsolutPosition->z;
  /* this is the position in the main frame of of the goalAbsolutPosition */

  //  printf(" \n");
  //  printf("posAbsGoal x %f\n",(goalAbsolutPose_th.vp.x));
  //  printf("posAbsGoal y %f\n",(goalAbsolutPose_th.vp.y));
  //  printf("posAbsGoal z %f\n",(goalAbsolutPose_th.vp.z));
 
  /* posGripOrigin_th Inversion */
  //invPosGripOrigin_th.vx.x = SDI_F->posGripOrigin_th.vx.x;
  //invPosGripOrigin_th.vx.y = SDI_F->posGripOrigin_th.vy.x;
  //invPosGripOrigin_th.vx.z = SDI_F->posGripOrigin_th.vz.x;   /* 0 */
  //
  //invPosGripOrigin_th.vy.x = SDI_F->posGripOrigin_th.vx.y;
  //invPosGripOrigin_th.vy.y = SDI_F->posGripOrigin_th.vy.y;
  //invPosGripOrigin_th.vy.z = SDI_F->posGripOrigin_th.vz.y;   /* 0 */
  //
  //invPosGripOrigin_th.vz.x = SDI_F->posGripOrigin_th.vx.z;   /* 0 */
  //invPosGripOrigin_th.vz.y = SDI_F->posGripOrigin_th.vy.z;   /* 0 */
  //invPosGripOrigin_th.vz.z = SDI_F->posGripOrigin_th.vz.z;   /* 1 */
  //
  //invPosGripOrigin_th.vp.x = -((invPosGripOrigin_th.vx.x)*(SDI_F->posGripOrigin_th.vp.x)+(invPosGripOrigin_th.vy.x)*(SDI_F->posGripOrigin_th.vp.y)+(invPosGripOrigin_th.vz.x)*(SDI_F->posGripOrigin_th.vp.z)) ;
  //invPosGripOrigin_th.vp.y = -((invPosGripOrigin_th.vx.y)*(SDI_F->posGripOrigin_th.vp.x)+(invPosGripOrigin_th.vy.y)*(SDI_F->posGripOrigin_th.vp.y)+(invPosGripOrigin_th.vz.y)*(SDI_F->posGripOrigin_th.vp.z)) ;
  //invPosGripOrigin_th.vp.z = -((invPosGripOrigin_th.vx.z)*(SDI_F->posGripOrigin_th.vp.x)+(invPosGripOrigin_th.vy.z)*(SDI_F->posGripOrigin_th.vp.y)+(invPosGripOrigin_th.vz.z)*(SDI_F->posGripOrigin_th.vp.z));

  Gb_th_inverse(&SDI_F->posGripOrigin_th, &invPosGripOrigin_th);

  Gb_th_produit(&invPosGripOrigin_th, &goalAbsolutPose_th, &relativeDisplacement_th);

    printf(" x_rel %f\n",relativeDisplacement_th.vp.x);
    printf(" y_rel %f\n",relativeDisplacement_th.vp.y);
    printf(" z_rel %f\n",relativeDisplacement_th.vp.z);
 
  /* INIT POSTERS */
  if (posterFind(QARM_POSITION_POSTER_NAME, &posterQarmPos) == ERROR) {
    PRINT_ERROR("Can not find POSTER POSITION");
    printf ("Can not find POSTER POSITION %s", QARM_POSITION_POSTER_NAME);
    *report =  S_xarm_POSTER_NOT_FOUND;
    return ETHER;
  }

 /* Read Poster Data and Status from QARM */
  if (xarmQARM_POSITION_STRPosterRead(posterQarmPos, &qarmPosition2)!=OK) {
    PRINT_ERROR("Can not read POSITION_POSTER");
    *report =  S_xarm_POSTER_NOT_READ;
    return ETHER;
  }

  /* Init xarm R6/Rt */
  Gb_th_produit(&(qarmPosition2.th06), xarmR6Rt, &thTool); 
  thTool.vp.z += XARM_BASE_HEIGHT;  
 
 /* Calcul of goal Matrix rotation in the base frame */
  Gb_th_produit(&thTool, &relativeDisplacement_th , &Next_thTool); 

  /* Calcul and normalization of goal quaternion in the base frame */ 
  if (RotMat2Quat(&Next_thTool, &GBQuatL) != OK) { 
    PRINT_ERROR("Pb - xarmDefGotoAbsolut_XYZ_Theta_PositionMain"); 
    return ETHER; 
  } 
//  normaQ = GBQuatL.n * GBQuatL.n
//    + GBQuatL.i * GBQuatL.i
//    + GBQuatL.j * GBQuatL.j
//    + GBQuatL.k * GBQuatL.k;
//  normaQ=sqrt(normaQ);
//  if(normaQ==0) {
//    PRINT_ERROR("DefGotoAbsolut_XYZ_Theta_PositionMain - ERROR - Quaternion Norm = 0");
//    *report =  S_xarm_WRONG_VALUE; 
//    return ETHER;
//  }
//  GBQuatL.n = GBQuatL.n /normaQ;
//  GBQuatL.i = GBQuatL.i /normaQ;
//  GBQuatL.j = GBQuatL.j /normaQ;
//  GBQuatL.k = GBQuatL.k /normaQ;


//  if ( (Next_thTool.vp.x < XARM_X_MIN) || (Next_thTool.vp.x > XARM_X_MAX) || (Next_thTool.vp.y < XARM_Y_MIN) || (Next_thTool.vp.y > XARM_Y_MAX)) {
//    printf("Next_thTool.vp.x %f\n",Next_thTool.vp.x);
//    printf("Next_thTool.vp.y %f\n",Next_thTool.vp.y);
//    printf("Next_thTool.vp.z %f\n",Next_thTool.vp.z);
//    *report =  S_xarm_WRONG_ABSOLUT_POSE;
//    return ETHER;
// }
//  else {
    /* Update the goal position and orientation */
    /* Position To = Current Position */
    SDI_F->matTrajQ.p.x = Next_thTool.vp.x;
    SDI_F->matTrajQ.p.y = Next_thTool.vp.y;
    SDI_F->matTrajQ.p.z = Next_thTool.vp.z;
    // TEST AKIN
    //SDI_F->matTrajQ.p.x = SDI_F->pose.p.x + relativeDisplacement_th.vp.z ;
    //SDI_F->matTrajQ.p.y = SDI_F->pose.p.y - relativeDisplacement_th.vp.x;
    //SDI_F->matTrajQ.p.z = SDI_F->pose.p.z - relativeDisplacement_th.vp.y;
    




/* Quaternion To = Current Orientation */
    SDI_F->matTrajQ.q.n = 0.5;
    SDI_F->matTrajQ.q.i = -0.5;
    SDI_F->matTrajQ.q.j = 0.5;
    SDI_F->matTrajQ.q.k = -0.5;
    
    printf("\n Goal Quaternion : \n");
    printf(" X %f\n",(SDI_F->matTrajQ.p.x));
    printf(" Y %f\n",(SDI_F->matTrajQ.p.y));
    printf(" Z %f\n",(SDI_F->matTrajQ.p.z));
    printf(" n %f\n",(SDI_F->matTrajQ.q.n));
    printf(" i %f\n",(SDI_F->matTrajQ.q.i));
    printf(" j %f\n",(SDI_F->matTrajQ.q.j));
    printf(" k %f\n",(SDI_F->matTrajQ.q.k));
    
    /* Init of Consign Type */
    SDI_F->consignType = XARM_QUATERNION_CONSIGN_TYPE;
    //  }
  return ETHER;
}




/*------------------------------------------------------------------------
 * DefGotoRobotXYZVAnglePosition
 *
 * Description: 
 *
 * Reports:      OK
 *              S_xarm_ERROR_DEF_POSITION
 */

/* xarmDefGotoRobotXYZVAnglePositionMain  -  codel EXEC of DefGotoRobotXYZVAnglePosition
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmDefGotoRobotXYZVAnglePositionMain(TP_ORIGIN_QUAT *goalPoseOriginQ, int *report)
{
  TP_QUAT QTO;
  Gb_th thTO_rob, thTO_arm, th_armRob, th_armRob_inv;
  double normaQ;
  TP_ORIGIN_QUAT QTO_1;
  Gb_quat QTO_arm, QTO_rob;

  QTO_1.x = goalPoseOriginQ->x;
  QTO_1.y = goalPoseOriginQ->y; 
  QTO_1.z = goalPoseOriginQ->z; 
  QTO_1.ex = goalPoseOriginQ->ex;
  QTO_1.ey = goalPoseOriginQ->ey;
  QTO_1.ez = goalPoseOriginQ->ez;
  QTO_1.angle = goalPoseOriginQ->angle;



  /* Quaternion To = Current Orientation */
  QTO.n = cos( (QTO_1.angle) / 2 );
  QTO.i = (QTO_1.ex) * sin((QTO_1.angle)/2);
  QTO.j = (QTO_1.ey) * sin((QTO_1.angle)/2);
  QTO.k = (QTO_1.ez) * sin((QTO_1.angle)/2);
  normaQ = QTO.n * QTO.n
    + QTO.i * QTO.i 
    + QTO.j * QTO.j
    + QTO.k * QTO.k;
  normaQ=sqrt(normaQ);
  if(normaQ==0) {
    PRINT_ERROR("DefGoto_XYZ_V_Angle_PositionCntrl - ERROR - Quaternion Goal Norm = 0");
    *report =  S_xarm_WRONG_VALUE; 
    return ERROR;
  }
  QTO.n = QTO.n/normaQ;
  QTO.i = QTO.i/normaQ;
  QTO.j = QTO.j/normaQ;
  QTO.k = QTO.k/normaQ;


  QTO_rob.x = QTO_1.x;
  QTO_rob.y = QTO_1.y;
  QTO_rob.z = QTO_1.z;
  QTO_rob.w = QTO.n;
  QTO_rob.vx = QTO.i;
  QTO_rob.vy = QTO.j;
  QTO_rob.vz = QTO.k;

  /* Gb quaternion to matrix rotation */
  Gb_quat_th(&QTO_rob, &thTO_rob);

  th_armRob.vx.x = 0;
  th_armRob.vx.y = 0;
  th_armRob.vx.z = 1;
  th_armRob.vy.x = -1;
  th_armRob.vy.y = 0;
  th_armRob.vy.z = 0;
  th_armRob.vz.x = 0;
  th_armRob.vz.y = -1;
  th_armRob.vz.z = 0;
  th_armRob.vp.x = 0.185;
  th_armRob.vp.y = -0.117;
  th_armRob.vp.z = 0.907;

  Gb_th_inverse(&th_armRob, &th_armRob_inv);
  Gb_th_produit(&th_armRob_inv, &thTO_rob, &thTO_arm);

  Gb_th_quat( &thTO_arm, &QTO_arm);

  /* Set the goal pose */
  SDI_F->matTrajQ.p.x = QTO_arm.x;
  SDI_F->matTrajQ.p.y = QTO_arm.y;
  SDI_F->matTrajQ.p.z = QTO_arm.z;
  SDI_F->matTrajQ.q.n = QTO_arm.w;
  SDI_F->matTrajQ.q.i = QTO_arm.vx;
  SDI_F->matTrajQ.q.j = QTO_arm.vy;
  SDI_F->matTrajQ.q.k = QTO_arm.vz;

  printf("goto_pos in arm frame: x %f y %f z %f\n",  SDI_F->matTrajQ.p.x,   SDI_F->matTrajQ.p.y,   SDI_F->matTrajQ.p.z);

  /* Init of Consign Type */
  SDI_F->consignType = XARM_QUATERNION_CONSIGN_TYPE;

  return ETHER;
}



static STATUS xarmPosPosterRead()
{
  static POM_POS      	pomPos;		/* current pos */
  static POM_ME_POS    	pomMEPos;	/* current pos */
  double                x, y, theta;
  Gb_th                 posBaseRob_th;
  Gb_th                 posGrip_th;
  Gb_th                 posArmBaseRob_th;

  if (posterIsPOM_POS) {
    if (xarmPOM_POSPosterRead(posPosterId, &pomPos) == OK) {
      x = pomPos.mainToOrigin.euler.x;
      y = pomPos.mainToOrigin.euler.y;
      theta = pomPos.mainToOrigin.euler.yaw;
      //x = pomPos.mainToBase.euler.x;
      //y = pomPos.mainToBase.euler.y;
      //theta = pomPos.mainToBase.euler.yaw;
      /*      *date = pomPos.date;*/
    }
    else {
      return FALSE;
    }
  }
  else {
    if (xarmPOM_ME_POSPosterRead(posPosterId, &pomMEPos) == OK) {
      x = pomMEPos.main.euler.x;
      y = pomMEPos.main.euler.y;
      theta = pomMEPos.main.euler.yaw;
    } else {
      return FALSE;
    }
  }
  printf("pomPos x : %f\n",x);
  printf("pomPos y : %f\n",y);
  printf("pomPos theta : %f\n",theta);

  posBaseRob_th.vx.x = cos(theta);
  posBaseRob_th.vx.y = sin(theta);
  posBaseRob_th.vx.z = 0;
  posBaseRob_th.vy.x = -posBaseRob_th.vx.y;
  posBaseRob_th.vy.y = posBaseRob_th.vx.x;
  posBaseRob_th.vy.z = 0;
  posBaseRob_th.vz.x = 0;
  posBaseRob_th.vz.y = 0;
  posBaseRob_th.vz.z = 1;
  posBaseRob_th.vp.x = x;
  posBaseRob_th.vp.y = y;
  posBaseRob_th.vp.z = 0;
  /* this is the position in the frame of the base of the arm */

//  posArmBaseRob_th.vx.x = 1;
//  posArmBaseRob_th.vx.y = 0;
//  posArmBaseRob_th.vx.z = 0;
//  posArmBaseRob_th.vy.x = 0;
//  posArmBaseRob_th.vy.y = 1;
//  posArmBaseRob_th.vy.z = 0;
//  posArmBaseRob_th.vz.x = 0;
//  posArmBaseRob_th.vz.y = 0;
//  posArmBaseRob_th.vz.z = 1;




  /* Get transformation to 'main' frame */
  baseArmPosT3d = t3dBryanIdentity;
  baseArmPosT3d.euler.euler[0] = baseArmPos.sensorToMain.euler.yaw;
  baseArmPosT3d.euler.euler[1] = baseArmPos.sensorToMain.euler.pitch;
  baseArmPosT3d.euler.euler[2] = baseArmPos.sensorToMain.euler.roll;
  baseArmPosT3d.euler.euler[3] = baseArmPos.sensorToMain.euler.x;
  baseArmPosT3d.euler.euler[4] = baseArmPos.sensorToMain.euler.y;
  baseArmPosT3d.euler.euler[5] = baseArmPos.sensorToMain.euler.z;
  t3dConvertTo(T3D_MATRIX, &baseArmPosT3d);

  posArmBaseRob_th.vx.x =  baseArmPosT3d.matrix.matrix[0];
  posArmBaseRob_th.vy.x =  baseArmPosT3d.matrix.matrix[1];
  posArmBaseRob_th.vz.x =  baseArmPosT3d.matrix.matrix[2];

  posArmBaseRob_th.vx.y =  baseArmPosT3d.matrix.matrix[4];
  posArmBaseRob_th.vy.y =  baseArmPosT3d.matrix.matrix[5];
  posArmBaseRob_th.vz.y =  baseArmPosT3d.matrix.matrix[6];


  posArmBaseRob_th.vx.z =  baseArmPosT3d.matrix.matrix[8];
  posArmBaseRob_th.vy.z =  baseArmPosT3d.matrix.matrix[9];
  posArmBaseRob_th.vz.z =  baseArmPosT3d.matrix.matrix[10];

  posArmBaseRob_th.vp.x =  baseArmPosT3d.matrix.matrix[3];
  posArmBaseRob_th.vp.y =  baseArmPosT3d.matrix.matrix[7];
  posArmBaseRob_th.vp.z =  baseArmPosT3d.matrix.matrix[11] ; /*- XARM_BASE_HEIGHT*/

  printf("matrix of the arm in the robot frame\n");
  printf(" %3.3f %3.3f %3.3f %3.3f\n",  posArmBaseRob_th.vx.x,   posArmBaseRob_th.vy.x,  posArmBaseRob_th.vz.x,   posArmBaseRob_th.vp.x);  
  printf(" %3.3f %3.3f %3.3f %3.3f\n",  posArmBaseRob_th.vx.y,   posArmBaseRob_th.vy.y,  posArmBaseRob_th.vz.y,   posArmBaseRob_th.vp.y);  
  printf(" %3.3f %3.3f %3.3f %3.3f\n",  posArmBaseRob_th.vx.z,   posArmBaseRob_th.vy.z,  posArmBaseRob_th.vz.z,   posArmBaseRob_th.vp.z);  

  /* this is the position in the frame of the base of the arm */
  
  Gb_th_produit(&posBaseRob_th, &posArmBaseRob_th, &(SDI_F->posArmOrigin_th));

  /*   printf("\nposArmBaseRob x : %f\n",posArmBaseRob_th.vp.x); */
  /*   printf("posArmBaseRob y : %f\n",posArmBaseRob_th.vp.y); */
  /*   printf("posArmBaseRob z : %f\n",posArmBaseRob_th.vp.z); */
  
  /*   printf("\nposArmOrigin_th x : %f\n",(SDI_F->posArmOrigin_th.vp.x)); */
  /*   printf("posArmOrigin_th y : %f\n",(SDI_F->posArmOrigin_th.vp.y)); */
  /*   printf("posArmOrigin_th z : %f\n",(SDI_F->posArmOrigin_th.vp.z)); */

  posGrip_th.vx.x = SDI_F->posGrip.nx;
  posGrip_th.vx.y = SDI_F->posGrip.ny;
  posGrip_th.vx.z = SDI_F->posGrip.nz;
  posGrip_th.vy.x = SDI_F->posGrip.ox;
  posGrip_th.vy.y = SDI_F->posGrip.oy;
  posGrip_th.vy.z = SDI_F->posGrip.oz;
  posGrip_th.vz.x = SDI_F->posGrip.ax;
  posGrip_th.vz.y = SDI_F->posGrip.ay;
  posGrip_th.vz.z = SDI_F->posGrip.az;
  posGrip_th.vp.x = SDI_F->posGrip.px;
  posGrip_th.vp.y = SDI_F->posGrip.py;
  posGrip_th.vp.z = SDI_F->posGrip.pz; 

  /*   printf("\nposGripNacho x : %f\n",posGrip_th.vp.x); */
  /*   printf("posGripNacho y : %f\n",posGrip_th.vp.y); */
  /*   printf("posGripNacho z : %f\n",posGrip_th.vp.z); */

  Gb_th_produit(&(SDI_F->posArmOrigin_th), &posGrip_th, &(SDI_F->posGripOrigin_th));

  printf("\nxarmPosterRead OK \n");
  return TRUE;
}

/*------------------------------------------------------------------------
 * SetRobotPositionPoster
 *
 * Description: 
 *
 * Reports:      OK
 *              S_xarm_POSTER_NOT_FOUND
 */

/* xarmSetRobotPositionPosterMain  -  codel EXEC of SetRobotPositionPoster
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmSetRobotPositionPosterMain(XARM_STRING *in_posPosterName, int *report)
{
    POSTER_ID baseArmPosterId;
  /**
   ** Look for the position poster
   **/
  /* look for a poster of type POM_POS or POM_ME_POS */
  posterIsPOM_POS = TRUE;
  if (xarmPOM_POSPosterFind(in_posPosterName->str, &posPosterId) == ERROR) {
    /* poster found but length not seem correct : try POM_ME_POS */
    if (posPosterId) {
      if (xarmPOM_ME_POSPosterFind(in_posPosterName->str, &posPosterId) == OK) {
	posterIsPOM_POS = FALSE;
      }
      /* no correct poster */
      else {
	if (posPosterId) 
	  printf ("xarm: poster %s of bad type\n", in_posPosterName->str);
	else
	  printf ("xarm: can't find poster %s\n", in_posPosterName->str);
	posPosterId = 0;
	*report = S_xarm_POSTER_NOT_FOUND;
      } /* no poster found */
    } /* POM_ME_POS */
  } /* POM_POS */
  if (posPosterId) {
    printf ("xarm: robot pos from %s poster\n", in_posPosterName->str);
  }
  else {
    printf ("xarm: fail to find pos poster\n");
    return ETHER;
  }

  if (xarmPOM_SENSOR_POSPosterFind("pomBaseArmFramePos", &baseArmPosterId)
      == ERROR) {
    *report = S_xarm_POSTER_NOT_FOUND;
    printf("xarmPOM_SENSOR_POSPosterFind pomBaseArmFramePos failed\n");
    return ETHER;
  }
  if (xarmPOM_SENSOR_POSPosterRead(baseArmPosterId, &baseArmPos) == ERROR) {
    *report = S_xarm_POSTER_NOT_FOUND;
    printf("xarmPOM_SENSOR_POSPosterRead failed\n");
  }

  xarmPosPosterRead();

  printf("posAbsGrip x %f\n",(SDI_F->posGripOrigin_th.vp.x));
  printf("posAbsGrip y %f\n",(SDI_F->posGripOrigin_th.vp.y));
  printf("posAbsGrip z %f\n",(SDI_F->posGripOrigin_th.vp.z));

  return ETHER;
}



/*------------------------------------------------------------------------
 * VerifyStateOfSegmentTracking
 *
 * Description: 
 *
 * Reports:      OK
 *              S_xarm_xarmBadTrackingSegmentMode
 */

/* xarmVerifyStateOfSegmentTrackingMain  -  codel EXEC of VerifyStateOfSegmentTracking
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmVerifyStateOfSegmentTrackingMain(int *(out_currentTimeOfTrajectory), int *report)
{
  if (SDI_F->xarmTrackSegment == XARM_TRACK_ON) {
    if (SDI_F->printData == 1) {
      printf("SDI_F->currentTimeOfTrajectory %d\n",SDI_F->currentTimeOfTrajectory );
    }
    return EXEC;
  }
  else {
    *(out_currentTimeOfTrajectory) = SDI_F->currentTimeOfTrajectory;
    SDI_F->currentTimeOfTrajectory = 0;
    return ETHER;
  }
}



/*------------------------------------------------------------------------
 * JoystickOn
 *
 * Description: 
 *
 * Reports:      OK
 *              S_xarm_JOYSTICK_ERROR
 *              S_xarm_POSTER_NOT_FOUND
 *              S_xarm_POSTER_NOT_READ
 */

/* xarmJoystickOnStart  -  codel START of JoystickOn
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmJoystickOnStart(int *report)
{
  int joyresult;
 // /* INIT POSTERS */
 //
 // if ( xarmJOYSTICK_DATAPosterFind("joystickJoystick",  &posterJoystick) == ERROR) {
 //   PRINT_ERROR("Can not find POSTER JOYSTICK");
 //   *report =  S_xarm_POSTER_NOT_FOUND;
 //   return ETHER;
 // }

  if(joystickInit(JOYSTICK_DEVICE_PATH, &(SDI_F->joystick))!=OK) {
    PRINT_ERROR("joystickInit\n");
    *report = S_xarm_JOYSTICK_ERROR;
    return ETHER;
    
  }
  Jerks.sel = 1;

  joyresult = joystickPoll(&(SDI_F->joystick));
  if(0 > joyresult){
    PRINT_ERROR("joystickPoll()\n");
    *report = S_xarm_JOYSTICK_ERROR;
    return ETHER;
  }

  cptVjoy = 0;
  printf("XARM JOYSTICK_ON START OK\n");
  return EXEC;
}

/* xarmJoystickOnMain  -  codel EXEC of JoystickOn
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmJoystickOnMain(int *report)
{
  //   double V0;
  double criticalLength;
  int joyresult;
//
//  /* Read Poster Data and Status from Joystick */
//  if(xarmJOYSTICK_DATAPosterRead( posterJoystick, &(SDI_F->joystick))!=OK) {
//    PRINT_ERROR("Can not read JOYSTICK_POSTER");
//    *report =  S_xarm_POSTER_NOT_READ;
//    return ETHER;
//  }

  joyresult = joystickPoll(&(SDI_F->joystick));
  if(0 > joyresult){
    PRINT_ERROR("joystickPoll()\n");
    *report = S_xarm_JOYSTICK_ERROR;
  }

  getJoystickSpeeds(&VxRef, &VyRef, &VzRef, &WxRef, &WyRef, &WzRef);

  if (ABS(VxRef) < 0.001) { VxRef = 0.0;}
  if (ABS(VyRef) < 0.001) { VyRef = 0.0;}
  if (ABS(VzRef) < 0.001) { VzRef = 0.0;}
  if (ABS(WxRef) < 0.001) { WxRef = 0.0;}
  if (ABS(WyRef) < 0.001) { WyRef = 0.0;}
  if (ABS(WzRef) < 0.001) { WzRef = 0.0;}

  /* Emergency Stop */
  if (SDI_F->joystick.buttons[4]) {
    SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
    SDI_F->mode = XARM_STOP;
    SDI_F->consignType = XARM_NOT_SELECTED_CONSIGN_TYPE;    
    return ETHER;
  }
   
  /* Check for user abort (button 4) */  
  if (SDI_F->joystick.buttons[3]) {
    return ETHER;
  }

  if (SDI_F->joystick.buttons[5]) { 
    //  printf("VelX %f  VelY %f  VelZ %f\n",VxRef, VyRef, VzRef);

    // Not needed anymore with the coldman collision checker

    //if (SDI_F->pose.p.x > XARM_X_MAX_JOY && (VxRef > 0) ) {VxRef = 0.0;}
    //if (SDI_F->pose.p.x < XARM_X_MIN_JOY && (VxRef < 0) ) {VxRef = 0.0;}
    //if (SDI_F->pose.p.y > XARM_Y_MAX_JOY && (VyRef > 0) ) {VyRef = 0.0;}
    //if (SDI_F->pose.p.y < XARM_Y_MIN_JOY && (VyRef < 0) ) {VyRef = 0.0;} 
    //if (SDI_F->pose.p.z > XARM_Z_MAX_JOY && (VzRef > 0) ) {VzRef = 0.0;}
    //if (SDI_F->pose.p.z < XARM_Z_MIN_JOY && (VzRef < 0) ) {VzRef = 0.0;}    
  }
  else {
    /* Sweet Stop */
    SDI_F->SweetStop = 1;
    return EXEC;
  }

  // VjoyRef[cptVjoy] = VxRef;
  // cptVjoy ++;

  /* We do nothinf if xarm is sweet stopping */
  if (SDI_F->SweetStop == 1) {
    return EXEC;
  }

  /* Initialize queues */
  queueInit(&(SDI_F->Seg[0]), "SegX");
  queueInit(&(SDI_F->Seg[1]), "SegY");
  queueInit(&(SDI_F->Seg[2]), "SegZ");
  queueInit(&(SDI_F->Seg[3]), "SegQN");
  queueInit(&(SDI_F->Seg[4]), "SegQI");
  queueInit(&(SDI_F->Seg[5]), "SegQJ");
  queueInit(&(SDI_F->Seg[6]), "SegQK");

  /**************** PX ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.x.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.x.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(VxRef * 1000000))/1000000;
  
  if ( ABS( SDI_F->motionConditions.x.v - FC.v) > JOYSTICK_SENSIBILITY) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.linear, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[0]), &Dir[0])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.x.x;
    GD =  FC.x * Dir[0];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC, Dir[0], AxisTimes[0], &FC, &Acc[0], &Vel[0], &Pos[0], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times PX");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[0]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[0]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[0]);
    queueAddDetails(&(SDI_F->Seg[0]), 2, TNE.Taca, Jerks.J1, Acc[0].Tjpa, Vel[0].Tjpa, Pos[0].Tjpa,  Dir[0]);
    queueAddDetails(&(SDI_F->Seg[0]), 3, TNE.Tjna, Jerks.J1, Acc[0].Taca, Vel[0].Taca, Pos[0].Taca,  Dir[0]);

    queueAddDetails(&(SDI_F->Seg[0]), 4, TNE.Tvc,  Jerks.J1, Acc[0].Tjna, Vel[0].Tjna, Pos[0].Tjna,  Dir[0]);
    //queueAddDetails(&(SDI_F->Seg[0]), 4, 1.0,  Jerks.J1, Acc[0].Tjna, Vel[0].Tjna, Pos[0].Tjna,  Dir[0]);

    queueAddDetails(&(SDI_F->Seg[0]), 5, TNE.Tjnb, Jerks.J1, Acc[0].Tvc,  Vel[0].Tvc,  Pos[0].Tvc,   Dir[0]);
    queueAddDetails(&(SDI_F->Seg[0]), 6, TNE.Tacb, Jerks.J1, Acc[0].Tjnb, Vel[0].Tjnb, Pos[0].Tjnb,  Dir[0]);
    queueAddDetails(&(SDI_F->Seg[0]), 7, TNE.Tjpb, Jerks.J1, Acc[0].Tacb, Vel[0].Tacb, Pos[0].Tacb,  Dir[0]);

    queueAddDetails(&(SDI_F->Seg[0]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.x.x;
    queueAddDetails(&(SDI_F->Seg[0]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
  }

  /**************** PY ****************/    
  /* Initial Conditions */                                     
  IC.a =  (double)((int)(SDI_F->motionConditions.y.a * 1000000))/1000000; 
  IC.v =  (double)((int)(SDI_F->motionConditions.y.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(VyRef * 1000000))/1000000;

  if ( ABS( SDI_F->motionConditions.y.v - FC.v) > JOYSTICK_SENSIBILITY) {

    auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
    //   V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.linear, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
    FC.x = criticalLength;     
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[1]), &Dir[1])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.y.x;
    GD =  FC.x* Dir[1];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC,  Dir[1], AxisTimes[1], &FC, &Acc[1], &Vel[1], &Pos[1], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times PY");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[1]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);

    queueAddDetails(&(SDI_F->Seg[1]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[1]);
    queueAddDetails(&(SDI_F->Seg[1]), 2, TNE.Taca, Jerks.J1, Acc[1].Tjpa, Vel[1].Tjpa, Pos[1].Tjpa,  Dir[1]);
    queueAddDetails(&(SDI_F->Seg[1]), 3, TNE.Tjna, Jerks.J1, Acc[1].Taca, Vel[1].Taca, Pos[1].Taca,  Dir[1]);

    queueAddDetails(&(SDI_F->Seg[1]), 4, TNE.Tvc,  Jerks.J1, Acc[1].Tjna, Vel[1].Tjna, Pos[1].Tjna,  Dir[1]);
    // queueAddDetails(&(SDI_F->Seg[1]), 4, 1.0,  Jerks.J1, Acc[1].Tjna, Vel[1].Tjna, Pos[1].Tjna,  Dir[1]);

    queueAddDetails(&(SDI_F->Seg[1]), 5, TNE.Tjnb, Jerks.J1, Acc[1].Tvc,  Vel[1].Tvc,  Pos[1].Tvc,   Dir[1]);
    queueAddDetails(&(SDI_F->Seg[1]), 6, TNE.Tacb, Jerks.J1, Acc[1].Tjnb, Vel[1].Tjnb, Pos[1].Tjnb,  Dir[1]);
    queueAddDetails(&(SDI_F->Seg[1]), 7, TNE.Tjpb, Jerks.J1, Acc[1].Tacb, Vel[1].Tacb, Pos[1].Tacb,  Dir[1]);

    queueAddDetails(&(SDI_F->Seg[1]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
      
  } else {
    IC.x = SDI_F->motionConditions.y.x;
    queueAddDetails(&(SDI_F->Seg[1]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
  }
  /**************** PZ ****************/
  /* Initial Conditions */                                     
  IC.a =  (double)((int)(SDI_F->motionConditions.z.a * 1000000))/1000000; 
  IC.v =  (double)((int)(SDI_F->motionConditions.z.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v = (double)((int)(VzRef * 1000000))/1000000;

  if ( ABS( SDI_F->motionConditions.z.v - FC.v) > JOYSTICK_SENSIBILITY) {

    auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
    //   V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.linear, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
    FC.x = criticalLength;   
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[2]), &Dir[2])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.z.x;
    GD =  FC.x* Dir[2];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC,  Dir[2], AxisTimes[2], &FC, &Acc[2], &Vel[2], &Pos[2], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times PZ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[2]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);

    queueAddDetails(&(SDI_F->Seg[2]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[2]);
    queueAddDetails(&(SDI_F->Seg[2]), 2, TNE.Taca, Jerks.J1, Acc[2].Tjpa, Vel[2].Tjpa, Pos[2].Tjpa,  Dir[2]);
    queueAddDetails(&(SDI_F->Seg[2]), 3, TNE.Tjna, Jerks.J1, Acc[2].Taca, Vel[2].Taca, Pos[2].Taca,  Dir[2]);

    queueAddDetails(&(SDI_F->Seg[2]), 4, TNE.Tvc,  Jerks.J1, Acc[2].Tjna, Vel[2].Tjna, Pos[2].Tjna,  Dir[2]);
    //queueAddDetails(&(SDI_F->Seg[2]), 4, 1.0,  Jerks.J1, Acc[2].Tjna, Vel[2].Tjna, Pos[2].Tjna,  Dir[2]);

    queueAddDetails(&(SDI_F->Seg[2]), 5, TNE.Tjnb, Jerks.J1, Acc[2].Tvc,  Vel[2].Tvc,  Pos[2].Tvc,   Dir[2]);
    queueAddDetails(&(SDI_F->Seg[2]), 6, TNE.Tacb, Jerks.J1, Acc[2].Tjnb, Vel[2].Tjnb, Pos[2].Tjnb,  Dir[2]);
    queueAddDetails(&(SDI_F->Seg[2]), 7, TNE.Tjpb, Jerks.J1, Acc[2].Tacb, Vel[2].Tacb, Pos[2].Tacb,  Dir[2]);

    queueAddDetails(&(SDI_F->Seg[2]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
      
  } else {
    IC.x = SDI_F->motionConditions.z.x;
    queueAddDetails(&(SDI_F->Seg[2]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
  }

  TP_QUAT quat, dquat;

  quat.n = SDI_F->motionConditions.n.x;
  quat.i = SDI_F->motionConditions.i.x;
  quat.j = SDI_F->motionConditions.j.x;
  quat.k = SDI_F->motionConditions.k.x;

  calculOfQuaternionDerivative(WxRef, WyRef, WzRef, &quat, &dquat);

  // printf("dquat       n %f i %f j %f k %f\n",dquat.n,dquat.i,dquat.j,dquat.k);
  // printf("dquatMotion n %f i %f j %f k %f\n",SDI_F->motionConditions.n.v, SDI_F->motionConditions.i.v, SDI_F->motionConditions.j.v, SDI_F->motionConditions.k.v);
  
  
  // quaternionNormalize(&dquat);
  
 
  Jerks.J1  = SDI_F->limitsGoto.angular.maxJerk; 



  /**************** QN ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.n.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.n.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(dquat.n * 1000000))/1000000;
  
  if ( ABS(SDI_F->motionConditions.n.a) > 0.001 || ABS( SDI_F->motionConditions.n.v - FC.v) > JOYSTICK_SENSIBILITY_ROT) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.angular, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[3]), &Dir[3])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.n.x;
    GD =  FC.x * Dir[3];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[3], AxisTimes[3], &FC, &Acc[3], &Vel[3], &Pos[3], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times QN");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[3]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[3]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[3]);
    queueAddDetails(&(SDI_F->Seg[3]), 2, TNE.Taca, Jerks.J1, Acc[3].Tjpa, Vel[3].Tjpa, Pos[3].Tjpa,  Dir[3]);
    queueAddDetails(&(SDI_F->Seg[3]), 3, TNE.Tjna, Jerks.J1, Acc[3].Taca, Vel[3].Taca, Pos[3].Taca,  Dir[3]);

    queueAddDetails(&(SDI_F->Seg[3]), 4, TNE.Tvc,  Jerks.J1, Acc[3].Tjna, Vel[3].Tjna, Pos[3].Tjna,  Dir[3]);
    //queueAddDetails(&(SDI_F->Seg[3]), 4, 1.0,  Jerks.J1, Acc[3].Tjna, Vel[3].Tjna, Pos[3].Tjna,  Dir[3]);

    queueAddDetails(&(SDI_F->Seg[3]), 5, TNE.Tjnb, Jerks.J1, Acc[3].Tvc,  Vel[3].Tvc,  Pos[3].Tvc,   Dir[3]);
    queueAddDetails(&(SDI_F->Seg[3]), 6, TNE.Tacb, Jerks.J1, Acc[3].Tjnb, Vel[3].Tjnb, Pos[3].Tjnb,  Dir[3]);
    queueAddDetails(&(SDI_F->Seg[3]), 7, TNE.Tjpb, Jerks.J1, Acc[3].Tacb, Vel[3].Tacb, Pos[3].Tacb,  Dir[3]);

    queueAddDetails(&(SDI_F->Seg[3]), 4,  2,  Jerks.J1, FC.a, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.n.x;
    queueAddDetails(&(SDI_F->Seg[3]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
    //  queueAddDetails(&(SDI_F->Seg[3]), 4, NB_TICK,  Jerks.J1, 0, 0, SDI_F->motionConditions.n.x, 1);
  }

  /**************** QI ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.i.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.i.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(dquat.i * 1000000))/1000000;
  
  if ( ABS(SDI_F->motionConditions.i.a) > 0.001 || ABS( SDI_F->motionConditions.i.v - FC.v) > JOYSTICK_SENSIBILITY_ROT) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.angular, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[4]), &Dir[4])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.i.x;
    GD =  FC.x * Dir[4];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[4], AxisTimes[4], &FC, &Acc[4], &Vel[4], &Pos[4], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times QI");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[4]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[4]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[4]);
    queueAddDetails(&(SDI_F->Seg[4]), 2, TNE.Taca, Jerks.J1, Acc[4].Tjpa, Vel[4].Tjpa, Pos[4].Tjpa,  Dir[4]);
    queueAddDetails(&(SDI_F->Seg[4]), 3, TNE.Tjna, Jerks.J1, Acc[4].Taca, Vel[4].Taca, Pos[4].Taca,  Dir[4]);

    queueAddDetails(&(SDI_F->Seg[4]), 4, TNE.Tvc,  Jerks.J1, Acc[4].Tjna, Vel[4].Tjna, Pos[4].Tjna,  Dir[4]);
    //queueAddDetails(&(SDI_F->Seg[4]), 4, 1.0,  Jerks.J1, Acc[4].Tjna, Vel[4].Tjna, Pos[4].Tjna,  Dir[4]);

    queueAddDetails(&(SDI_F->Seg[4]), 5, TNE.Tjnb, Jerks.J1, Acc[4].Tvc,  Vel[4].Tvc,  Pos[4].Tvc,   Dir[4]);
    queueAddDetails(&(SDI_F->Seg[4]), 6, TNE.Tacb, Jerks.J1, Acc[4].Tjnb, Vel[4].Tjnb, Pos[4].Tjnb,  Dir[4]);
    queueAddDetails(&(SDI_F->Seg[4]), 7, TNE.Tjpb, Jerks.J1, Acc[4].Tacb, Vel[4].Tacb, Pos[4].Tacb,  Dir[4]);

    queueAddDetails(&(SDI_F->Seg[4]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.i.x;
    queueAddDetails(&(SDI_F->Seg[4]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
    //  queueAddDetails(&(SDI_F->Seg[4]), 4, NB_TICK,  Jerks.J1, 0, 0, SDI_F->motionConditions.i.x, 1);
  }

  /**************** QJ ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.j.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.j.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(dquat.j * 1000000))/1000000;
  
  if ( ABS(SDI_F->motionConditions.j.a) > 0.001 || ABS( SDI_F->motionConditions.j.v - FC.v) > JOYSTICK_SENSIBILITY_ROT) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.angular, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[5]), &Dir[5])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.j.x;
    GD =  FC.x * Dir[5];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[5], AxisTimes[5], &FC, &Acc[5], &Vel[5], &Pos[5], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times QJ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[5]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[5]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[5]);
    queueAddDetails(&(SDI_F->Seg[5]), 2, TNE.Taca, Jerks.J1, Acc[5].Tjpa, Vel[5].Tjpa, Pos[5].Tjpa,  Dir[5]);
    queueAddDetails(&(SDI_F->Seg[5]), 3, TNE.Tjna, Jerks.J1, Acc[5].Taca, Vel[5].Taca, Pos[5].Taca,  Dir[5]);

    queueAddDetails(&(SDI_F->Seg[5]), 4, TNE.Tvc,  Jerks.J1, Acc[5].Tjna, Vel[5].Tjna, Pos[5].Tjna,  Dir[5]);
    //queueAddDetails(&(SDI_F->Seg[5]), 4, 1.0,  Jerks.J1, Acc[5].Tjna, Vel[5].Tjna, Pos[5].Tjna,  Dir[5]);

    queueAddDetails(&(SDI_F->Seg[5]), 5, TNE.Tjnb, Jerks.J1, Acc[5].Tvc,  Vel[5].Tvc,  Pos[5].Tvc,   Dir[5]);
    queueAddDetails(&(SDI_F->Seg[5]), 6, TNE.Tacb, Jerks.J1, Acc[5].Tjnb, Vel[5].Tjnb, Pos[5].Tjnb,  Dir[5]);
    queueAddDetails(&(SDI_F->Seg[5]), 7, TNE.Tjpb, Jerks.J1, Acc[5].Tacb, Vel[5].Tacb, Pos[5].Tacb,  Dir[5]);

    queueAddDetails(&(SDI_F->Seg[5]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.j.x;
    queueAddDetails(&(SDI_F->Seg[5]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
    //  queueAddDetails(&(SDI_F->Seg[5]), 4, NB_TICK,  Jerks.J1, 0, 0, SDI_F->motionConditions.j.x, 1);
  }

  /**************** QK ****************/  
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.k.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.k.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(dquat.k * 1000000))/1000000;
  
  if ( ABS(SDI_F->motionConditions.k.a) > 0.001 || ABS( SDI_F->motionConditions.k.v - FC.v) > JOYSTICK_SENSIBILITY_ROT) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.angular, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[6]), &Dir[6])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.k.x;
    GD =  FC.x * Dir[6];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[6], AxisTimes[6], &FC, &Acc[6], &Vel[6], &Pos[6], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times QK");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[6]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[6]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[6]);
    queueAddDetails(&(SDI_F->Seg[6]), 2, TNE.Taca, Jerks.J1, Acc[6].Tjpa, Vel[6].Tjpa, Pos[6].Tjpa,  Dir[6]);
    queueAddDetails(&(SDI_F->Seg[6]), 3, TNE.Tjna, Jerks.J1, Acc[6].Taca, Vel[6].Taca, Pos[6].Taca,  Dir[6]);

    queueAddDetails(&(SDI_F->Seg[6]), 4, TNE.Tvc,  Jerks.J1, Acc[6].Tjna, Vel[6].Tjna, Pos[6].Tjna,  Dir[6]);
    //queueAddDetails(&(SDI_F->Seg[6]), 4, 1.0,  Jerks.J1, Acc[6].Tjna, Vel[6].Tjna, Pos[6].Tjna,  Dir[6]);

    queueAddDetails(&(SDI_F->Seg[6]), 5, TNE.Tjnb, Jerks.J1, Acc[6].Tvc,  Vel[6].Tvc,  Pos[6].Tvc,   Dir[6]);
    queueAddDetails(&(SDI_F->Seg[6]), 6, TNE.Tacb, Jerks.J1, Acc[6].Tjnb, Vel[6].Tjnb, Pos[6].Tjnb,  Dir[6]);
    queueAddDetails(&(SDI_F->Seg[6]), 7, TNE.Tjpb, Jerks.J1, Acc[6].Tacb, Vel[6].Tacb, Pos[6].Tacb,  Dir[6]);

    queueAddDetails(&(SDI_F->Seg[6]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.k.x;
    queueAddDetails(&(SDI_F->Seg[6]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
    //  queueAddDetails(&(SDI_F->Seg[6]), 4, NB_TICK,  Jerks.J1, 0, 0, SDI_F->motionConditions.k.x, 1);
  }


  SDI_F->xarmTrackSegment = XARM_TRACK_ON;  

  return EXEC;
}

/* xarmJoystickOnEnd  -  codel END of JoystickOn
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmJoystickOnEnd(int *report)
{
  /* ... add your code here ... */
  return ETHER;
}

/* xarmJoystickOnInter  -  codel INTER of JoystickOn
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmJoystickOnInter(int *report)
{
  /* ... add your code here ... */
  return ETHER;
}



/* xarmMouse3dOnStart  -  codel START of Mouse3dOn
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmMouse3dOnStart(int *report)
{
  /* INIT POSTERS */
  if ( xarmGENMANIP_MOUSE3D_STRPosterFind("mouse3dMeasures",  &posterMouse3d) == ERROR) {
   PRINT_ERROR("Can not find POSTER MOUSE3D");
   *report =  S_xarm_POSTER_NOT_FOUND;
   return ETHER;
  }
  /* Read Poster Data and Status from Joystick */
  if(xarmGENMANIP_MOUSE3D_STRPosterRead( posterMouse3d, &(SDI_F->mouse3dData))!=OK) {
    PRINT_ERROR("Can not read MOUSE3D_POSTER");
    *report =  S_xarm_POSTER_NOT_READ;
    return ETHER;
  }
  Jerks.sel = 1;
  cptVjoy = 0;
  printf("XARM MOUSE3D_ON START OK\n");
  return EXEC;
}


/* xarmMouse3dOnMain  -  codel EXEC of Mouse3dOn
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmMouse3dOnMain(int *report)
{
  
  double criticalLength;

  /* Read Poster Data and Status from Joystick */
  if(xarmGENMANIP_MOUSE3D_STRPosterRead( posterMouse3d, &(SDI_F->mouse3dData))!=OK) {
    PRINT_ERROR("Can not read MOUSE3D_POSTER");
    *report =  S_xarm_POSTER_NOT_READ;
    return ETHER;
  }
  

  getMouse3dSpeeds(&VxRef, &VyRef, &VzRef, &WxRef, &WyRef, &WzRef);
  // printf("VxRef %f\n", VxRef);
  //////////////////////////////////
  ///    POUR DEV
    //return ETHER;
 //////////////////////////////////

  if (ABS(VxRef) < 0.01) { VxRef = 0.0;}
  if (ABS(VyRef) < 0.01) { VyRef = 0.0;}
  if (ABS(VzRef) < 0.01) { VzRef = 0.0;}
  if (ABS(WxRef) < 0.01) { WxRef = 0.0;}
  if (ABS(WyRef) < 0.01) { WyRef = 0.0;}
  if (ABS(WzRef) < 0.01) { WzRef = 0.0;}

  //  printf("VxRef %f\n", VxRef);

  //  return EXEC;

//  /* Emergency Stop */
//  if (SDI_F->joystick.buttons[4]) {
//    SDI_F->xarmTrackSegment =  XARM_TRACK_OFF;
//    SDI_F->mode = XARM_STOP;
//    SDI_F->consignType = XARM_NOT_SELECTED_CONSIGN_TYPE;    
//    return ETHER;
//  }
//   
//  /* Check for user abort (button 4) */  
//  if (SDI_F->joystick.buttons[3]) {
//    return ETHER;
//  }
//
//  if (SDI_F->joystick.buttons[5]) { 
//    //  printf("VelX %f  VelY %f  VelZ %f\n",VxRef, VyRef, VzRef);
//    
//    if (SDI_F->pose.p.x > XARM_X_MAX_JOY && (VxRef > 0) ) {VxRef = 0.0;}
//    if (SDI_F->pose.p.x < XARM_X_MIN_JOY && (VxRef < 0) ) {VxRef = 0.0;}
//    if (SDI_F->pose.p.y > XARM_Y_MAX_JOY && (VyRef > 0) ) {VyRef = 0.0;}
//    if (SDI_F->pose.p.y < XARM_Y_MIN_JOY && (VyRef < 0) ) {VyRef = 0.0;} 
//    if (SDI_F->pose.p.z > XARM_Z_MAX_JOY && (VzRef > 0) ) {VzRef = 0.0;}
//    if (SDI_F->pose.p.z < XARM_Z_MIN_JOY && (VzRef < 0) ) {VzRef = 0.0;}    
//  }
//  else {




  /* Sweet Stop */
  if(SDI_F->mouse3dData.b1 == 1) {
    // SDI_F->SweetStop = 1; 
    return END;
  }

 //  if (SDI_F->pose.p.x > XARM_X_MAX_JOY && (VxRef > 0) ) {VxRef = 0.0;}
 //  if (SDI_F->pose.p.x < XARM_X_MIN_JOY && (VxRef < 0) ) {VxRef = 0.0;}
 //  if (SDI_F->pose.p.y > XARM_Y_MAX_JOY && (VyRef > 0) ) {VyRef = 0.0;}
 //  if (SDI_F->pose.p.y < XARM_Y_MIN_JOY && (VyRef < 0) ) {VyRef = 0.0;} 
 //  if (SDI_F->pose.p.z > XARM_Z_MAX_JOY && (VzRef > 0) ) {VzRef = 0.0;}
 //  if (SDI_F->pose.p.z < XARM_Z_MIN_JOY && (VzRef < 0) ) {VzRef = 0.0;}    


  // VjoyRef[cptVjoy] = VxRef;
  // cptVjoy ++;

//   if ((VxRef == 0.0) && (VyRef == 0.0) && (VzRef == 0.0) && (WxRef == 0.0) && (WyRef == 0.0) && (WzRef == 0.0)) {
//
//   }


  /* We do nothinf if xarm is sweet stopping */
  if (SDI_F->SweetStop == 1) {
    return EXEC;
  }

  /* Initialize queues */
  queueInit(&(SDI_F->Seg[0]), "SegX");
  queueInit(&(SDI_F->Seg[1]), "SegY");
  queueInit(&(SDI_F->Seg[2]), "SegZ");
  queueInit(&(SDI_F->Seg[3]), "SegQN");
  queueInit(&(SDI_F->Seg[4]), "SegQI");
  queueInit(&(SDI_F->Seg[5]), "SegQJ");
  queueInit(&(SDI_F->Seg[6]), "SegQK");

  /**************** PX ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.x.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.x.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(VxRef * 1000000))/1000000;
  
  if ( ABS( SDI_F->motionConditions.x.v - FC.v) > JOYSTICK_SENSIBILITY) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.linear, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[0]), &Dir[0])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.x.x;
    GD =  FC.x * Dir[0];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC, Dir[0], AxisTimes[0], &FC, &Acc[0], &Vel[0], &Pos[0], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times PX");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[0]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[0]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[0]);
    queueAddDetails(&(SDI_F->Seg[0]), 2, TNE.Taca, Jerks.J1, Acc[0].Tjpa, Vel[0].Tjpa, Pos[0].Tjpa,  Dir[0]);
    queueAddDetails(&(SDI_F->Seg[0]), 3, TNE.Tjna, Jerks.J1, Acc[0].Taca, Vel[0].Taca, Pos[0].Taca,  Dir[0]);

    queueAddDetails(&(SDI_F->Seg[0]), 4, TNE.Tvc,  Jerks.J1, Acc[0].Tjna, Vel[0].Tjna, Pos[0].Tjna,  Dir[0]);
    //queueAddDetails(&(SDI_F->Seg[0]), 4, 1.0,  Jerks.J1, Acc[0].Tjna, Vel[0].Tjna, Pos[0].Tjna,  Dir[0]);

    queueAddDetails(&(SDI_F->Seg[0]), 5, TNE.Tjnb, Jerks.J1, Acc[0].Tvc,  Vel[0].Tvc,  Pos[0].Tvc,   Dir[0]);
    queueAddDetails(&(SDI_F->Seg[0]), 6, TNE.Tacb, Jerks.J1, Acc[0].Tjnb, Vel[0].Tjnb, Pos[0].Tjnb,  Dir[0]);
    queueAddDetails(&(SDI_F->Seg[0]), 7, TNE.Tjpb, Jerks.J1, Acc[0].Tacb, Vel[0].Tacb, Pos[0].Tacb,  Dir[0]);

    queueAddDetails(&(SDI_F->Seg[0]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.x.x;
    queueAddDetails(&(SDI_F->Seg[0]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
  }

  /**************** PY ****************/    
  /* Initial Conditions */                                     
  IC.a =  (double)((int)(SDI_F->motionConditions.y.a * 1000000))/1000000; 
  IC.v =  (double)((int)(SDI_F->motionConditions.y.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(VyRef * 1000000))/1000000;

  if ( ABS( SDI_F->motionConditions.y.v - FC.v) > JOYSTICK_SENSIBILITY) {

    auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
    //   V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.linear, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
    FC.x = criticalLength;     
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[1]), &Dir[1])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.y.x;
    GD =  FC.x* Dir[1];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC,  Dir[1], AxisTimes[1], &FC, &Acc[1], &Vel[1], &Pos[1], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times PY");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[1]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);

    queueAddDetails(&(SDI_F->Seg[1]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[1]);
    queueAddDetails(&(SDI_F->Seg[1]), 2, TNE.Taca, Jerks.J1, Acc[1].Tjpa, Vel[1].Tjpa, Pos[1].Tjpa,  Dir[1]);
    queueAddDetails(&(SDI_F->Seg[1]), 3, TNE.Tjna, Jerks.J1, Acc[1].Taca, Vel[1].Taca, Pos[1].Taca,  Dir[1]);

    queueAddDetails(&(SDI_F->Seg[1]), 4, TNE.Tvc,  Jerks.J1, Acc[1].Tjna, Vel[1].Tjna, Pos[1].Tjna,  Dir[1]);
    // queueAddDetails(&(SDI_F->Seg[1]), 4, 1.0,  Jerks.J1, Acc[1].Tjna, Vel[1].Tjna, Pos[1].Tjna,  Dir[1]);

    queueAddDetails(&(SDI_F->Seg[1]), 5, TNE.Tjnb, Jerks.J1, Acc[1].Tvc,  Vel[1].Tvc,  Pos[1].Tvc,   Dir[1]);
    queueAddDetails(&(SDI_F->Seg[1]), 6, TNE.Tacb, Jerks.J1, Acc[1].Tjnb, Vel[1].Tjnb, Pos[1].Tjnb,  Dir[1]);
    queueAddDetails(&(SDI_F->Seg[1]), 7, TNE.Tjpb, Jerks.J1, Acc[1].Tacb, Vel[1].Tacb, Pos[1].Tacb,  Dir[1]);

    queueAddDetails(&(SDI_F->Seg[1]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
      
  } else {
    IC.x = SDI_F->motionConditions.y.x;
    queueAddDetails(&(SDI_F->Seg[1]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
  }
  /**************** PZ ****************/
  /* Initial Conditions */                                     
  IC.a =  (double)((int)(SDI_F->motionConditions.z.a * 1000000))/1000000; 
  IC.v =  (double)((int)(SDI_F->motionConditions.z.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v = (double)((int)(VzRef * 1000000))/1000000;

  if ( ABS( SDI_F->motionConditions.z.v - FC.v) > JOYSTICK_SENSIBILITY) {

    auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
    //   V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.linear, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
    FC.x = criticalLength;   
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[2]), &Dir[2])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.z.x;
    GD =  FC.x* Dir[2];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC,  Dir[2], AxisTimes[2], &FC, &Acc[2], &Vel[2], &Pos[2], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times PZ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[2]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);

    queueAddDetails(&(SDI_F->Seg[2]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[2]);
    queueAddDetails(&(SDI_F->Seg[2]), 2, TNE.Taca, Jerks.J1, Acc[2].Tjpa, Vel[2].Tjpa, Pos[2].Tjpa,  Dir[2]);
    queueAddDetails(&(SDI_F->Seg[2]), 3, TNE.Tjna, Jerks.J1, Acc[2].Taca, Vel[2].Taca, Pos[2].Taca,  Dir[2]);

    queueAddDetails(&(SDI_F->Seg[2]), 4, TNE.Tvc,  Jerks.J1, Acc[2].Tjna, Vel[2].Tjna, Pos[2].Tjna,  Dir[2]);
    //queueAddDetails(&(SDI_F->Seg[2]), 4, 1.0,  Jerks.J1, Acc[2].Tjna, Vel[2].Tjna, Pos[2].Tjna,  Dir[2]);

    queueAddDetails(&(SDI_F->Seg[2]), 5, TNE.Tjnb, Jerks.J1, Acc[2].Tvc,  Vel[2].Tvc,  Pos[2].Tvc,   Dir[2]);
    queueAddDetails(&(SDI_F->Seg[2]), 6, TNE.Tacb, Jerks.J1, Acc[2].Tjnb, Vel[2].Tjnb, Pos[2].Tjnb,  Dir[2]);
    queueAddDetails(&(SDI_F->Seg[2]), 7, TNE.Tjpb, Jerks.J1, Acc[2].Tacb, Vel[2].Tacb, Pos[2].Tacb,  Dir[2]);

    queueAddDetails(&(SDI_F->Seg[2]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
      
  } else {
    IC.x = SDI_F->motionConditions.z.x;
    queueAddDetails(&(SDI_F->Seg[2]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
  }

  TP_QUAT quat, dquat;

  quat.n = SDI_F->motionConditions.n.x;
  quat.i = SDI_F->motionConditions.i.x;
  quat.j = SDI_F->motionConditions.j.x;
  quat.k = SDI_F->motionConditions.k.x;

  calculOfQuaternionDerivative(WxRef, WyRef, WzRef, &quat, &dquat);

  // printf("dquat       n %f i %f j %f k %f\n",dquat.n,dquat.i,dquat.j,dquat.k);
  // printf("dquatMotion n %f i %f j %f k %f\n",SDI_F->motionConditions.n.v, SDI_F->motionConditions.i.v, SDI_F->motionConditions.j.v, SDI_F->motionConditions.k.v);
  
  
  // quaternionNormalize(&dquat);
  
 
  Jerks.J1  = SDI_F->limitsGoto.angular.maxJerk; 



  /**************** QN ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.n.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.n.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(dquat.n * 1000000))/1000000;
  
  if ( ABS(SDI_F->motionConditions.n.a) > 0.001 || ABS( SDI_F->motionConditions.n.v - FC.v) > JOYSTICK_SENSIBILITY_ROT) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.angular, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[3]), &Dir[3])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.n.x;
    GD =  FC.x * Dir[3];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[3], AxisTimes[3], &FC, &Acc[3], &Vel[3], &Pos[3], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times QN");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[3]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[3]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[3]);
    queueAddDetails(&(SDI_F->Seg[3]), 2, TNE.Taca, Jerks.J1, Acc[3].Tjpa, Vel[3].Tjpa, Pos[3].Tjpa,  Dir[3]);
    queueAddDetails(&(SDI_F->Seg[3]), 3, TNE.Tjna, Jerks.J1, Acc[3].Taca, Vel[3].Taca, Pos[3].Taca,  Dir[3]);

    queueAddDetails(&(SDI_F->Seg[3]), 4, TNE.Tvc,  Jerks.J1, Acc[3].Tjna, Vel[3].Tjna, Pos[3].Tjna,  Dir[3]);
    //queueAddDetails(&(SDI_F->Seg[3]), 4, 1.0,  Jerks.J1, Acc[3].Tjna, Vel[3].Tjna, Pos[3].Tjna,  Dir[3]);

    queueAddDetails(&(SDI_F->Seg[3]), 5, TNE.Tjnb, Jerks.J1, Acc[3].Tvc,  Vel[3].Tvc,  Pos[3].Tvc,   Dir[3]);
    queueAddDetails(&(SDI_F->Seg[3]), 6, TNE.Tacb, Jerks.J1, Acc[3].Tjnb, Vel[3].Tjnb, Pos[3].Tjnb,  Dir[3]);
    queueAddDetails(&(SDI_F->Seg[3]), 7, TNE.Tjpb, Jerks.J1, Acc[3].Tacb, Vel[3].Tacb, Pos[3].Tacb,  Dir[3]);

    queueAddDetails(&(SDI_F->Seg[3]), 4,  2,  Jerks.J1, FC.a, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.n.x;
    queueAddDetails(&(SDI_F->Seg[3]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
    //  queueAddDetails(&(SDI_F->Seg[3]), 4, NB_TICK,  Jerks.J1, 0, 0, SDI_F->motionConditions.n.x, 1);
  }

  /**************** QI ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.i.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.i.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(dquat.i * 1000000))/1000000;
  
  if ( ABS(SDI_F->motionConditions.i.a) > 0.001 || ABS( SDI_F->motionConditions.i.v - FC.v) > JOYSTICK_SENSIBILITY_ROT) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.angular, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[4]), &Dir[4])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.i.x;
    GD =  FC.x * Dir[4];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[4], AxisTimes[4], &FC, &Acc[4], &Vel[4], &Pos[4], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times QI");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[4]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[4]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[4]);
    queueAddDetails(&(SDI_F->Seg[4]), 2, TNE.Taca, Jerks.J1, Acc[4].Tjpa, Vel[4].Tjpa, Pos[4].Tjpa,  Dir[4]);
    queueAddDetails(&(SDI_F->Seg[4]), 3, TNE.Tjna, Jerks.J1, Acc[4].Taca, Vel[4].Taca, Pos[4].Taca,  Dir[4]);

    queueAddDetails(&(SDI_F->Seg[4]), 4, TNE.Tvc,  Jerks.J1, Acc[4].Tjna, Vel[4].Tjna, Pos[4].Tjna,  Dir[4]);
    //queueAddDetails(&(SDI_F->Seg[4]), 4, 1.0,  Jerks.J1, Acc[4].Tjna, Vel[4].Tjna, Pos[4].Tjna,  Dir[4]);

    queueAddDetails(&(SDI_F->Seg[4]), 5, TNE.Tjnb, Jerks.J1, Acc[4].Tvc,  Vel[4].Tvc,  Pos[4].Tvc,   Dir[4]);
    queueAddDetails(&(SDI_F->Seg[4]), 6, TNE.Tacb, Jerks.J1, Acc[4].Tjnb, Vel[4].Tjnb, Pos[4].Tjnb,  Dir[4]);
    queueAddDetails(&(SDI_F->Seg[4]), 7, TNE.Tjpb, Jerks.J1, Acc[4].Tacb, Vel[4].Tacb, Pos[4].Tacb,  Dir[4]);

    queueAddDetails(&(SDI_F->Seg[4]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.i.x;
    queueAddDetails(&(SDI_F->Seg[4]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
    //  queueAddDetails(&(SDI_F->Seg[4]), 4, NB_TICK,  Jerks.J1, 0, 0, SDI_F->motionConditions.i.x, 1);
  }

  /**************** QJ ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.j.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.j.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(dquat.j * 1000000))/1000000;
  
  if ( ABS(SDI_F->motionConditions.j.a) > 0.001 || ABS( SDI_F->motionConditions.j.v - FC.v) > JOYSTICK_SENSIBILITY_ROT) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.angular, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[5]), &Dir[5])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.j.x;
    GD =  FC.x * Dir[5];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[5], AxisTimes[5], &FC, &Acc[5], &Vel[5], &Pos[5], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times QJ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[5]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[5]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[5]);
    queueAddDetails(&(SDI_F->Seg[5]), 2, TNE.Taca, Jerks.J1, Acc[5].Tjpa, Vel[5].Tjpa, Pos[5].Tjpa,  Dir[5]);
    queueAddDetails(&(SDI_F->Seg[5]), 3, TNE.Tjna, Jerks.J1, Acc[5].Taca, Vel[5].Taca, Pos[5].Taca,  Dir[5]);

    queueAddDetails(&(SDI_F->Seg[5]), 4, TNE.Tvc,  Jerks.J1, Acc[5].Tjna, Vel[5].Tjna, Pos[5].Tjna,  Dir[5]);
    //queueAddDetails(&(SDI_F->Seg[5]), 4, 1.0,  Jerks.J1, Acc[5].Tjna, Vel[5].Tjna, Pos[5].Tjna,  Dir[5]);

    queueAddDetails(&(SDI_F->Seg[5]), 5, TNE.Tjnb, Jerks.J1, Acc[5].Tvc,  Vel[5].Tvc,  Pos[5].Tvc,   Dir[5]);
    queueAddDetails(&(SDI_F->Seg[5]), 6, TNE.Tacb, Jerks.J1, Acc[5].Tjnb, Vel[5].Tjnb, Pos[5].Tjnb,  Dir[5]);
    queueAddDetails(&(SDI_F->Seg[5]), 7, TNE.Tjpb, Jerks.J1, Acc[5].Tacb, Vel[5].Tacb, Pos[5].Tacb,  Dir[5]);

    queueAddDetails(&(SDI_F->Seg[5]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.j.x;
    queueAddDetails(&(SDI_F->Seg[5]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
    //  queueAddDetails(&(SDI_F->Seg[5]), 4, NB_TICK,  Jerks.J1, 0, 0, SDI_F->motionConditions.j.x, 1);
  }

  /**************** QK ****************/  
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.k.a * 1000000))/1000000; 
  IC.v = (double)((int)(SDI_F->motionConditions.k.v * 1000000))/1000000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v =  (double)((int)(dquat.k * 1000000))/1000000;
  
  if ( ABS(SDI_F->motionConditions.k.a) > 0.001 || ABS( SDI_F->motionConditions.k.v - FC.v) > JOYSTICK_SENSIBILITY_ROT) {
    
    auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
    auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
    auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
    //  V0 = ABS(IC.v) + (1/(2*auxLimits.maxJerk))*(IC.a*IC.a); 
    //   auxLimits.maxVel = MAX(V0, ABS(FC.v));
    Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
    Jerks.sel = 1;

    if(xarmCalculOfCriticalLength( IC, FC, SDI_F->limitsGoto.angular, &criticalLength)!= OK) {
      PRINT_ERROR("ERROR Calcul Of Critical Length\n");
      return ETHER;
    }
 //   printf("Vx %f      Critical Length %f \n",IC.v,criticalLength);
    FC.x = criticalLength; 

    if (isnan(criticalLength)) {
      printf("IC.a = %f    IC.v = %f    IC.x = %f\n",IC.a,IC.v,IC.x);
      printf("FC.a = %f    FC.v = %f    FC.x = %f\n",FC.a,FC.v,FC.x);

    }
    if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[6]), &Dir[6])!=0) {
      PRINT_ERROR("ERROR Jerk Profile\n");
      return ETHER;
    }  
    /* Get initial conditions for each vectors Acc Vel and Pos */
    IC.x = SDI_F->motionConditions.k.x;
    GD =  FC.x * Dir[6];
    if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[6], AxisTimes[6], &FC, &Acc[6], &Vel[6], &Pos[6], TP_ON)!=OK) {
      PRINT_ERROR(" Verify Times QK");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    /* Get Number of Elements needed for Vectors Definition */
    if (tpGetMonotonicTimes((AxisTimes[6]), &TM, &NOE)!=OK) {
      PRINT_ERROR(" Get MonotonicTimes ");
      *report = S_xarm_ERROR_TP;
      return ETHER;
    }
    tpGetNumberOfElement (&TM, &TNE);
    queueAddDetails(&(SDI_F->Seg[6]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[6]);
    queueAddDetails(&(SDI_F->Seg[6]), 2, TNE.Taca, Jerks.J1, Acc[6].Tjpa, Vel[6].Tjpa, Pos[6].Tjpa,  Dir[6]);
    queueAddDetails(&(SDI_F->Seg[6]), 3, TNE.Tjna, Jerks.J1, Acc[6].Taca, Vel[6].Taca, Pos[6].Taca,  Dir[6]);

    queueAddDetails(&(SDI_F->Seg[6]), 4, TNE.Tvc,  Jerks.J1, Acc[6].Tjna, Vel[6].Tjna, Pos[6].Tjna,  Dir[6]);
    //queueAddDetails(&(SDI_F->Seg[6]), 4, 1.0,  Jerks.J1, Acc[6].Tjna, Vel[6].Tjna, Pos[6].Tjna,  Dir[6]);

    queueAddDetails(&(SDI_F->Seg[6]), 5, TNE.Tjnb, Jerks.J1, Acc[6].Tvc,  Vel[6].Tvc,  Pos[6].Tvc,   Dir[6]);
    queueAddDetails(&(SDI_F->Seg[6]), 6, TNE.Tacb, Jerks.J1, Acc[6].Tjnb, Vel[6].Tjnb, Pos[6].Tjnb,  Dir[6]);
    queueAddDetails(&(SDI_F->Seg[6]), 7, TNE.Tjpb, Jerks.J1, Acc[6].Tacb, Vel[6].Tacb, Pos[6].Tacb,  Dir[6]);

    queueAddDetails(&(SDI_F->Seg[6]), 4,  2,  Jerks.J1, 0, FC.v, FC.x, SIGN(FC.v));
  } 
  else {
    IC.x = SDI_F->motionConditions.k.x;
    queueAddDetails(&(SDI_F->Seg[6]), 4, NB_TICK,  Jerks.J1, 0, IC.v, IC.x, SIGN(FC.v));
    //  queueAddDetails(&(SDI_F->Seg[6]), 4, NB_TICK,  Jerks.J1, 0, 0, SDI_F->motionConditions.k.x, 1);
  }


  SDI_F->xarmTrackSegment = XARM_TRACK_ON;  

  return EXEC;

}


/* xarmMouse3dOnEnd  -  codel END of Mouse3dOn
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmMouse3dOnEnd(int *report)
{
  printf("xarmMouse3D exited by user\n");
  return ETHER;
}


/* xarmMouse3dOnInter  -  codel INTER of Mouse3dOn
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmMouse3dOnInter(int *report)
{
  /* ... add your code here ... */
  return ETHER;
}


/*------------------------------------------------------------------------
 * ExternalTrackingFromViman
 *
 * Description: 
 *
 * Reports:      OK
 *              S_xarm_CANNOT_FIND_POSTER
 *              S_xarm_CANNOT_READ_POSTER
 */

/* xarmExternalTrackingFromVimanStart  -  codel START of ExternalTrackingFromViman
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmExternalTrackingFromVimanStart(XARM_STRING *posPosterName, int *report)
{ 
  double px = 0.0, py = 0.0, pz= 0.0, qn=1.0, qi=0.0, qj=0.0, qk=0.0;
  /* INIT POSTERS */
  if ( xarmGb_objectPosePosterFind(posPosterName->str , &posterViman) == ERROR) {
    PRINT_ERROR("Can not find POSTER VIMAN");
    *report =  S_xarm_CANNOT_FIND_POSTER;
    return ETHER;
  }
  /* Read Poster Data and Status from Viman*/
  if(xarmGb_objectPosePosterRead( posterViman, &(SDI_F->gbObjectPose))!=OK) {
    PRINT_ERROR("Can not read VIMAN_POSTER");
    *report =  S_xarm_CANNOT_READ_POSTER;
    return ETHER;
  }

  if( SDI_F->gbObjectPose.visible == 0) {
    return START;
  }

  getVimanObjectMotion(&px, &py, &pz, &qn, &qi, &qj, &qk) ;

  Jerks.sel = 1;

  cptVjoy = 0;
  printf("XARM EXTERNAL_TRACKING START OK\n");
  return EXEC;
}

/* xarmExternalTrackingFromVimanMain  -  codel EXEC of ExternalTrackingFromViman
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmExternalTrackingFromVimanMain(XARM_STRING *posPosterName, int *report)
{
  double px = 0.0, py = 0.0, pz= 0.0, qn=1.0, qi=0.0, qj=0.0, qk=0.0;
  double criticalLength = 0.0;

  /* We do nothinf if xarm is sweet stopping */
  if (SDI_F->SweetStop == 1) {
    return EXEC;
  }

  /* Read Poster Data and Status from Viman*/
  if(xarmGb_objectPosePosterRead( posterViman, &(SDI_F->gbObjectPose))!=OK) {
    PRINT_ERROR("Can not read VIMAN_POSTER");
    *report =  S_xarm_CANNOT_READ_POSTER;
    SDI_F->SweetStop = 1;
    return ETHER;
  }

  getVimanObjectMotion(&px, &py, &pz, &qn, &qi, &qj, &qk) ;

//  qn = 1.0;
//  qi = 0.0;
//  qj = 0.0;
//  qk = 0.0;


//  if (SDI_F->pose.p.x > XARM_X_MAX_JOY && (SDI_F->motionConditions.x.v  > 0) ) {px = vimanObjectPoseOld.th.px;}
//  if (SDI_F->pose.p.x < XARM_X_MIN_JOY && (SDI_F->motionConditions.x.v  < 0) ) {px = vimanObjectPoseOld.th.px;}
//  if (SDI_F->pose.p.y > XARM_Y_MAX_JOY && (SDI_F->motionConditions.y.v  > 0) ) {px = vimanObjectPoseOld.th.py;}
//  if (SDI_F->pose.p.y < XARM_Y_MIN_JOY && (SDI_F->motionConditions.y.v  < 0) ) {px = vimanObjectPoseOld.th.py;} 
//  if (SDI_F->pose.p.z > XARM_Z_MAX_JOY && (SDI_F->motionConditions.z.v  > 0) ) {px = vimanObjectPoseOld.th.pz;}
//  if (SDI_F->pose.p.z < XARM_Z_MIN_JOY && (SDI_F->motionConditions.z.v  < 0) ) {px = vimanObjectPoseOld.th.pz;}    
  

  //  printf("xarm track object : PX=%f PY=%f PZ=%f \n",px, py, pz);

 /* Initialize queues */
  queueInit(&(SDI_F->Seg[0]), "SegX");
  queueInit(&(SDI_F->Seg[1]), "SegY");
  queueInit(&(SDI_F->Seg[2]), "SegZ");
  queueInit(&(SDI_F->Seg[3]), "SegQN");
  queueInit(&(SDI_F->Seg[4]), "SegQI");
  queueInit(&(SDI_F->Seg[5]), "SegQJ");
  queueInit(&(SDI_F->Seg[6]), "SegQK");

  /**************** PX ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.x.a * 10000))/10000; 
  IC.v = (double)((int)(SDI_F->motionConditions.x.v * 10000))/10000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v = 0.0;
  FC.x = ((double)((int)(px * 10000))/10000) -  ((double)((int)(SDI_F->pose.p.x * 10000))/10000);



  auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
  auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
  auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
  Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
  Jerks.sel = 1;

  if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[0]), &Dir[0])!=0) {
    printf("IC.x     %f \n",IC.x);
    printf("FC.x     %f \n",FC.x);
    printf("motion.x %f \n",SDI_F->pose.p.x);
    printf("px       %f \n",px);
    printf("IC.x %f \n",IC.x);

    PRINT_ERROR("ERROR Jerk Profile\n");
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
  }  
  /* Get initial conditions for each vectors Acc Vel and Pos */
  IC.x = (double)((int)(SDI_F->pose.p.x * 10000))/10000; 
  GD =   FC.x   * Dir[0];
  if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC, Dir[0], AxisTimes[0], &FC, &Acc[0], &Vel[0], &Pos[0], TP_ON)!=OK) {
    PRINT_ERROR(" Verify Times PX");
    *report = S_xarm_ERROR_TP;
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
  }
  /* Get Number of Elements needed for Vectors Definition */
  if (tpGetMonotonicTimes((AxisTimes[0]), &TM, &NOE)!=OK) {
    PRINT_ERROR(" Get MonotonicTimes ");
    *report = S_xarm_ERROR_TP;
    return ETHER;
  }
  tpGetNumberOfElement (&TM, &TNE);
  queueAddDetails(&(SDI_F->Seg[0]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[0]);
  queueAddDetails(&(SDI_F->Seg[0]), 2, TNE.Taca, Jerks.J1, Acc[0].Tjpa, Vel[0].Tjpa, Pos[0].Tjpa,  Dir[0]);
  queueAddDetails(&(SDI_F->Seg[0]), 3, TNE.Tjna, Jerks.J1, Acc[0].Taca, Vel[0].Taca, Pos[0].Taca,  Dir[0]);
  
  queueAddDetails(&(SDI_F->Seg[0]), 4, TNE.Tvc,  Jerks.J1, Acc[0].Tjna, Vel[0].Tjna, Pos[0].Tjna,  Dir[0]);
 
  queueAddDetails(&(SDI_F->Seg[0]), 5, TNE.Tjnb, Jerks.J1, Acc[0].Tvc,  Vel[0].Tvc,  Pos[0].Tvc,   Dir[0]);
  queueAddDetails(&(SDI_F->Seg[0]), 6, TNE.Tacb, Jerks.J1, Acc[0].Tjnb, Vel[0].Tjnb, Pos[0].Tjnb,  Dir[0]);
  queueAddDetails(&(SDI_F->Seg[0]), 7, TNE.Tjpb, Jerks.J1, Acc[0].Tacb, Vel[0].Tacb, Pos[0].Tacb,  Dir[0]);

  
  IC.x =  ((double)((int)(px * 10000))/10000) ;
  queueAddDetails(&(SDI_F->Seg[0]), 4, NB_TICK,  Jerks.J1, 0, 0.0, IC.x, 1);
  

  
  /**************** PY ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.y.a * 10000))/10000; 
  IC.v = (double)((int)(SDI_F->motionConditions.y.v * 10000))/10000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v = 0.0;
  FC.x = ((double)((int)(py * 10000))/10000) -  ((double)((int)(SDI_F->pose.p.y   * 10000))/10000);



  auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
  auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
  auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
  Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
  Jerks.sel = 1;

  if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[1]), &Dir[1])!=0) {
    printf("IC.x     %f \n",IC.x);
    printf("FC.x     %f \n",FC.x);
    printf("motion.x %f \n",SDI_F->pose.p.y );
    printf("py       %f \n",py);
    printf("IC.x %f \n",IC.x);
    PRINT_ERROR("ERROR Jerk Profile\n");
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
   }  
  /* Get initial conditions for each vectors Acc Vel and Pos */
  IC.x =  (double)((int)(SDI_F->pose.p.y  * 10000))/10000 ;
  GD =   FC.x * Dir[1];
  if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC, Dir[1], AxisTimes[1], &FC, &Acc[1], &Vel[1], &Pos[1], TP_ON)!=OK) {
    PRINT_ERROR(" Verify Times PY");
    *report = S_xarm_ERROR_TP;
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
    }
  /* Get Number of Elements needed for Vectors Definition */
  if (tpGetMonotonicTimes((AxisTimes[1]), &TM, &NOE)!=OK) {
    PRINT_ERROR(" Get MonotonicTimes ");
    *report = S_xarm_ERROR_TP;
    return ETHER;
  }
  tpGetNumberOfElement (&TM, &TNE);
  queueAddDetails(&(SDI_F->Seg[1]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[1]);
  queueAddDetails(&(SDI_F->Seg[1]), 2, TNE.Taca, Jerks.J1, Acc[1].Tjpa, Vel[1].Tjpa, Pos[1].Tjpa,  Dir[1]);
  queueAddDetails(&(SDI_F->Seg[1]), 3, TNE.Tjna, Jerks.J1, Acc[1].Taca, Vel[1].Taca, Pos[1].Taca,  Dir[1]);
  
  queueAddDetails(&(SDI_F->Seg[1]), 4, TNE.Tvc,  Jerks.J1, Acc[1].Tjna, Vel[1].Tjna, Pos[1].Tjna,  Dir[1]);
 
  queueAddDetails(&(SDI_F->Seg[1]), 5, TNE.Tjnb, Jerks.J1, Acc[1].Tvc,  Vel[1].Tvc,  Pos[1].Tvc,   Dir[1]);
  queueAddDetails(&(SDI_F->Seg[1]), 6, TNE.Tacb, Jerks.J1, Acc[1].Tjnb, Vel[1].Tjnb, Pos[1].Tjnb,  Dir[1]);
  queueAddDetails(&(SDI_F->Seg[1]), 7, TNE.Tjpb, Jerks.J1, Acc[1].Tacb, Vel[1].Tacb, Pos[1].Tacb,  Dir[1]);

  IC.x =  ((double)((int)(py * 10000))/10000) ;
  queueAddDetails(&(SDI_F->Seg[1]), 4, NB_TICK,  Jerks.J1, 0, 0.0, IC.x, 1);


  
  /**************** PZ ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.z.a * 10000))/10000; 
  IC.v = (double)((int)(SDI_F->motionConditions.z.v * 10000))/10000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v = 0.0;
  FC.x = ((double)((int)(pz * 10000))/10000) -  ((double)((int)(SDI_F->pose.p.z  * 10000))/10000);


  auxLimits.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
  auxLimits.maxAcc = SDI_F->limitsGoto.linear.maxAcc;
  auxLimits.maxVel = SDI_F->limitsGoto.linear.maxVel;
  Jerks.J1 = SDI_F->limitsGoto.linear.maxJerk; 
  Jerks.sel = 1;

  if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[2]), &Dir[2])!=0) {
    printf("IC.x     %f \n",IC.x);
    printf("FC.x     %f \n",FC.x);
    printf("motion.x %f \n",SDI_F->pose.p.z );
    printf("pz       %f \n",pz);
    printf("IC.x %f \n",IC.x);

    PRINT_ERROR("ERROR Jerk Profile\n");
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
    }  
  /* Get initial conditions for each vectors Acc Vel and Pos */
  IC.x =  (double)((int)(SDI_F->pose.p.z  * 10000))/10000; 
  GD = FC.x * Dir[2];

  if (tpVerifyTimes(DISTANCE_TOLERANCE_LINEAR, GD, Jerks, IC, Dir[2], AxisTimes[2], &FC, &Acc[2], &Vel[2], &Pos[2], TP_ON)!=OK) {
    PRINT_ERROR(" Verify Times PZ");
    *report = S_xarm_ERROR_TP;
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
  }
  /* Get Number of Elements needed for Vectors Definition */
  if (tpGetMonotonicTimes((AxisTimes[2]), &TM, &NOE)!=OK) {
    PRINT_ERROR(" Get MonotonicTimes ");
    *report = S_xarm_ERROR_TP;
    return ETHER;
  }
  tpGetNumberOfElement (&TM, &TNE);
  queueAddDetails(&(SDI_F->Seg[2]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[2]);
  queueAddDetails(&(SDI_F->Seg[2]), 2, TNE.Taca, Jerks.J1, Acc[2].Tjpa, Vel[2].Tjpa, Pos[2].Tjpa,  Dir[2]);
  queueAddDetails(&(SDI_F->Seg[2]), 3, TNE.Tjna, Jerks.J1, Acc[2].Taca, Vel[2].Taca, Pos[2].Taca,  Dir[2]);
  
  queueAddDetails(&(SDI_F->Seg[2]), 4, TNE.Tvc,  Jerks.J1, Acc[2].Tjna, Vel[2].Tjna, Pos[2].Tjna,  Dir[2]);
 
  queueAddDetails(&(SDI_F->Seg[2]), 5, TNE.Tjnb, Jerks.J1, Acc[2].Tvc,  Vel[2].Tvc,  Pos[2].Tvc,   Dir[2]);
  queueAddDetails(&(SDI_F->Seg[2]), 6, TNE.Tacb, Jerks.J1, Acc[2].Tjnb, Vel[2].Tjnb, Pos[2].Tjnb,  Dir[2]);
  queueAddDetails(&(SDI_F->Seg[2]), 7, TNE.Tjpb, Jerks.J1, Acc[2].Tacb, Vel[2].Tacb, Pos[2].Tacb,  Dir[2]);


  IC.x =  ((double)((int)(pz * 10000))/10000) ;
  queueAddDetails(&(SDI_F->Seg[2]), 4, NB_TICK,  Jerks.J1, 0, 0.0, IC.x, 1);


   Jerks.J1  = SDI_F->limitsGoto.angular.maxJerk; 

  /**************** QN ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.n.a * 10000))/10000; 
  IC.v = (double)((int)(SDI_F->motionConditions.n.v * 10000))/10000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v = 0.0;
  FC.x = ((double)((int)(qn * 10000))/10000) -  ((double)((int)(SDI_F->motionConditions.n.x   * 10000))/10000);


  auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
  auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
  auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
  Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
  Jerks.sel = 1;

  if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[3]), &Dir[3])!=0) {
    printf("IC.x     %f \n",IC.x);
    printf("FC.x     %f \n",FC.x);
    printf("motion.x %f \n",SDI_F->motionConditions.n.x  );
    printf("qn       %f \n",qn);
    printf("IC.x %f \n",IC.x);

    PRINT_ERROR("ERROR Jerk Profile\n");
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
    }  
  /* Get initial conditions for each vectors Acc Vel and Pos */
  IC.x =  (double)((int)(SDI_F->motionConditions.n.x   * 10000))/10000; 
  GD = FC.x * Dir[3];

  if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[3], AxisTimes[3], &FC, &Acc[3], &Vel[3], &Pos[3], TP_ON)!=OK) {
    PRINT_ERROR(" Verify Times QN");
    *report = S_xarm_ERROR_TP;
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
  }
  /* Get Number of Elements needed for Vectors Definition */
  if (tpGetMonotonicTimes((AxisTimes[3]), &TM, &NOE)!=OK) {
    PRINT_ERROR(" Get MonotonicTimes ");
    *report = S_xarm_ERROR_TP;
    return ETHER;
  }
  tpGetNumberOfElement (&TM, &TNE);
  queueAddDetails(&(SDI_F->Seg[3]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[3]);
  queueAddDetails(&(SDI_F->Seg[3]), 2, TNE.Taca, Jerks.J1, Acc[3].Tjpa, Vel[3].Tjpa, Pos[3].Tjpa,  Dir[3]);
  queueAddDetails(&(SDI_F->Seg[3]), 3, TNE.Tjna, Jerks.J1, Acc[3].Taca, Vel[3].Taca, Pos[3].Taca,  Dir[3]);
  
  queueAddDetails(&(SDI_F->Seg[3]), 4, TNE.Tvc,  Jerks.J1, Acc[3].Tjna, Vel[3].Tjna, Pos[3].Tjna,  Dir[3]);
 
  queueAddDetails(&(SDI_F->Seg[3]), 5, TNE.Tjnb, Jerks.J1, Acc[3].Tvc,  Vel[3].Tvc,  Pos[3].Tvc,   Dir[3]);
  queueAddDetails(&(SDI_F->Seg[3]), 6, TNE.Tacb, Jerks.J1, Acc[3].Tjnb, Vel[3].Tjnb, Pos[3].Tjnb,  Dir[3]);
  queueAddDetails(&(SDI_F->Seg[3]), 7, TNE.Tjpb, Jerks.J1, Acc[3].Tacb, Vel[3].Tacb, Pos[3].Tacb,  Dir[3]);


  IC.x =  ((double)((int)(qn * 10000))/10000) ;
  queueAddDetails(&(SDI_F->Seg[3]), 4, NB_TICK,  Jerks.J1, 0, 0.0, IC.x, 1);





  /**************** QI ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.i.a * 10000))/10000; 
  IC.v = (double)((int)(SDI_F->motionConditions.i.v * 10000))/10000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v = 0.0;
  FC.x = ((double)((int)(qi * 10000))/10000) -  ((double)((int)(SDI_F->motionConditions.i.x   * 10000))/10000);


  auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
  auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
  auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
  Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
  Jerks.sel = 1;

  if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[4]), &Dir[4])!=0) {
    printf("IC.x     %f \n",IC.x);
    printf("FC.x     %f \n",FC.x);
    printf("motion.x %f \n",SDI_F->motionConditions.i.x  );
    printf("qn       %f \n",qi);
    printf("IC.x %f \n",IC.x);

    PRINT_ERROR("ERROR Jerk Profile\n");
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
    }  
  /* Get initial conditions for each vectors Acc Vel and Pos */
  IC.x =  (double)((int)(SDI_F->motionConditions.i.x   * 10000))/10000; 
  GD = FC.x * Dir[4];

  if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[4], AxisTimes[4], &FC, &Acc[4], &Vel[4], &Pos[4], TP_ON)!=OK) {
    PRINT_ERROR(" Verify Times QN");
    *report = S_xarm_ERROR_TP;
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
  }
  /* Get Number of Elements needed for Vectors Definition */
  if (tpGetMonotonicTimes((AxisTimes[4]), &TM, &NOE)!=OK) {
    PRINT_ERROR(" Get MonotonicTimes ");
    *report = S_xarm_ERROR_TP;
    return ETHER;
  }
  tpGetNumberOfElement (&TM, &TNE);
  queueAddDetails(&(SDI_F->Seg[4]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[4]);
  queueAddDetails(&(SDI_F->Seg[4]), 2, TNE.Taca, Jerks.J1, Acc[4].Tjpa, Vel[4].Tjpa, Pos[4].Tjpa,  Dir[4]);
  queueAddDetails(&(SDI_F->Seg[4]), 3, TNE.Tjna, Jerks.J1, Acc[4].Taca, Vel[4].Taca, Pos[4].Taca,  Dir[4]);
  
  queueAddDetails(&(SDI_F->Seg[4]), 4, TNE.Tvc,  Jerks.J1, Acc[4].Tjna, Vel[4].Tjna, Pos[4].Tjna,  Dir[4]);
 
  queueAddDetails(&(SDI_F->Seg[4]), 5, TNE.Tjnb, Jerks.J1, Acc[4].Tvc,  Vel[4].Tvc,  Pos[4].Tvc,   Dir[4]);
  queueAddDetails(&(SDI_F->Seg[4]), 6, TNE.Tacb, Jerks.J1, Acc[4].Tjnb, Vel[4].Tjnb, Pos[4].Tjnb,  Dir[4]);
  queueAddDetails(&(SDI_F->Seg[4]), 7, TNE.Tjpb, Jerks.J1, Acc[4].Tacb, Vel[4].Tacb, Pos[4].Tacb,  Dir[4]);


  IC.x =  ((double)((int)(qi * 10000))/10000) ;
  queueAddDetails(&(SDI_F->Seg[4]), 4, NB_TICK,  Jerks.J1, 0, 0.0, IC.x, 1);



  /**************** QJ ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.j.a * 10000))/10000; 
  IC.v = (double)((int)(SDI_F->motionConditions.j.v * 10000))/10000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v = 0.0;
  FC.x = ((double)((int)(qj * 10000))/10000) -  ((double)((int)(SDI_F->motionConditions.j.x   * 10000))/10000);


  auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
  auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
  auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
  Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
  Jerks.sel = 1;

  if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[5]), &Dir[5])!=0) {
    printf("IC.x     %f \n",IC.x);
    printf("FC.x     %f \n",FC.x);
    printf("motion.x %f \n",SDI_F->motionConditions.j.x  );
    printf("qn       %f \n",qj);
    printf("IC.x %f \n",IC.x);

    PRINT_ERROR("ERROR Jerk Profile\n");
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
    }  
  /* Get initial conditions for each vectors Acc Vel and Pos */
  IC.x =  (double)((int)(SDI_F->motionConditions.j.x   * 10000))/10000; 
  GD = FC.x * Dir[5];

  if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[5], AxisTimes[5], &FC, &Acc[5], &Vel[5], &Pos[5], TP_ON)!=OK) {
    PRINT_ERROR(" Verify Times QN");
    *report = S_xarm_ERROR_TP;
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
  }
  /* Get Number of Elements needed for Vectors Definition */
  if (tpGetMonotonicTimes((AxisTimes[5]), &TM, &NOE)!=OK) {
    PRINT_ERROR(" Get MonotonicTimes ");
    *report = S_xarm_ERROR_TP;
    return ETHER;
  }
  tpGetNumberOfElement (&TM, &TNE);
  queueAddDetails(&(SDI_F->Seg[5]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[5]);
  queueAddDetails(&(SDI_F->Seg[5]), 2, TNE.Taca, Jerks.J1, Acc[5].Tjpa, Vel[5].Tjpa, Pos[5].Tjpa,  Dir[5]);
  queueAddDetails(&(SDI_F->Seg[5]), 3, TNE.Tjna, Jerks.J1, Acc[5].Taca, Vel[5].Taca, Pos[5].Taca,  Dir[5]);
  
  queueAddDetails(&(SDI_F->Seg[5]), 4, TNE.Tvc,  Jerks.J1, Acc[5].Tjna, Vel[5].Tjna, Pos[5].Tjna,  Dir[5]);
 
  queueAddDetails(&(SDI_F->Seg[5]), 5, TNE.Tjnb, Jerks.J1, Acc[5].Tvc,  Vel[5].Tvc,  Pos[5].Tvc,   Dir[5]);
  queueAddDetails(&(SDI_F->Seg[5]), 6, TNE.Tacb, Jerks.J1, Acc[5].Tjnb, Vel[5].Tjnb, Pos[5].Tjnb,  Dir[5]);
  queueAddDetails(&(SDI_F->Seg[5]), 7, TNE.Tjpb, Jerks.J1, Acc[5].Tacb, Vel[5].Tacb, Pos[5].Tacb,  Dir[5]);


  IC.x =  ((double)((int)(qj * 10000))/10000) ;
  queueAddDetails(&(SDI_F->Seg[5]), 4, NB_TICK,  Jerks.J1, 0, 0.0, IC.x, 1);


  /**************** QK ****************/
  /* Initial Conditions */                                     
  IC.a = (double)((int)(SDI_F->motionConditions.k.a * 10000))/10000; 
  IC.v = (double)((int)(SDI_F->motionConditions.k.v * 10000))/10000;
  IC.x = 0.0; /* to calcule dc */
  /* Final Conditions */ 
  FC.a = 0.0;
  FC.v = 0.0;
  FC.x = ((double)((int)(qk * 10000))/10000) -  ((double)((int)(SDI_F->motionConditions.k.x   * 10000))/10000);


  auxLimits.maxJerk = SDI_F->limitsGoto.angular.maxJerk;
  auxLimits.maxAcc = SDI_F->limitsGoto.angular.maxAcc;
  auxLimits.maxVel = SDI_F->limitsGoto.angular.maxVel;
  Jerks.J1 = SDI_F->limitsGoto.angular.maxJerk; 
  Jerks.sel = 1;

  if (xarmJerkProfile(IC, FC, auxLimits, &(AxisTimes[6]), &Dir[6])!=0) {
    printf("IC.x     %f \n",IC.x);
    printf("FC.x     %f \n",FC.x);
    printf("motion.x %f \n",SDI_F->motionConditions.k.x  );
    printf("qn       %f \n",qk);
    printf("IC.x %f \n",IC.x);

    PRINT_ERROR("ERROR Jerk Profile\n");
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
    }  
  /* Get initial conditions for each vectors Acc Vel and Pos */
  IC.x =  (double)((int)(SDI_F->motionConditions.k.x   * 10000))/10000; 
  GD = FC.x * Dir[6];

  if (tpVerifyTimes(DISTANCE_TOLERANCE_ANGULAR, GD, Jerks, IC, Dir[6], AxisTimes[6], &FC, &Acc[6], &Vel[6], &Pos[6], TP_ON)!=OK) {
    PRINT_ERROR(" Verify Times QN");
    *report = S_xarm_ERROR_TP;
    SDI_F->SweetStop = 1;
    return EXEC;
    //  return ETHER;
  }
  /* Get Number of Elements needed for Vectors Definition */
  if (tpGetMonotonicTimes((AxisTimes[6]), &TM, &NOE)!=OK) {
    PRINT_ERROR(" Get MonotonicTimes ");
    *report = S_xarm_ERROR_TP;
    return ETHER;
  }
  tpGetNumberOfElement (&TM, &TNE);
  queueAddDetails(&(SDI_F->Seg[6]), 1, TNE.Tjpa, Jerks.J1, IC.a, IC.v,  IC.x, Dir[6]);
  queueAddDetails(&(SDI_F->Seg[6]), 2, TNE.Taca, Jerks.J1, Acc[6].Tjpa, Vel[6].Tjpa, Pos[6].Tjpa,  Dir[6]);
  queueAddDetails(&(SDI_F->Seg[6]), 3, TNE.Tjna, Jerks.J1, Acc[6].Taca, Vel[6].Taca, Pos[6].Taca,  Dir[6]);
  
  queueAddDetails(&(SDI_F->Seg[6]), 4, TNE.Tvc,  Jerks.J1, Acc[6].Tjna, Vel[6].Tjna, Pos[6].Tjna,  Dir[6]);
 
  queueAddDetails(&(SDI_F->Seg[6]), 5, TNE.Tjnb, Jerks.J1, Acc[6].Tvc,  Vel[6].Tvc,  Pos[6].Tvc,   Dir[6]);
  queueAddDetails(&(SDI_F->Seg[6]), 6, TNE.Tacb, Jerks.J1, Acc[6].Tjnb, Vel[6].Tjnb, Pos[6].Tjnb,  Dir[6]);
  queueAddDetails(&(SDI_F->Seg[6]), 7, TNE.Tjpb, Jerks.J1, Acc[6].Tacb, Vel[6].Tacb, Pos[6].Tacb,  Dir[6]);


  IC.x =  ((double)((int)(qk * 10000))/10000) ;
  queueAddDetails(&(SDI_F->Seg[6]), 4, NB_TICK,  Jerks.J1, 0, 0.0, IC.x, 1);




  SDI_F->xarmTrackSegment = XARM_TRACK_ON;  
  return EXEC;
}

/* xarmExternalTrackingFromVimanEnd  -  codel END of ExternalTrackingFromViman
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmExternalTrackingFromVimanEnd(XARM_STRING *posPosterName, int *report)
{
  /* ... add your code here ... */
  return ETHER;
}







